/* apoplexy v1.9b (September 2013)
 * Copyright (C) 2008-2013 Norbert de Jonge <mail@norbertdejonge.nl>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see [ www.gnu.org/licenses/ ].
 *
 * To properly read this code, set your program's tab stop to: 2.
 */

/*========== Includes ==========*/
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <math.h>
#include <time.h>
#include <errno.h>

#include "SDL.h"
#include "SDL_audio.h"
#include "SDL_thread.h"
#include "SDL/SDL_ttf.h"
#include "SDL/SDL_image.h"
#include "SDL/SDL_rotozoom.h"
/*========== Includes ==========*/

/*========== Defines ==========*/
#define EXIT_NORMAL 0
#define EXIT_ERROR 1
#define MAX_DATA 720
#define MAX_TEXT 100
#define ROOMS 24
#define USER_DATA 2000
#define SCREEN_WIDTH 642 + 50
#define SCREEN_HEIGHT 380 + 75
#define SCREEN_BPP 16
#define EDITOR_NAME "apoplexy"
#define EDITOR_VERSION "v1.9b (September 2013)"
#define COPYRIGHT "Copyright (C) 2013 Norbert de Jonge"
#define REFRESH 30 /*** That is 33 frames per second, 1000/30. ***/
#define MAX_FILE 100
#define MAX_ADJ 10
#define MAX_GRID 10
#define FONT_SIZE_1 15
#define FONT_SIZE_2 11
#define FONT_SIZE_3 20
#define MAX_TOWRITE 720
#define IN_DIR "levels"
#define OUT_DIR "levels"
#define XML_DIR "xml"
#define NUM_SOUNDS 20 /*** Sounds that may play at the same time. ***/
#define MAX_OPTION 100
#define MAX_CUSTOM 10
#define MAX_DATETIME 50

#define OFFSET_X 25
#define OFFSET_Y 50
#define TTP 27
#define HOR0 -64 + OFFSET_X
#define HOR1 0 + OFFSET_X
#define HOR2 64 + OFFSET_X
#define HOR3 128 + OFFSET_X
#define HOR4 192 + OFFSET_X
#define HOR5 256 + OFFSET_X
#define HOR6 320 + OFFSET_X
#define HOR7 384 + OFFSET_X
#define HOR8 448 + OFFSET_X
#define HOR9 512 + OFFSET_X
#define HOR10 576 + OFFSET_X
#define VER1 OFFSET_Y - TTP
#define VER2 OFFSET_Y - TTP + 126
#define VER3 OFFSET_Y - TTP + 252
#define VER4 OFFSET_Y - TTP + 378
#define TX 4
#define TY 4
#define TX2 1
#define TY2 1
#define BROKEN_ROOM_X 438
#define BROKEN_ROOM_Y 76
#define BROKEN_LEFT_X 423
#define BROKEN_LEFT_Y 76
#define BROKEN_RIGHT_X 453
#define BROKEN_RIGHT_Y 76
#define BROKEN_UP_X 438
#define BROKEN_UP_Y 61
#define BROKEN_DOWN_X 438
#define BROKEN_DOWN_Y 91
#define ADJ_BASE_X 422
#define ADJ_BASE_Y 60

#ifndef O_BINARY
#define O_BINARY 0
#endif

#if defined WIN32 || _WIN32 || WIN64 || _WIN64
#define DEVNULL "NUL"
#define RANDOM rand
#define SRANDOM srand
#define PUTENV putenv
#define PR_EXECUTABLE "pr\\pr.exe"
#define LEVELS_DAT "prince\\LEVELS.DAT"
#define LEVELS_BAK "prince\\LEVELS.BAK"
#define PR_RESOURCES "pr\\resources.xml"
#define BATCH_FILE "prince\\apoplexy.bat"
#else
#define DEVNULL "/dev/null"
#define RANDOM random
#define SRANDOM srandom
#define PUTENV SDL_putenv
#define PR_EXECUTABLE "pr/pr"
#define LEVELS_DAT "prince/LEVELS.DAT"
#define LEVELS_BAK "prince/LEVELS.BAK"
#define PR_RESOURCES "pr/resources.xml"
#define BATCH_FILE "prince/apoplexy.bat"
#endif
/*========== Defines ==========*/

/*** for text ***/
SDL_Color color_bl = {0x00, 0x00, 0x00};
SDL_Color color_wh = {0xff, 0xff, 0xff};
SDL_Rect offset;
SDL_Surface *message = NULL;

/* Text. The magic identifier "POP_LVL" text distinguishes a POP
 * level file from other file formats.
 */
unsigned char sMagicID[7 + 2];

/*** UC. 01 = POP1, 02 = POP2 ***/
unsigned char sPOPVersion[1 + 2];

/*** UC. Level version. ***/
unsigned char sPLVVersion[1 + 2];

/*** UC. Level number. ***/
unsigned char sLevelNumber[1 + 2];

/*** UL. Number of fields. ***/
unsigned char sNumberOfFields[4 + 2];

/* UL. Size of the level in bytes. Excluding the checksum, all are 2305,
 * except the potions level, which is 2304. PR 1.1 includes the checksum
 * in the count, PR 1.2 does not.
 */
unsigned char sLevelSize[4 + 2];

/*** The checksum. ***/
unsigned char sChecksum[1 + 2];

/*** The first and second door events. ***/
unsigned char sFirstDoorEvents[256 + 2];
unsigned char sSecondDoorEvents[256 + 2];

unsigned char sRoomLinks[96 + 2];
unsigned char sUnknownI[64 + 2];
unsigned char sStartPosition[3 + 2];
unsigned char sUnknownIIandIII[4 + 2];
unsigned char sGuardLocations[ROOMS + 2];
unsigned char sGuardDirections[ROOMS + 2];
unsigned char sUnknownIVaandIVb[48 + 2];
unsigned char sGuardSkills[ROOMS + 2];
unsigned char sUnknownIVc[24 + 2];
unsigned char sGuardColors[ROOMS + 2];
unsigned char sUnknownIVd[16 + 2];
unsigned char sEndCode[2 + 2];
unsigned char sUserDataCount[4 + 2];
unsigned char sUserData[USER_DATA + 2];

unsigned int gamespeed;
Uint32 looptime;

int iDebug;
int iNoAudio;
int iScreen;
int iCurType;
int iCurRoom;
int iDownAt;
int iSelected;
int iOnTile;
int iOKOn;
int iYesOn;
int iNoOn;
int iCloseOn;
int iCloseCustomOn;
int iCustomOn;
int iCustomUseOn;
int iHelpOK;
unsigned long luLevelSize;
unsigned long luLevelNr;
unsigned long luKidRoom;
unsigned long luKidPos;
unsigned long luKidDir;
unsigned long luNumberOfFields;
int iRoomConnections[ROOMS + 2][4 + 2];
int iRoomConnectionsBroken[ROOMS + 2][4 + 2];
int iBrokenRoomLinks;
int iThingA[ROOMS + 2][30 + 2];
char sGroupA[ROOMS + 2][30 + 2];
int iModifierA[ROOMS + 2][30 + 2];
char sInformation[20 + 2][2 + 2][100 + 2]; /*** 20 times User Data ***/
int iInformationNr;
TTF_Font *font1 = NULL;
TTF_Font *font2 = NULL;
TTF_Font *font3 = NULL;
SDL_Surface *screen;
int iXPos, iYPos;
int iChanged;
int iChangeEvent;
int iGuardSkill;
char sAuthor[MAX_OPTION];
char sStartLevel[MAX_OPTION];
char sCheat[MAX_OPTION];
int iRoomArray[ROOMS + 1 + 2][ROOMS + 2];
int iMovingRoom;
int iMovingOldX;
int iMovingOldY;
int iMovingNewX;
int iMovingNewY;
int iMovingNewBusy;
int iChangingBrokenRoom;
int iChangingBrokenSide;
int iScale;
int arImage[1] = { 1 };
int arFont[1] = { 0 };
int iChangeForeRand;
int iChangeForeMod;
int iChangeFore;
int iChangeMod;
int iIsCustom;
int iExtras;
int iLastThing;
int iLastModifier;
int iThingACopyPaste[30 + 2];
int iModifierACopyPaste[30 + 2];
int iGuardACopyPaste[4 + 2];
int iCopied;

SDL_Surface *imgd1_2[2 + 2];
SDL_Surface *imgp1_2[2 + 2];
SDL_Surface *imgd1_12[2 + 2];
SDL_Surface *imgp1_12[2 + 2];
SDL_Surface *imgd1_22[2 + 2];
SDL_Surface *imgp1_22[2 + 2];
SDL_Surface *imgd1_32[2 + 2];
SDL_Surface *imgp1_32[2 + 2];
SDL_Surface *imgd1_2551[2 + 2];
SDL_Surface *imgp1_2551[2 + 2];
SDL_Surface *imgd2_2[2 + 2];
SDL_Surface *imgp2_2[2 + 2];
SDL_Surface *imgd2_12[2 + 2];
SDL_Surface *imgp2_12[2 + 2];
SDL_Surface *imgd2_22[2 + 2];
SDL_Surface *imgp2_22[2 + 2];
SDL_Surface *imgd2_32[2 + 2];
SDL_Surface *imgp2_32[2 + 2];
SDL_Surface *imgd2_2551[2 + 2];
SDL_Surface *imgp2_2551[2 + 2];
SDL_Surface *imgd3_6[2 + 2];
SDL_Surface *imgp3_6[2 + 2];
SDL_Surface *imgd3_15[2 + 2];
SDL_Surface *imgp3_15[2 + 2];
SDL_Surface *imgd3_24[2 + 2];
SDL_Surface *imgp3_24[2 + 2];
SDL_Surface *imgd3_34[2 + 2];
SDL_Surface *imgp3_34[2 + 2];
SDL_Surface *imgd3_43[2 + 2];
SDL_Surface *imgp3_43[2 + 2];
SDL_Surface *imgd3_53[2 + 2];
SDL_Surface *imgp3_53[2 + 2];
SDL_Surface *imgd3_62[2 + 2];
SDL_Surface *imgp3_62[2 + 2];
SDL_Surface *imgd3_71[2 + 2];
SDL_Surface *imgp3_71[2 + 2];
SDL_Surface *imgd3_81[2 + 2];
SDL_Surface *imgp3_81[2 + 2];
SDL_Surface *imgd3_91[2 + 2];
SDL_Surface *imgp3_91[2 + 2];
SDL_Surface *imgd4_4[2 + 2];
SDL_Surface *imgp4_4[2 + 2];
SDL_Surface *imgd5_0[2 + 2];
SDL_Surface *imgp5_0[2 + 2];
SDL_Surface *imgd5_1[2 + 2];
SDL_Surface *imgp5_1[2 + 2];
SDL_Surface *imgd6_4[2 + 2];
SDL_Surface *imgp6_4[2 + 2];
SDL_Surface *imgd7[2 + 2];
SDL_Surface *imgp7[2 + 2];
SDL_Surface *imgd8_7[2 + 2];
SDL_Surface *imgp8_7[2 + 2];
SDL_Surface *imgd8_16[2 + 2];
SDL_Surface *imgp8_16[2 + 2];
SDL_Surface *imgd8_25[2 + 2];
SDL_Surface *imgp8_25[2 + 2];
SDL_Surface *imgd8_35[2 + 2];
SDL_Surface *imgp8_35[2 + 2];
SDL_Surface *imgd9_4[2 + 2];
SDL_Surface *imgp9_4[2 + 2];
SDL_Surface *imgd10_4[2 + 2];
SDL_Surface *imgp10_4[2 + 2];
SDL_Surface *imgd11_5[2 + 2];
SDL_Surface *imgp11_5[2 + 2];
SDL_Surface *imgd11_14[2 + 2];
SDL_Surface *imgp11_14[2 + 2];
SDL_Surface *imgd11_23[2 + 2];
SDL_Surface *imgp11_23[2 + 2];
SDL_Surface *imgd11_33[2 + 2];
SDL_Surface *imgp11_33[2 + 2];
SDL_Surface *imgd11_42[2 + 2];
SDL_Surface *imgp11_42[2 + 2];
SDL_Surface *imgd11_52[2 + 2];
SDL_Surface *imgp11_52[2 + 2];
SDL_Surface *imgd11_61[2 + 2];
SDL_Surface *imgp11_61[2 + 2];
SDL_Surface *imgd12_4[2 + 2];
SDL_Surface *imgp12_4[2 + 2];
SDL_Surface *imgd13_8[2 + 2];
SDL_Surface *imgp13_8[2 + 2];
SDL_Surface *imgd13_17[2 + 2];
SDL_Surface *imgp13_17[2 + 2];
SDL_Surface *imgd13_26[2 + 2];
SDL_Surface *imgp13_26[2 + 2];
SDL_Surface *imgd13_36[2 + 2];
SDL_Surface *imgp13_36[2 + 2];
SDL_Surface *imgd13_44[2 + 2];
SDL_Surface *imgp13_44[2 + 2];
SDL_Surface *imgd13_54[2 + 2];
SDL_Surface *imgp13_54[2 + 2];
SDL_Surface *imgd13_63[2 + 2];
SDL_Surface *imgp13_63[2 + 2];
SDL_Surface *imgd13_72[2 + 2];
SDL_Surface *imgp13_72[2 + 2];
SDL_Surface *imgd14_4[2 + 2];
SDL_Surface *imgp14_4[2 + 2];
SDL_Surface *imgd15_4[2 + 2];
SDL_Surface *imgp15_4[2 + 2];
SDL_Surface *imgd16[2 + 2];
SDL_Surface *imgp16[2 + 2];
SDL_Surface *imgd17_4[2 + 2];
SDL_Surface *imgp17_4[2 + 2];
SDL_Surface *imgd18_4[2 + 2];
SDL_Surface *imgp18_4[2 + 2];
SDL_Surface *imgd19_1[2 + 2];
SDL_Surface *imgp19_1[2 + 2];
SDL_Surface *imgd19_11[2 + 2];
SDL_Surface *imgp19_11[2 + 2];
SDL_Surface *imgd19_21[2 + 2];
SDL_Surface *imgp19_21[2 + 2];
SDL_Surface *imgd19_31[2 + 2];
SDL_Surface *imgp19_31[2 + 2];
SDL_Surface *imgd19_41[2 + 2];
SDL_Surface *imgp19_41[2 + 2];
SDL_Surface *imgd19_51[2 + 2];
SDL_Surface *imgp19_51[2 + 2];
SDL_Surface *imgd20_4[2 + 2];
SDL_Surface *imgp20_4[2 + 2];
SDL_Surface *imgd21_9[2 + 2];
SDL_Surface *imgp21_9[2 + 2];
SDL_Surface *imgd21_18[2 + 2];
SDL_Surface *imgp21_18[2 + 2];
SDL_Surface *imgd22_4[2 + 2];
SDL_Surface *imgp22_4[2 + 2];
SDL_Surface *imgd23_4[2 + 2];
SDL_Surface *imgp23_4[2 + 2];
SDL_Surface *imgd24_4[2 + 2];
SDL_Surface *imgp24_4[2 + 2];
SDL_Surface *imgd25_4[2 + 2];
SDL_Surface *imgp25_4[2 + 2];
SDL_Surface *imgd26_4[2 + 2];
SDL_Surface *imgp26_4[2 + 2];
SDL_Surface *imgd27_4[2 + 2];
SDL_Surface *imgp27_4[2 + 2];
SDL_Surface *imgd28_4[2 + 2];
SDL_Surface *imgp28_4[2 + 2];
SDL_Surface *imgd29_4[2 + 2];
SDL_Surface *imgp29_4[2 + 2];
SDL_Surface *imgd30_4[2 + 2];
SDL_Surface *imgp30_4[2 + 2];
SDL_Surface *imgd31_4[2 + 2];
SDL_Surface *imgp31_4[2 + 2];
SDL_Surface *imgd44_4[2 + 2];
SDL_Surface *imgp44_4[2 + 2];

/*** guards ***/
SDL_Surface *imggdb[2 + 2];
SDL_Surface *imgggy[2 + 2];
SDL_Surface *imgglp[2 + 2];
SDL_Surface *imggoy[2 + 2];
SDL_Surface *imggpb[2 + 2];
SDL_Surface *imggrp[2 + 2];
SDL_Surface *imggyo[2 + 2];
SDL_Surface *imgskel[2 + 2];
SDL_Surface *imgfat[2 + 2];
SDL_Surface *imgjaffar[2 + 2];
SDL_Surface *imgshadow[2 + 2];

/*** guards, selected ***/
SDL_Surface *imggsel[2 + 2];
SDL_Surface *imgskelsel[2 + 2];
SDL_Surface *imgfatsel[2 + 2];
SDL_Surface *imgjaffarsel[2 + 2];
SDL_Surface *imgshadowsel[2 + 2];

/*** various ***/
SDL_Surface *imgunk[2 + 2], *imgup_0, *imgup_1, *imgdown_0, *imgdown_1;
SDL_Surface *imgleft_0, *imgleft_1, *imgright_0, *imgright_1, *imgroomson_0;
SDL_Surface *imgroomson_1, *imgroomsoff, *imgeventson_0, *imgeventson_1;
SDL_Surface *imgeventsoff, *imgsaveoff, *imgsaveon_0, *imgsaveon_1;
SDL_Surface *imgquit_0, *imgquit_1, *imgevents, *imgsele, *imgsell;
SDL_Surface *imgblack, *imghourglass, *imgprevon_0, *imgprevon_1;
SDL_Surface *imgnexton_0, *imgnexton_1, *imgbar;
SDL_Surface *extras_0, *extras_1, *extras_2, *extras_3, *extras_4;
SDL_Surface *extras_5, *extras_6, *extras_7, *extras_8;
SDL_Surface *imgudno, *imglrno, *imgudnonfo, *imgkidr[2 + 2], *imgkidl[2 + 2];
SDL_Surface *imgprevoff, *imgnextoff, *imgrl;
SDL_Surface *imgsrc, *imgsrs, *imgsrm, *imgsrp, *imgsrb;
SDL_Surface *imgfadedl, *imgfadeds, *imgpopup, *imgok[2 + 2];
SDL_Surface *imgpopup_yn, *imgyes[2 + 2], *imgno[2 + 2], *imghelp;
SDL_Surface *imgdungeon, *imgpalace, *imgclose_0, *imgclose_1;
SDL_Surface *imgborderb, *imgborders, *imgborderbl, *imgbordersl;
SDL_Surface *imgdother, *imgdjaffar, *imgdguards;
SDL_Surface *imgbroomson_0, *imgbroomson_1, *imgbroomsoff, *imgbrl;
SDL_Surface *imgcustoma0, *imgcustomi0, *imgcustoma1, *imgcustomi1, *imgcustom;
SDL_Surface *imgcustom_0, *imgcustom_1;

/*** the rooms ***/
SDL_Surface *imgroom1, *imgroom2, *imgroom3, *imgroom4, *imgroom5, *imgroom6;
SDL_Surface *imgroom7, *imgroom8, *imgroom9, *imgroom10, *imgroom11;
SDL_Surface *imgroom12, *imgroom13, *imgroom14, *imgroom15, *imgroom16;
SDL_Surface *imgroom17, *imgroom18, *imgroom19, *imgroom20, *imgroom21;
SDL_Surface *imgroom22, *imgroom23, *imgroom24;

int iMinX, iMaxX, iMinY, iMaxY, iStartRoomsX, iStartRoomsY;
int iDone[ROOMS + 2];

struct sample {
	Uint8 *data;
	Uint32 dpos;
	Uint32 dlen;
} sounds[NUM_SOUNDS];

void CheckRequiredFiles (void);
int LoadPLV (char *sFileName);
int EventInfo (int iNr, int iType);
int ReadFromFile (int iFd, char *sWhat, int iSize, unsigned char *sRetString);
void GetAsEightBits (unsigned char cChar, char *sBinary);
int BitsToInt (char *sString);
void ShowBinaryPosAsName (char *sBinaryFore, int iRoom, int iTile, int iFore);
void St (int iRoom, int iTile, char *sDescr, char cGroup);
void ShowBinaryModifierAsName (char *sBinaryMod,
	int iRoom, int iTile, int iMod);
char cShowDirection (int iDirection);
void ShowImage (int iThing, int iModifier, SDL_Surface *screen, int iLocation,
	int iFromImageX, int iFromImageY, int iFromImageWidth, int iFromImageHeight);
int InArea (SDL_Event event, int iUpperLeftX, int iUpperLeftY,
	int iLowerRightX, int iLowerRightY);
void InitScreen (void);
void EventDoor (int iX, int iY);
void EventNext (int iNoNext);
void RunLevel (int iLevel);
void EventRoom (int iRoom);
void RemoveOldRoom (void);
void AddNewRoom (int iX, int iY, int iRoom);
void InitRooms (void);
int StartGame (void *unused);
void Quit (void);
void ShowScreen (int iScreen);
void Prev (int iCurLevel);
void Next (int iCurLevel);
void WhereToStart (void);
void CheckSides (int iRoom, int iX, int iY);
void ShowRooms (int iRoom, int iX, int iY, int iNext);
void InitPopUp (int iAtScreen, int iStart);
void ShowPopUp (void);
void InitPopUpSave (int iAtScreen);
void ShowPopUpSave (void);
void DisplayText (int iStartX, int iStartY, int iFontSize,
	char sText[9 + 2][MAX_TEXT + 2], int iLines);
void ShowPos (SDL_Rect loc, SDL_Rect dest, int iLocation,
	SDL_Surface *img_first[2 + 2], SDL_Surface *img_second[2 + 2]);
void ChangePos (int iLocation, SDL_Surface *screen);
int ChangePosCustom (int iLocation, SDL_Surface *screen);
void ChangeEvent (int iAmount, int iChangePos);
void ChangeCustom (int iAmount, int iForeBack);
int IsDisabled (int iTile);
void UseTile (int iTile, int iLocation, int iRoom);
void SetGuard (int iRoom, int iLoc, int iDirection, int iSkill, int iColor);
void ShowChange (int iLocation);
void ShowChangeCustom (int iLocation);
void DisableSome (int iCurLevel);
int OnTile (SDL_Event event);
int Modulo (int iOne, int iTwo, int iType);
void SetLocation (int iRoom, int iLocation, int iThing, int iModifier);
void CallSave (int iJustXML);
void SavePLV (char *sFileName);
int WriteUserData (int iFd, int iType);
void WriteCharByChar (int iFd, unsigned char *sString, int iLength);
void ShowUsage (void);
void PrIfDe (char *sString);
void MixAudio (void *unused, Uint8 *stream, int iLen);
void PlaySound (char *sFile);
int BrokenRoomLinks (int iRoomConnections[ROOMS + 2][4 + 2], int iPrint);
int BrokenRoomChange (int iRoom, int iSide);
int MouseSelectAdj (SDL_Event event);
void CustomBlitSurface (SDL_Surface* src, SDL_Rect* srcrect,
	SDL_Surface* dst, SDL_Rect *dstrect, int arOptions[1]);
void LoadFonts (void);
void CustomTile (int iThing, int iModifier, char *sRetString);
void ShowCurrentCustom (SDL_Surface *screen);
void CreateBAK (void);
void Help (int iAtScreen);
void ShowHelp (void);
void Zoom (void);
void Sprinkle (void);
void LoadLevel (int iLevel);
void GetOptionValue (char *sArgv, char *sValue);
void FlipRoom (int iRoom, int iAxis);
void CopyPaste (int iRoom, int iAction);
void DateTime (char *sDateTime);
void SaveXML (char *sFileName);

/*****************************************************************************/
int main (int argc, char *argv[])
/*****************************************************************************/
{
	time_t tm;
	int iTemp;
	int iStartLevel;
	int iLevel;

	iDebug = 0;
	iNoAudio = 0;
	snprintf (sAuthor, MAX_OPTION, "%s user", EDITOR_NAME);
	snprintf (sCheat, MAX_OPTION, "%s", "megahit");
	iScale = 1;
	iStartLevel = 1;
	iExtras = 0;
	iLastThing = 0;
	iLastModifier = 0;
	iCopied = 0;

	if (argc > 1)
	{
		for (iTemp = 1; iTemp <= argc - 1; iTemp++)
		{
			if ((strcmp (argv[iTemp], "-h") == 0) ||
				(strcmp (argv[iTemp], "-?") == 0) ||
				(strcmp (argv[iTemp], "--help") == 0))
			{
				ShowUsage();
			}
			else if ((strcmp (argv[iTemp], "-v") == 0) ||
				(strcmp (argv[iTemp], "--version") == 0))
			{
				printf ("%s %s\n", EDITOR_NAME, EDITOR_VERSION);
				exit (EXIT_NORMAL);
			}
			else if ((strcmp (argv[iTemp], "-x") == 0) ||
				(strcmp (argv[iTemp], "--xml") == 0))
			{
				for (iLevel = 0; iLevel <= 15; iLevel++)
					{ LoadLevel (iLevel); CallSave (1); }
				printf ("Exported all levels to the \"" XML_DIR "\" directory.\n");
				exit (EXIT_NORMAL);
			}
			else if ((strcmp (argv[iTemp], "-d") == 0) ||
				(strcmp (argv[iTemp], "--debug") == 0))
			{
				iDebug = 1;
			}
			else if ((strcmp (argv[iTemp], "-n") == 0) ||
				(strcmp (argv[iTemp], "--noaudio") == 0))
			{
				iNoAudio = 1;
			}
			else if ((strcmp (argv[iTemp], "-i") == 0) ||
				(strcmp (argv[iTemp], "--improved") == 0))
			{
				snprintf (sCheat, MAX_OPTION, "%s", "improved");
			}
			else if ((strncmp (argv[iTemp], "-a=", 3) == 0) ||
				(strncmp (argv[iTemp], "--author=", 9) == 0))
			{
				GetOptionValue (argv[iTemp], sAuthor);
				if (iDebug == 1)
					printf ("[ INFO ] Using author name: %s\n", sAuthor);
			}
			else if ((strcmp (argv[iTemp], "-z") == 0) ||
				(strcmp (argv[iTemp], "--zoom") == 0))
			{
				iScale = 2;
			}
			else if ((strncmp (argv[iTemp], "-l=", 3) == 0) ||
				(strncmp (argv[iTemp], "--level=", 8) == 0))
			{
				GetOptionValue (argv[iTemp], sStartLevel);
				iStartLevel = atoi (sStartLevel);
				if ((iStartLevel < 0) || (iStartLevel > 15)) { iStartLevel = 1; }
				if (iDebug == 1)
					printf ("[ INFO ] Starting in level: %i\n", iStartLevel);
			}
			else if ((strncmp (argv[iTemp], "-c=", 3) == 0) ||
				(strncmp (argv[iTemp], "--cheat=", 8) == 0))
			{
				GetOptionValue (argv[iTemp], sCheat);
				if (iDebug == 1)
					printf ("[ INFO ] Using cheat code: %s\n", sCheat);
			}
			else
			{
				ShowUsage();
			}
		}
	}
	SRANDOM ((unsigned)time(&tm));

	CheckRequiredFiles();
	if (system (PR_EXECUTABLE " -x -f --resource=" PR_RESOURCES
		" " LEVELS_DAT " > " DEVNULL) == -1)
	{
		printf ("[FAILED] Could not export the levels: %s!\n", strerror (errno));
		exit (EXIT_ERROR);
	}

	LoadLevel (iStartLevel);
	iChangeEvent = 0;
	InitScreen();
	Quit();

	return 0;
}
/*****************************************************************************/
void CheckRequiredFiles (void)
/*****************************************************************************/
{
	int iTemp;

	if (access (LEVELS_DAT, R_OK|W_OK) == -1)
	{
		printf ("[FAILED] %s: %s!\n", LEVELS_DAT, strerror (errno));
		for (iTemp = 1; iTemp <= 79; iTemp++) { printf ("-"); }
		printf ("\nIn order to use %s, you need at least one file from the"
			" DOS version of\n", EDITOR_NAME);
		printf ("Prince of Persia 1. Copy the entire game, or just the file"
			" \"LEVELS.DAT\",\ninto the prince/ directory.\n\n");
		printf ("IMPORTANT: If you want to copy the entire game, use a version"
			" that has all\n");
		printf ("resources enabled in all levels. For example the version in this"
			" zip archive:\n");
		printf ("http://www.popot.org/get_the_games/software/PoP1.zip\n");
		for (iTemp = 1; iTemp <= 79; iTemp++) { printf ("-"); }
		printf ("\n\n");
		exit (EXIT_ERROR);
	}
	if ((access (PR_EXECUTABLE, R_OK) == -1)
		|| (access (PR_RESOURCES, R_OK) == -1))
	{
		printf ("[FAILED] %s or %s: %s!\n", PR_EXECUTABLE, PR_RESOURCES,
			strerror (errno));
		for (iTemp = 1; iTemp <= 79; iTemp++) { printf ("-"); }
		printf ("\nIn order to use %s, you need two files from the Princed"
			" Resources editor,\n", EDITOR_NAME);
		printf ("also known as PR. Download the source of PR 1.1 from www.princed"
			".org, compile\n");
		printf ("it and copy the files \"pr\" and \"resources.xml\" that can be"
			" found in src/bin to\n");
		printf ("pr/. IMPORTANT: PR 1.2 contains a level import bug, do NOT use"
			" it!\n");
		for (iTemp = 1; iTemp <= 79; iTemp++) { printf ("-"); }
		printf ("\n\n");
		exit (EXIT_ERROR);
	}
}
/*****************************************************************************/
int LoadPLV (char *sFileName)
/*****************************************************************************/
{
	int iFd;
	int iRoom;
	int iTile;
	int iTileStart;
	int iTileEnd;
	int iPos;
	int iType;
	int iTemp;
	int iTemp2;
	int iUD;
	char sBinaryFore[9 + 2]; /*** 8 chars, plus \0 ***/
	char sBinaryMod[9 + 2]; /*** 8 chars, plus \0 ***/
	char sString[MAX_DATA + 2];
	char sTemp[100 + 2];
	unsigned long luNumber;
	char sReferenceNr[7 + 2];
	int iFieldOrValue;

	/*** Everything on the foreground of the level. ***/
	unsigned char sLevelForeground[720 + 2];
	/*** The modifiers of the level. ***/
	unsigned char sLevelModifier[720 + 2];

	iChanged = 0;

	/*** Open file. ***/
	iFd = open (sFileName, O_RDONLY|O_BINARY);
	if (iFd == -1)
	{
		printf ("[FAILED] Could not open %s: %s!\n", sFileName, strerror (errno));
		printf ("[ INFO ] PR did not (or incorrectly) export this level\n\n");
		exit (EXIT_ERROR);
	}

	/*** Check the Magic ID. ***/
	ReadFromFile (iFd, "Magic ID", 7, sMagicID);
	snprintf (sString, MAX_DATA, "%s", sMagicID);
	if (strcmp (sString, "POP_LVL") != 0)
	{ printf ("[FAILED] This is not a POP level!\n"); exit (EXIT_ERROR); }

	/*** Check the POP Version. ***/
	ReadFromFile (iFd, "POP Version", 1, sPOPVersion);
	if (sPOPVersion[0] != 0x01)
	{ printf ("[FAILED] This is not a POP1 level!\n"); exit (EXIT_ERROR); }

	/*** Check the PLV Version. ***/
	ReadFromFile (iFd, "PLV Version", 1, sPLVVersion);
	if ((sPLVVersion[0] != 0x01) && (iDebug == 1))
	{ printf ("[ INFO ] Strange version detected: %02x\n", sPLVVersion[0]); }

	/*** Load the level number. ***/
	ReadFromFile (iFd, "Level Number", 1, sLevelNumber);
	snprintf (sString, MAX_DATA, "%02x", sLevelNumber[0]);
	luLevelNr = strtoul (sString, NULL, 16);
	if (iDebug == 1)
	{
		printf ("[ INFO ] The level number is: %lu\n", luLevelNr);
	}

	/*** Load the number of fields. ***/
	ReadFromFile (iFd, "Number Of Fields", 4, sNumberOfFields);
	snprintf (sString, MAX_DATA, "%02x%02x%02x%02x", sNumberOfFields[3],
		sNumberOfFields[2], sNumberOfFields[1], sNumberOfFields[0]);
	luNumber = strtoul (sString, NULL, 16);
	luNumberOfFields = luNumber;
	if (iDebug == 1)
	{
		printf ("[ INFO ] The number of user data fields is: %lu\n", luNumber);
	}

	/*** Load the level size. ***/
	ReadFromFile (iFd, "Level Size", 4, sLevelSize);
	snprintf (sString, MAX_DATA, "%02x%02x%02x%02x", sLevelSize[3],
		sLevelSize[2], sLevelSize[1], sLevelSize[0]);
	luLevelSize = strtoul (sString, NULL, 16);
	if ((luLevelSize < 2304) || (luLevelSize > 2306))
	{
		printf ("[FAILED] Level has an incorrect size!\n");
		exit (EXIT_ERROR);
	}
	if (iDebug == 1)
	{
		printf ("[ INFO ] The level size is: %lu\n", luLevelSize);
	}

	/*** Load the checksum. ***/
	ReadFromFile (iFd, "Checksum", 1, sChecksum);

	/*** Load the level foreground and modifier. ***/
	ReadFromFile (iFd, "Level Foreground", 720, sLevelForeground);
	ReadFromFile (iFd, "Level Modifier", 720, sLevelModifier);
	for (iRoom = 1; iRoom <= ROOMS; iRoom++)
	{
		if (iDebug == 1)
		{
			printf ("\nRoom %i:\n\n", iRoom);
		}
		for (iType = 1; iType <= 6; iType++)
		{
			if ((iType == 1) || (iType == 2)) { iTileStart = 0; iTileEnd = 9; }
			if ((iType == 3) || (iType == 4)) { iTileStart = 10; iTileEnd = 19; }
			if ((iType == 5) || (iType == 6)) { iTileStart = 20; iTileEnd = 29; }
			for (iTile = iTileStart; iTile <= iTileEnd; iTile++)
			{
				iPos = ((iRoom * 30) - 30) + iTile;
				switch (iType)
				{
					case 1:
					case 3:
					case 5:
						GetAsEightBits (sLevelForeground[iPos], sBinaryFore);
						ShowBinaryPosAsName (sBinaryFore, iRoom, iTile,
							(int)sLevelForeground[iPos]);
						break;
					case 2:
					case 4:
					case 6:
						GetAsEightBits (sLevelModifier[iPos], sBinaryMod);
						if ((iThingA[iRoom][iTile] == 4) ||
							(iThingA[iRoom][iTile] == 36))
						{
							if (iDebug == 1)
							{
								switch (BitsToInt (sBinaryMod))
								{
									case 1: printf ("Open   "); break;
									default: printf ("Closed "); break;
								}
							}
							iModifierA[iRoom][iTile] = BitsToInt (sBinaryMod);
						} else if ((iThingA[iRoom][iTile] == 6) ||
							(iThingA[iRoom][iTile] == 15) ||
							(iThingA[iRoom][iTile] == 38) ||
							(iThingA[iRoom][iTile] == 47)) /*** drop or raise ***/
						{
							snprintf (sString, 3 + 6, "%i      ",
								BitsToInt (sBinaryMod) + 1);
							strncpy (sReferenceNr, sString, 7);
							sReferenceNr[7] = '\0';
							PrIfDe (sReferenceNr);
							iModifierA[iRoom][iTile] = BitsToInt (sBinaryMod);
						} else {
							ShowBinaryModifierAsName (sBinaryMod, iRoom, iTile,
								(int)sLevelModifier[iPos]);
						}
						break;
					default:
						printf ("[FAILED] Unknown type!\n");
						exit (EXIT_ERROR);
				}
				if (iDebug == 1)
				{
					if ((iTile != 9) && (iTile != 19) && (iTile != 29)) { printf ("|"); }
				}
			}
			if (iDebug == 1)
			{
				if ((iType == 2) || (iType == 4))
				{
					printf ("\n");
					for (iTemp = 1; iTemp <= 79; iTemp++) { printf ("-"); }
					printf ("\n");
				} else {
					printf ("\n");
				}
			}
		}
		PrIfDe ("\n");
	}

	/*** Load first and second door events. ***/
	ReadFromFile (iFd, "First Door Events", 256, sFirstDoorEvents);
	ReadFromFile (iFd, "Second Door Events", 256, sSecondDoorEvents);
	if (iDebug == 1)
	{
		for (iTemp = 0; iTemp < 256; iTemp++)
		{
			printf ("[ INFO ] Event %i changes the door in room: %i, location: %i.",
				iTemp + 1, EventInfo (iTemp, 1), EventInfo (iTemp, 2));
			switch (EventInfo (iTemp, 3))
			{
				case 1: printf (" (next: yes)\n"); break;
				case 0: printf (" (next: no)\n"); break;
			}
		}
	}

	/*** Load the room links. ***/
	ReadFromFile (iFd, "Room Links", 96, sRoomLinks);
	for (iTemp = 0; iTemp < 96; iTemp+=4)
	{
		iRoomConnections[(iTemp / 4) + 1][1] =
			sRoomLinks[iTemp]; /*** left ***/
		iRoomConnections[(iTemp / 4) + 1][2] =
			sRoomLinks[iTemp + 1]; /*** right ***/
		iRoomConnections[(iTemp / 4) + 1][3] =
			sRoomLinks[iTemp + 2]; /*** up ***/
		iRoomConnections[(iTemp / 4) + 1][4] =
			sRoomLinks[iTemp + 3]; /*** down ***/
		if (iDebug == 1)
		{
			printf ("[ INFO ] Room %i is connected to room (0 = none): l%i, "
				"r%i, u%i, d%i\n", (iTemp / 4) + 1, sRoomLinks[iTemp],
				sRoomLinks[iTemp + 1], sRoomLinks[iTemp + 2], sRoomLinks[iTemp + 3]);
		}
	}

	/*** Load Unknown I. ***/
	ReadFromFile (iFd, "Unknown I", 64, sUnknownI);
	/*** I want remapped modifiers in all 24 rooms. ***/
	sUnknownI[0] = 0x18;

	/*** Load Start Position. ***/
	ReadFromFile (iFd, "Start Position", 3, sStartPosition);
	snprintf (sString, MAX_DATA, "%02x", sStartPosition[0]);
	luKidRoom = strtoul (sString, NULL, 16);
	snprintf (sString, MAX_DATA, "%02x", sStartPosition[1]);
	luKidPos = strtoul (sString, NULL, 16) + 1;
	snprintf (sString, MAX_DATA, "%02x", sStartPosition[2]);
	luKidDir = strtoul (sString, NULL, 16);
	if (((int)luLevelNr == 1) || ((int)luLevelNr == 13)) /*** 1 of 4 ***/
	{
		if (luKidDir == 0) { luKidDir = 255; }
		else if (luKidDir == 255) { luKidDir = 0; }
	}
	if (iDebug == 1)
	{
		printf ("[ INFO ] The kid starts in room: %lu, position: %lu, turned: "
			"%c\n", luKidRoom, luKidPos, cShowDirection ((int)luKidDir));
	}

	PrIfDe ("[  OK  ] Checking for broken room links.\n");
	iBrokenRoomLinks = BrokenRoomLinks (iRoomConnections, 1);

	/*** Load Unknown II-IVd and guard data. ***/
	switch ((int)luLevelNr)
	{
		case 15: /*** potions level ***/
			ReadFromFile (iFd, "Unknown II and III", 3, sUnknownIIandIII); break;
		default:
			ReadFromFile (iFd, "Unknown II and III", 4, sUnknownIIandIII); break;
	}
	ReadFromFile (iFd, "Guard Locations", ROOMS, sGuardLocations);
	ReadFromFile (iFd, "Guard Directions", ROOMS, sGuardDirections);
	ReadFromFile (iFd, "Unknown IVa and IVb", 48, sUnknownIVaandIVb);
	ReadFromFile (iFd, "Guard Skills", ROOMS, sGuardSkills);
	ReadFromFile (iFd, "Unknown IVc", 24, sUnknownIVc);
	ReadFromFile (iFd, "Guard Colors", ROOMS, sGuardColors);
	ReadFromFile (iFd, "Unknown IVd", 16, sUnknownIVd);

	/*** Load and display the end code. ***/
	ReadFromFile (iFd, "End Code", 2, sEndCode);
	snprintf (sString, MAX_DATA, "%02x%02x", sEndCode[0], sEndCode[1]);
	if (iDebug == 1)
	{
		printf ("[ INFO ] The end code is (usually 0f09): %s\n", sString);
	}

	/*** Display guard information. ***/
	if (iDebug == 1)
	{
		for (iTemp = 0; iTemp < ROOMS; iTemp++)
		{
			if (sGuardLocations[iTemp] <= 29)
			{
				printf ("[ INFO ] A guard in room: %i, location: %i, turned: %c, "
					"skill: %i, color: %i\n", iTemp + 1, sGuardLocations[iTemp] + 1,
					cShowDirection (sGuardDirections[iTemp]), sGuardSkills[iTemp],
					sGuardColors[iTemp]);
				if (((sUnknownIVaandIVb[iTemp] != 255) && (sUnknownIVaandIVb[iTemp]
					!= 0)) || ((sUnknownIVaandIVb[iTemp + 24] != 255) &&
					(sUnknownIVaandIVb[iTemp + 24] != 0)) || ((sUnknownIVc[iTemp] != 255)
					&& (sUnknownIVc[iTemp] != 0)))
				{
					printf ("[ INFO ] Extra data for the above guard: "
						"%i|%i|%i\n", sUnknownIVaandIVb[iTemp],
						sUnknownIVaandIVb[iTemp + 24], sUnknownIVc[iTemp]);
				}
			}
		}
	}

	/*** Read the user data. ***/
	ReadFromFile (iFd, "User Data Count", 4, sUserDataCount);
	snprintf (sString, MAX_DATA, "%02x%02x%02x%02x", sUserDataCount[3],
		sUserDataCount[2], sUserDataCount[1], sUserDataCount[0]);
	luNumber = strtoul (sString, NULL, 16);
	if (iDebug == 1)
	{
		printf ("[ INFO ] The user data count is: %lu (PR 1.1 has wrong value)\n",
			luNumber);
	}
	/*** NOTE: We use USER_DATA instead of (int)luNumber, because of PR 1.1. ***/
	iTemp2 = ReadFromFile (iFd, "User Data", USER_DATA, sUserData);
	snprintf (sString, 100, "%s", "");
	iInformationNr = 1;
	iFieldOrValue = 0;
	/*** NOTE: We use iTemp2 instead of (int)luNumber, because of PR 1.1. ***/
	for (iTemp = 0; iTemp < iTemp2; iTemp++)
	{
		if (sUserData[iTemp] == '\0')
		{
			snprintf (sInformation[iInformationNr][iFieldOrValue], 100, "%s",
				sString);
			snprintf (sString, 100, "%s", "");
			switch (iFieldOrValue)
			{
				case 0:
					iFieldOrValue = 1; break;
				case 1:
					iInformationNr++; iFieldOrValue = 0; break;
			}
		} else {
			snprintf (sTemp, 100, "%s", sString);
			snprintf (sString, 100, "%s%c", sTemp, sUserData[iTemp]);
		}
	}
	iUD = 0;
	iTemp2 = 0;
	for (iTemp = 1; iTemp < iInformationNr; iTemp++)
	{
		if (strcmp (sInformation[iTemp][0], "Editor Name") == 0)
		{
			iUD = iUD + 1;
			if (strcmp (sInformation[iTemp][1], "PR") == 0) { iTemp2++; }
		}
		if (strcmp (sInformation[iTemp][0], "Editor Version") == 0)
		{
			iUD = iUD + 2;
			if (strcmp (sInformation[iTemp][1], "v1.2") == 0) { iTemp2++; }
		}
		if (strcmp (sInformation[iTemp][0], "Level Author") == 0)
			{ iUD = iUD + 4; }
		if (strcmp (sInformation[iTemp][0], "Level Title") == 0)
			{ iUD = iUD + 8; }
		if (strcmp (sInformation[iTemp][0], "Level Description") == 0)
			{ iUD = iUD + 16; }
		if (strcmp (sInformation[iTemp][0], "Time Created") == 0)
			{ iUD = iUD + 32; }
		if (strcmp (sInformation[iTemp][0], "Time Last Modified") == 0)
			{ iUD = iUD + 64; }
		if (strcmp (sInformation[iTemp][0], "Original Filename") == 0)
			{ iUD = iUD + 128; }
		if (strcmp (sInformation[iTemp][0], "Original Level Number") == 0)
			{ iUD = iUD + 256; }
		if (iDebug == 1)
		{
			printf ("[ INFO ] %s: %s\n", sInformation[iTemp][0],
				sInformation[iTemp][1]);
		}
	}
	if ((iUD != 511) && (iDebug == 1))
	{
		printf ("[ INFO ] Could not find required User Data: %i\n", iUD);
	}
	if (iTemp2 == 2)
	{
		printf ("[FAILED] PR 1.2 contains a level import bug, do NOT use it!\n");
		exit (EXIT_ERROR);
	}

	iCurRoom = (int)luKidRoom;
	close (iFd);

	return (0);
}
/*****************************************************************************/
int EventInfo (int iNr, int iType)
/*****************************************************************************/
{
	char sBinaryFDoors[9 + 2]; /*** 8 chars, plus \0 ***/
	char sBinarySDoors[9 + 2]; /*** 8 chars, plus \0 ***/
	char sTemp[MAX_DATA + 2];

	GetAsEightBits (sFirstDoorEvents[iNr], sBinaryFDoors);
	GetAsEightBits (sSecondDoorEvents[iNr], sBinarySDoors);

	switch (iType)
	{
		case 1: /*** room ***/
			snprintf (sTemp, MAX_DATA, "%c%c%c%c%c", sBinarySDoors[0],
				sBinarySDoors[1], sBinarySDoors[2], sBinaryFDoors[1],
				sBinaryFDoors[2]);
			return (BitsToInt (sTemp)); break;
		case 2: /*** location ***/
			snprintf (sTemp, MAX_DATA, "%c%c%c%c%c", sBinaryFDoors[3],
				sBinaryFDoors[4], sBinaryFDoors[5], sBinaryFDoors[6],
				sBinaryFDoors[7]);
			return (BitsToInt (sTemp) + 1); break;
		case 3: /*** next ***/
			switch (sBinaryFDoors[0])
			{
				case '0': return (1); break;
				case '1': return (0); break;
			}
		default: return (-1); break;
	}
}
/*****************************************************************************/
int ReadFromFile (int iFd, char *sWhat, int iSize, unsigned char *sRetString)
/*****************************************************************************/
{
	int iLength;
	int iRead;
	char sRead[1 + 2];
	int iEOF;

	if (iDebug == 1)
	{
		printf ("[  OK  ] Loading: %s\n", sWhat);
	}
	iLength = 0;
	iEOF = 0;
	do {
		iRead = read (iFd, sRead, 1);
		switch (iRead)
		{
			case -1: printf ("[FAILED] Could not read!\n"); exit (EXIT_ERROR); break;
			case 0: PrIfDe ("[ INFO ] End of level file\n"); iEOF = 1; break;
			default:
				sRetString[iLength] = sRead[0];
				iLength++;
				break;
		}
	} while ((iLength < iSize) && (iEOF == 0));
	sRetString[iLength] = '\0';

	return (iLength);
}
/*****************************************************************************/
void GetAsEightBits (unsigned char cChar, char *sBinary)
/*****************************************************************************/
{
	int i = CHAR_BIT;
	int iTemp;

	iTemp = 0;
	while (i > 0)
	{
		i--;
		if (cChar&(1 << i))
		{
			sBinary[iTemp] = '1';
		} else {
			sBinary[iTemp] = '0';
		}
		iTemp++;
	}
	sBinary[iTemp] = '\0';
}
/*****************************************************************************/
int BitsToInt (char *sString)
/*****************************************************************************/
{
	/*** Converts binary to decimal. ***/
	/*** Example: 11111111 to 255 ***/

	int iTemp = 0;

	for (; *sString; iTemp = (iTemp << 1) | (*sString++ - '0'));
	return (iTemp);
}
/*****************************************************************************/
void ShowBinaryPosAsName (char *sBinaryFore, int iRoom, int iTile, int iFore)
/*****************************************************************************/
{
	char sPart[6 + 2];

	strncpy (sPart, sBinaryFore + 3, 5);
	sPart[5]='\0';
	if (strcmp (sPart, "00000") == 0) { St (iRoom, iTile, "Empty  ", 'f'); }
	if (strcmp (sPart, "00001") == 0) { St (iRoom, iTile, "Floor  ", 'f'); }
	if (strcmp (sPart, "00010") == 0) { St (iRoom, iTile, "Spikes ", 's'); }
	if (strcmp (sPart, "00011") == 0) { St (iRoom, iTile, "Pillar ", 'n'); }
	if (strcmp (sPart, "00100") == 0) { St (iRoom, iTile, "Gate   ", 'g'); }
	if (strcmp (sPart, "00101") == 0) { St (iRoom, iTile, "StckBtn", 'n'); }
	if (strcmp (sPart, "00110") == 0) { St (iRoom, iTile, "DropBtn", 'e'); }
	if (strcmp (sPart, "00111") == 0) { St (iRoom, iTile, "Tpstry ", 'y'); }
	if (strcmp (sPart, "01000") == 0) { St (iRoom, iTile, "BBgpilr", 'n'); }
	if (strcmp (sPart, "01001") == 0) { St (iRoom, iTile, "TBgpilr", 'n'); }
	if (strcmp (sPart, "01010") == 0) { St (iRoom, iTile, "Potion ", 'p'); }
	if (strcmp (sPart, "01011") == 0) { St (iRoom, iTile, "Loose  ", 'n'); }
	if (strcmp (sPart, "01100") == 0) { St (iRoom, iTile, "TpstryT", 't'); }
	if (strcmp (sPart, "01101") == 0) { St (iRoom, iTile, "Mirror ", 'n'); }
	if (strcmp (sPart, "01110") == 0) { St (iRoom, iTile, "Debris ", 'n'); }
	if (strcmp (sPart, "01111") == 0) { St (iRoom, iTile, "RaisBtn", 'e'); }
	if (strcmp (sPart, "10000") == 0) { St (iRoom, iTile, "ExitLft", 'd'); }
	if (strcmp (sPart, "10001") == 0) { St (iRoom, iTile, "ExitRgt", 'n'); }
	if (strcmp (sPart, "10010") == 0) { St (iRoom, iTile, "Chomper", 'c'); }
	if (strcmp (sPart, "10011") == 0) { St (iRoom, iTile, "Torch  ", 'n'); }
	if (strcmp (sPart, "10100") == 0) { St (iRoom, iTile, "Wall   ", 'w'); }
	if (strcmp (sPart, "10101") == 0) { St (iRoom, iTile, "Skeletn", 'n'); }
	if (strcmp (sPart, "10110") == 0) { St (iRoom, iTile, "Sword  ", 'n'); }
	if (strcmp (sPart, "10111") == 0) { St (iRoom, iTile, "BcnyLft", 'n'); }
	if (strcmp (sPart, "11000") == 0) { St (iRoom, iTile, "BcnyRgt", 'n'); }
	if (strcmp (sPart, "11001") == 0) { St (iRoom, iTile, "LcePilr", 'n'); }
	if (strcmp (sPart, "11010") == 0) { St (iRoom, iTile, "LceSprt", 'n'); }
	if (strcmp (sPart, "11011") == 0) { St (iRoom, iTile, "SmalLce", 'n'); }
	if (strcmp (sPart, "11100") == 0) { St (iRoom, iTile, "LceLft ", 'n'); }
	if (strcmp (sPart, "11101") == 0) { St (iRoom, iTile, "LceRgt ", 'n'); }
	if (strcmp (sPart, "11110") == 0) { St (iRoom, iTile, "TorchDb", 'n'); }
	if (strcmp (sPart, "11111") == 0) { St (iRoom, iTile, "Null   ", 'n'); }

	iThingA[iRoom][iTile] = iFore;
}
/*****************************************************************************/
void St (int iRoom, int iTile, char *sDescr, char cGroup)
/*****************************************************************************/
{
	PrIfDe (sDescr);

	sGroupA[iRoom][iTile] = cGroup;
}
/*****************************************************************************/
void ShowBinaryModifierAsName (char *sBinaryMod,
	int iRoom, int iTile, int iMod)
/*****************************************************************************/
{
	if (strcmp (sBinaryMod, "00000000") == 0)
	{
		switch (sGroupA[iRoom][iTile])
		{
			case 'c': PrIfDe ("Normal "); break;
			case 'f': PrIfDe ("+Not-Bl"); break;
			case 'g': PrIfDe ("Closed "); break; /*** says the documentation ***/
			case 'n': PrIfDe ("Always "); break;
			case 'p': PrIfDe ("Empty  "); break;
			case 's': PrIfDe ("Normal "); break;
			case 'y': PrIfDe ("-WithLt"); break;
			case 't': PrIfDe ("-WithLt"); break;
			case 'w': PrIfDe ("+Norm-B"); break;
			default: PrIfDe ("???????"); break;
		}
	}
	else if (strcmp (sBinaryMod, "00000001") == 0)
	{
		switch (sGroupA[iRoom][iTile])
		{
			case 'c': PrIfDe ("HalfOpn"); break;
			case 'f': PrIfDe ("+Spt1-N"); break;
			case 'g': PrIfDe ("Open   "); break;
			case 'p': PrIfDe ("HealthP"); break;
			case 's': PrIfDe ("BarelyO"); break;
			case 'y': PrIfDe ("-AlterD"); break;
			case 't': PrIfDe ("-AlterD"); break;
			case 'w': PrIfDe ("+Norm-N"); break;
			default: PrIfDe ("???????"); break;
		}
	}
	else if (strcmp (sBinaryMod, "00000010") == 0)
	{
		switch (sGroupA[iRoom][iTile])
		{
			case 'c': PrIfDe ("Closed "); break;
			case 'f': PrIfDe ("+Spt2-D"); break;
			case 'g': PrIfDe ("Closed "); break; /*** says the game ***/
			case 'p': PrIfDe ("Life   "); break;
			case 's': PrIfDe ("HalfOut"); break;
			case 'y': PrIfDe ("-Normal"); break;
			case 't': PrIfDe ("-Normal"); break;
			default: PrIfDe ("???????"); break;
		}
	}
	else if (strcmp (sBinaryMod, "00000011") == 0)
	{
		switch (sGroupA[iRoom][iTile])
		{
			case 'c': PrIfDe ("PartOpn"); break;
			case 'f': PrIfDe ("Window "); break;
			case 'p': PrIfDe ("FeatFal"); break;
			case 's': PrIfDe ("FulyOut"); break;
			case 'y': PrIfDe ("-Black "); break;
			case 't': PrIfDe ("-Black "); break;
			default: PrIfDe ("???????"); break;
		}
	}
	else if (strcmp (sBinaryMod, "00000100") == 0)
	{
		switch (sGroupA[iRoom][iTile])
		{
			case 'c': PrIfDe ("ExtrOpn"); break;
			case 'p': PrIfDe ("Invert "); break;
			case 's': PrIfDe ("FulyOut"); break;
			case 't': PrIfDe ("-Black "); break;
			default: PrIfDe ("???????"); break;
		}
	}
	else if (strcmp (sBinaryMod, "00000101") == 0)
	{
		switch (sGroupA[iRoom][iTile])
		{
			case 'c': PrIfDe ("StckOpn"); break;
			case 'p': PrIfDe ("Poison "); break;
			case 's': PrIfDe ("Out    "); break;
			case 't': PrIfDe ("-AltDes"); break;
			default: PrIfDe ("???????"); break;
		}
	}
	else if (strcmp (sBinaryMod, "00000110") == 0)
	{
		switch (sGroupA[iRoom][iTile])
		{
			case 'p': PrIfDe ("Open   "); break;
			case 's': PrIfDe ("Out    "); break;
			case 't': PrIfDe ("-WthBtm"); break;
			case 'f': PrIfDe ("POTIONS"); break;
			default: PrIfDe ("???????"); break;
		}
	}
	else if (strcmp (sBinaryMod, "00000111") == 0)
	{
		switch (sGroupA[iRoom][iTile])
		{
			case 's': PrIfDe ("HalfOut"); break;
			case 't': PrIfDe ("-WthWin"); break;
			default: PrIfDe ("???????"); break;
		}
	}
	else if (strcmp (sBinaryMod, "00001000") == 0)
	{
		switch (sGroupA[iRoom][iTile])
		{
			case 's': PrIfDe ("BarelyO"); break;
			default: PrIfDe ("???????"); break;
		}
	}
	else if (strcmp (sBinaryMod, "00001001") == 0)
	{
		switch (sGroupA[iRoom][iTile])
		{
			case 's': PrIfDe ("Disabld"); break;
			default: PrIfDe ("???????"); break;
		}
	}
	else if (strcmp (sBinaryMod, "11111111") == 0)
	{
		switch (sGroupA[iRoom][iTile])
		{
			case 'f': PrIfDe ("+Spt3-B"); break;
			case 'n': PrIfDe ("POTIONS"); break;
			default: PrIfDe ("???????"); break;
		}
	}
	else if (strcmp (sBinaryMod, "00100000") == 0)
	{
		switch (sGroupA[iRoom][iTile])
		{
			case 'p': PrIfDe ("PRINCES"); break;
			default: PrIfDe ("???????"); break;
		}
	}
	else if (strcmp (sBinaryMod, "01000000") == 0)
	{
		switch (sGroupA[iRoom][iTile])
		{
			case 'p': PrIfDe ("PRINCES"); break;
			default: PrIfDe ("???????"); break;
		}
	}
	else if (strcmp (sBinaryMod, "01001110") == 0)
	{
		switch (sGroupA[iRoom][iTile])
		{
			case 'f': PrIfDe ("POTIONS"); break;
			default: PrIfDe ("???????"); break;
		}
	}
	else if (strcmp (sBinaryMod, "10010000") == 0)
	{
		switch (sGroupA[iRoom][iTile])
		{
			case 'f': PrIfDe ("POTIONS"); break;
			default: PrIfDe ("???????"); break;
		}
	}
	else if (strcmp (sBinaryMod, "10100000") == 0)
	{
		switch (sGroupA[iRoom][iTile])
		{
			case 'p': PrIfDe ("PRINCES"); break;
			default: PrIfDe ("???????"); break;
		}
	}
	else if (strcmp (sBinaryMod, "11111101") == 0)
	{
		switch (sGroupA[iRoom][iTile])
		{
			case 'n': PrIfDe ("POTIONS"); break;
			default: PrIfDe ("???????"); break;
		}
	} else {
		PrIfDe ("???????");
	}
	iModifierA[iRoom][iTile] = iMod;
}
/*****************************************************************************/
char cShowDirection (int iDirection)
/*****************************************************************************/
{
	switch (iDirection)
	{
		case 0: return ('r'); break;
		case 255: return ('l'); break;
		default: return ('?'); break;
	}
}
/*****************************************************************************/
void ShowImage (int iThing, int iModifier, SDL_Surface *screen, int iLocation,
	int iFromImageX, int iFromImageY, int iFromImageWidth, int iFromImageHeight)
/*****************************************************************************/
{
	SDL_Rect dest;
	SDL_Rect loc;
	char sAdj[MAX_ADJ + 2];
	char sAdjR[MAX_ADJ + 2];
	char sAdjL[MAX_ADJ + 2];
	char sGrid[MAX_GRID + 2];
	char sGridX[MAX_GRID + 2];
	char sGridY[MAX_GRID + 2];
	char sText[MAX_TEXT + 2];
	char sCustomTile[MAX_CUSTOM + 2];
	int iTemp;
	int iTemp2;

	loc.x = (iFromImageX * iScale);
	loc.y = (iFromImageY * iScale);
	loc.w = (iFromImageWidth * iScale);
	loc.h = (iFromImageHeight * iScale);

	switch (iLocation)
	{
		case -1: dest.x = HOR0; dest.y = VER1; break;
		case -2: dest.x = HOR0; dest.y = VER2; break;
		case -3: dest.x = HOR0; dest.y = VER3; break;
		case -4: dest.x = HOR0; dest.y = VER4; break;
		case -5: dest.x = HOR1; dest.y = VER4; break;
		case -6: dest.x = HOR2; dest.y = VER4; break;
		case -7: dest.x = HOR3; dest.y = VER4; break;
		case -8: dest.x = HOR4; dest.y = VER4; break;
		case -9: dest.x = HOR5; dest.y = VER4; break;
		case -10: dest.x = HOR6; dest.y = VER4; break;
		case -11: dest.x = HOR7; dest.y = VER4; break;
		case -12: dest.x = HOR8; dest.y = VER4; break;
		case -13: dest.x = HOR9; dest.y = VER4; break;
		case -14: dest.x = HOR10; dest.y = VER4; break;
		case 1:
			if (iThing >= 0) /*** A tile. ***/
			{
				if (iRoomConnections[iCurRoom][1] != 0)
				{
					ShowImage (iThingA[iRoomConnections[iCurRoom][1]][9],
						iModifierA[iRoomConnections[iCurRoom][1]][9], screen,
						-1, 0, 0, 120, 155);
				} else {
					ShowImage (20, 0, screen, -1, 0, 0, 120, 155);
				}
				ShowImage (-11, 0, screen, -1, 0, 0, 120, 155);
			}
			dest.x = HOR1; dest.y = VER1;
			break;
		case 2: dest.x = HOR2; dest.y = VER1; break;
		case 3: dest.x = HOR3; dest.y = VER1; break;
		case 4: dest.x = HOR4; dest.y = VER1; break;
		case 5: dest.x = HOR5; dest.y = VER1; break;
		case 6: dest.x = HOR6; dest.y = VER1; break;
		case 7: dest.x = HOR7; dest.y = VER1; break;
		case 8: dest.x = HOR8; dest.y = VER1; break;
		case 9: dest.x = HOR9; dest.y = VER1; break;
		case 10: dest.x = HOR10; dest.y = VER1; break;

		case 11:
			if (iThing >= 0) /*** A tile. ***/
			{
				if (iRoomConnections[iCurRoom][1] != 0)
				{
					ShowImage (iThingA[iRoomConnections[iCurRoom][1]][19],
						iModifierA[iRoomConnections[iCurRoom][1]][19], screen,
						-2, 0, 0, 120, 155);
				} else {
					ShowImage (20, 0, screen, -2, 0, 0, 120, 155);
				}
				ShowImage (-11, 0, screen, -2, 0, 0, 120, 155);
			}
			dest.x = HOR1; dest.y = VER2;
			break;
		case 12: dest.x = HOR2; dest.y = VER2; break;
		case 13: dest.x = HOR3; dest.y = VER2; break;
		case 14: dest.x = HOR4; dest.y = VER2; break;
		case 15: dest.x = HOR5; dest.y = VER2; break;
		case 16: dest.x = HOR6; dest.y = VER2; break;
		case 17: dest.x = HOR7; dest.y = VER2; break;
		case 18: dest.x = HOR8; dest.y = VER2; break;
		case 19: dest.x = HOR9; dest.y = VER2; break;
		case 20: dest.x = HOR10; dest.y = VER2; break;

		case 21:
			if (iThing >= 0) /*** A tile. ***/
			{
				if (iRoomConnections[iRoomConnections[iCurRoom][1]][4] != 0)
				{
					ShowImage (iThingA[iRoomConnections[iRoomConnections
						[iCurRoom][1]][4]][9], iModifierA[iRoomConnections
						[iRoomConnections[iCurRoom][1]][4]][9], screen,
						-4, 0, 0, 120, 155);
				} else {
					ShowImage (20, 0, screen, -4, 0, 0, 120, 155);
				}
				ShowImage (-11, 0, screen, -4, 0, 0, 120, 155);
				if (iRoomConnections[iCurRoom][1] != 0)
				{
					ShowImage (iThingA[iRoomConnections[iCurRoom][1]][29],
						iModifierA[iRoomConnections[iCurRoom][1]][29], screen,
						-3, 0, 0, 120, 155);
				} else {
					ShowImage (20, 0, screen, -3, 0, 0, 120, 155);
				}
				ShowImage (-11, 0, screen, -3, 0, 0, 120, 155);
			}
			dest.x = HOR1; dest.y = VER3;
			break;
		case 22: dest.x = HOR2; dest.y = VER3; break;
		case 23: dest.x = HOR3; dest.y = VER3; break;
		case 24: dest.x = HOR4; dest.y = VER3; break;
		case 25: dest.x = HOR5; dest.y = VER3; break;
		case 26: dest.x = HOR6; dest.y = VER3; break;
		case 27: dest.x = HOR7; dest.y = VER3; break;
		case 28: dest.x = HOR8; dest.y = VER3; break;
		case 29: dest.x = HOR9; dest.y = VER3; break;
		case 30: dest.x = HOR10; dest.y = VER3; break;

		/*** black background & previous & faded background & changes & help ***/
		case 31: dest.x = 0; dest.y = 0; break;
		case 32: dest.x = 0; dest.y = 50; break; /*** left ***/
		case 33: dest.x = 667; dest.y = 50; break; /*** right ***/
		case 34: dest.x = 25; dest.y = 25; break; /*** up ***/
		case 35: dest.x = 25; dest.y = 430; break; /*** down ***/
		case 36: dest.x = 0; dest.y = 25; break; /*** rooms ***/
		case 37: dest.x = 667; dest.y = 25; break; /*** events ***/
		case 38: dest.x = 0; dest.y = 430; break; /*** save ***/
		case 39: dest.x = 667; dest.y = 430; break; /*** quit ***/
		case 40: dest.x = 667; dest.y = 0; break; /*** next ***/
		case 41: dest.x = 25; dest.y = 0; break; /*** level bar ***/
		case 42: dest.x = 25; dest.y = 50; break; /*** room links and events ***/
		case 43: dest.x = 590; dest.y = 405; break; /*** OK (help) ***/
		case 44: dest.x = 84; dest.y = 5; break; /*** popup ***/
		case 45: dest.x = 440; dest.y = 317; break; /*** OK ***/
		case 46: dest.x = 300; dest.y = 183; break; /*** hourglass ***/
		case 47: dest.x = 656; dest.y = 0; break; /*** close ***/
		case 48: /*** changing ***/
			if (iOnTile <= 78)
			{
				dest.x = 2 + ((Modulo (iOnTile, 13, 1) - 1) * 50);
				dest.y = 2 + ((Modulo (iOnTile, 13, 2) - 1) * 64);
			} else {
				dest.x = 2 + ((iOnTile - 78 - 1) * 25);
				dest.y = 386;
			}
			break;
		case 49: /*** sel event, room ***/
			dest.x = 282 + (EventInfo (iChangeEvent, 1) * 15);
			dest.y = 115; break;
		case 50: /*** sel event, location ***/
			iTemp = EventInfo (iChangeEvent, 2);
			if ((iTemp >= 1) && (iTemp <= 10))
			{
				dest.x = 447 + (iTemp * 15);
				dest.y = 155;
			}
			if ((iTemp >= 11) && (iTemp <= 20))
			{
				dest.x = 447 + ((iTemp - 10) * 15);
				dest.y = 170;
			}
			if ((iTemp >= 21) && (iTemp <= 30))
			{
				dest.x = 447 + ((iTemp - 20) * 15);
				dest.y = 185;
			}
			break;
		case 51: /*** sel event, next ***/
			dest.x = 582 + (EventInfo (iChangeEvent, 3) * 15);
			dest.y = 225;
			break;
		case 52: dest.x = 552; dest.y = 322; break;
		case 53: dest.x = 452; dest.y = 322; break;
		case 54: dest.x = 402; dest.y = 386; break;
		case 55: dest.x = 352; dest.y = 322; break;
		case 56: dest.x = 2; dest.y = 386; break;
		case 57: dest.x = 352; dest.y = 386; break; /*** kid, turned right ***/
		case 58: dest.x = 377; dest.y = 386; break; /*** kid, turned left ***/

		/*** guards ***/
		case 59: dest.x = 2; dest.y = 386; break;
		case 60: dest.x = 27; dest.y = 386; break;
		case 61: dest.x = 52; dest.y = 386; break;
		case 62: dest.x = 77; dest.y = 386; break;
		case 63: dest.x = 102; dest.y = 386; break;
		case 64: dest.x = 127; dest.y = 386; break;
		case 65: dest.x = 152; dest.y = 386; break;
		case 66: dest.x = 177; dest.y = 386; break;
		case 67: dest.x = 202; dest.y = 386; break;
		case 68: dest.x = 227; dest.y = 386; break;
		case 69: dest.x = 252; dest.y = 386; break;
		case 70: dest.x = 277; dest.y = 386; break;
		case 71: dest.x = 302; dest.y = 386; break;
		case 72: dest.x = 327; dest.y = 386; break;

		/*** skeleton, fat, jaffar, shadow ***/
		case 73: dest.x = 552; dest.y = 322; break;
		case 74: dest.x = 602; dest.y = 322; break;
		case 75: dest.x = 452; dest.y = 322; break;
		case 76: dest.x = 502; dest.y = 322; break;
		case 77: dest.x = 402; dest.y = 386; break;
		case 78: dest.x = 427; dest.y = 386; break;
		case 79: dest.x = 352; dest.y = 322; break;
		case 80: dest.x = 402; dest.y = 322; break;

		case 81:
			switch (iGuardSkill)
			{
				case 0: dest.x = 455; dest.y = 420; break;
				case 1: dest.x = 470; dest.y = 420; break;
				case 2: dest.x = 485; dest.y = 420; break;
				case 3: dest.x = 500; dest.y = 420; break;
				case 4: dest.x = 515; dest.y = 420; break;
				case 5: dest.x = 455; dest.y = 435; break;
				case 6: dest.x = 470; dest.y = 435; break;
				case 7: dest.x = 485; dest.y = 435; break;
				case 8: dest.x = 500; dest.y = 435; break;
				case 9: dest.x = 515; dest.y = 435; break;
				case 10: dest.x = 500; dest.y = 405; break;
				case 11: dest.x = 515; dest.y = 405; break;
			}
			break;
		case 82:
			iIsCustom = 0;
			iTemp = iThingA[(iCurRoom)][iSelected - 1];
			iTemp2 = iModifierA[iCurRoom][iSelected - 1];
			/*** first row ***/
			if (((iTemp == 0) || (iTemp == 32)) && (iTemp2 == 0))
				{ dest.x = 2; dest.y = 2; }
			else if (((iTemp == 0) || (iTemp == 32)) && (iTemp2 == 1))
				{ dest.x = 52; dest.y = 2; }
			else if (((iTemp == 0) || (iTemp == 32)) && (iTemp2 == 2))
				{ dest.x = 102; dest.y = 2; }
			else if (((iTemp == 0) || (iTemp == 32)) && (iTemp2 == 3))
				{ dest.x = 152; dest.y = 2; }
			else if (((iTemp == 0) || (iTemp == 32)) && (iTemp2 == 255))
				{ dest.x = 202; dest.y = 2; }
			else if (((iTemp == 1) || (iTemp == 33)) && (iTemp2 == 0))
				{ dest.x = 252; dest.y = 2; }
			else if (((iTemp == 1) || (iTemp == 33)) && (iTemp2 == 1))
				{ dest.x = 302; dest.y = 2; }
			else if (((iTemp == 1) || (iTemp == 33)) && (iTemp2 == 2))
				{ dest.x = 352; dest.y = 2; }
			else if (((iTemp == 1) || (iTemp == 33)) && (iTemp2 == 3))
				{ dest.x = 402; dest.y = 2; }
			else if (((iTemp == 1) || (iTemp == 33)) && (iTemp2 == 255))
				{ dest.x = 452; dest.y = 2; }
			else if (((iTemp == 2) || (iTemp == 34)) && (iTemp2 == 0))
				{ dest.x = 502; dest.y = 2; }
			else if (((iTemp == 2) || (iTemp == 34)) && (iTemp2 == 1))
				{ dest.x = 552; dest.y = 2; }
			else if (((iTemp == 2) || (iTemp == 34)) && (iTemp2 == 2))
				{ dest.x = 602; dest.y = 2; }

			/*** second row ***/
			else if (((iTemp == 2) || (iTemp == 34)) && (iTemp2 == 3))
				{ dest.x = 2; dest.y = 66; }
			else if (((iTemp == 2) || (iTemp == 34)) && (iTemp2 == 4))
				{ dest.x = 52; dest.y = 66; }
			else if (((iTemp == 2) || (iTemp == 34)) && (iTemp2 == 5))
				{ dest.x = 102; dest.y = 66; }
			else if (((iTemp == 2) || (iTemp == 34)) && (iTemp2 == 6))
				{ dest.x = 152; dest.y = 66; }
			else if (((iTemp == 2) || (iTemp == 34)) && (iTemp2 == 7))
				{ dest.x = 202; dest.y = 66; }
			else if (((iTemp == 2) || (iTemp == 34)) && (iTemp2 == 8))
				{ dest.x = 252; dest.y = 66; }
			else if (((iTemp == 2) || (iTemp == 34)) && (iTemp2 == 9))
				{ dest.x = 302; dest.y = 66; }
			else if (((iTemp == 3) || (iTemp == 35)) && (iTemp2 == 0))
				{ dest.x = 352; dest.y = 66; }
			else if (((iTemp == 4) || (iTemp == 36)) &&
				((iTemp2 == 0) || (iTemp2 == 2)))
				{ dest.x = 402; dest.y = 66; }
			else if (((iTemp == 4) || (iTemp == 36)) && (iTemp2 == 1))
				{ dest.x = 452; dest.y = 66; }
			else if (((iTemp == 5) || (iTemp == 37)) && (iTemp2 == 0))
				{ dest.x = 502; dest.y = 66; }
			else if ((iTemp == 6) || (iTemp == 38))
				{ dest.x = 552; dest.y = 66; iChangeEvent = iTemp2; }
			else if (((iTemp == 7) || (iTemp == 39)) && (iTemp2 == 0))
				{ dest.x = 602; dest.y = 66; }

			/*** third row ***/
			else if (((iTemp == 7) || (iTemp == 39)) && (iTemp2 == 1))
				{ dest.x = 2; dest.y = 130; }
			else if (((iTemp == 7) || (iTemp == 39)) && (iTemp2 == 2))
				{ dest.x = 52; dest.y = 130; }
			else if (((iTemp == 7) || (iTemp == 39)) && (iTemp2 == 3))
				{ dest.x = 102; dest.y = 130; }
			else if (((iTemp == 8) || (iTemp == 40)) && (iTemp2 == 0))
				{ dest.x = 152; dest.y = 130; }
			else if (((iTemp == 9) || (iTemp == 41)) && (iTemp2 == 0))
				{ dest.x = 202; dest.y = 130; }
			else if (((iTemp == 10) || (iTemp == 42)) && (iTemp2 == 0))
				{ dest.x = 252; dest.y = 130; }
			else if (((iTemp == 10) || (iTemp == 42)) && (iTemp2 == 1))
				{ dest.x = 302; dest.y = 130; }
			else if (((iTemp == 10) || (iTemp == 42)) && (iTemp2 == 2))
				{ dest.x = 352; dest.y = 130; }
			else if (((iTemp == 10) || (iTemp == 42)) && (iTemp2 == 3))
				{ dest.x = 402; dest.y = 130; }
			else if (((iTemp == 10) || (iTemp == 42)) && (iTemp2 == 4))
				{ dest.x = 452; dest.y = 130; }
			else if (((iTemp == 10) || (iTemp == 42)) && (iTemp2 == 5))
				{ dest.x = 502; dest.y = 130; }
			else if (((iTemp == 10) || (iTemp == 42)) && (iTemp2 == 6))
				{ dest.x = 552; dest.y = 130; }
			else if ((iTemp == 11) && (iTemp2 == 0))
				{ dest.x = 602; dest.y = 130; }

			/*** fourth row ***/
			else if (((iTemp == 12) || (iTemp == 44)) && (iTemp2 == 0))
				{ dest.x = 2; dest.y = 194; }
			else if (((iTemp == 12) || (iTemp == 44)) && (iTemp2 == 1))
				{ dest.x = 52; dest.y = 194; }
			else if (((iTemp == 12) || (iTemp == 44)) && (iTemp2 == 2))
				{ dest.x = 102; dest.y = 194; }
			else if (((iTemp == 12) || (iTemp == 44)) && (iTemp2 == 3))
				{ dest.x = 152; dest.y = 194; }
			else if (((iTemp == 12) || (iTemp == 44)) && (iTemp2 == 4))
				{ dest.x = 202; dest.y = 194; }
			else if (((iTemp == 12) || (iTemp == 44)) && (iTemp2 == 5))
				{ dest.x = 252; dest.y = 194; }
			else if (((iTemp == 12) || (iTemp == 44)) && (iTemp2 == 6))
				{ dest.x = 302; dest.y = 194; }
			else if (((iTemp == 12) || (iTemp == 44)) && (iTemp2 == 7))
				{ dest.x = 352; dest.y = 194; }
			else if (((iTemp == 13) || (iTemp == 45)) && (iTemp2 == 0))
				{ dest.x = 402; dest.y = 194; }
			else if (((iTemp == 14) || (iTemp == 46)) && (iTemp2 == 0))
				{ dest.x = 452; dest.y = 194; }
			else if ((iTemp == 15) || (iTemp == 47))
				{ dest.x = 502; dest.y = 194; iChangeEvent = iTemp2; }
			else if (((iTemp == 16) || (iTemp == 48)) && (iTemp2 == 0))
				{ dest.x = 552; dest.y = 194; }
			else if (((iTemp == 17) || (iTemp == 49)) && (iTemp2 == 0))
				{ dest.x = 602; dest.y = 194; }

			/*** fifth row ***/
			else if (((iTemp == 18) || (iTemp == 50)) && (iTemp2 == 0))
				{ dest.x = 2; dest.y = 258; }
			else if (((iTemp == 18) || (iTemp == 50)) && (iTemp2 == 1))
				{ dest.x = 52; dest.y = 258; }
			else if (((iTemp == 18) || (iTemp == 50)) && (iTemp2 == 2))
				{ dest.x = 102; dest.y = 258; }
			else if (((iTemp == 18) || (iTemp == 50)) && (iTemp2 == 3))
				{ dest.x = 152; dest.y = 258; }
			else if (((iTemp == 18) || (iTemp == 50)) && (iTemp2 == 4))
				{ dest.x = 202; dest.y = 258; }
			else if (((iTemp == 18) || (iTemp == 50)) && (iTemp2 == 5))
				{ dest.x = 252; dest.y = 258; }
			else if (((iTemp == 19) || (iTemp == 51)) && (iTemp2 == 0))
				{ dest.x = 302; dest.y = 258; }
			else if (((iTemp == 20) || (iTemp == 52)) && (iTemp2 == 0))
				{ dest.x = 352; dest.y = 258; }
			else if (((iTemp == 20) || (iTemp == 52)) && (iTemp2 == 1))
				{ dest.x = 402; dest.y = 258; }
			else if (((iTemp == 21) || (iTemp == 53)) && (iTemp2 == 0))
				{ dest.x = 452; dest.y = 258; }
			else if (((iTemp == 22) || (iTemp == 54)) && (iTemp2 == 0))
				{ dest.x = 502; dest.y = 258; }
			else if (((iTemp == 23) || (iTemp == 55)) && (iTemp2 == 0))
				{ dest.x = 552; dest.y = 258; }
			else if (((iTemp == 24) || (iTemp == 56)) && (iTemp2 == 0))
				{ dest.x = 602; dest.y = 258; }

			/*** sixth row ***/
			else if (((iTemp == 25) || (iTemp == 57)) && (iTemp2 == 0))
				{ dest.x = 2; dest.y = 322; }
			else if (((iTemp == 26) || (iTemp == 58)) && (iTemp2 == 0))
				{ dest.x = 52; dest.y = 322; }
			else if (((iTemp == 27) || (iTemp == 59)) && (iTemp2 == 0))
				{ dest.x = 102; dest.y = 322; }
			else if (((iTemp == 28) || (iTemp == 60)) && (iTemp2 == 0))
				{ dest.x = 152; dest.y = 322; }
			else if (((iTemp == 29) || (iTemp == 61)) && (iTemp2 == 0))
				{ dest.x = 202; dest.y = 322; }
			else if (((iTemp == 30) || (iTemp == 62)) && (iTemp2 == 0))
				{ dest.x = 252; dest.y = 322; }
			else if ((iTemp == 43) && (iTemp2 == 0))
				{ dest.x = 302; dest.y = 322; }
			else
				{ iIsCustom = 1; }
			break;
		case 83:
			/*** mouse cursor doesn't need scaling ***/
			dest.x = (iXPos + 10) / iScale;
			dest.y = (iYPos + 10) / iScale;
			break;
		case 84: dest.x = 150; dest.y = 89; break; /*** popup_yn ***/
		case 85: dest.x = 440; dest.y = 317; break; /*** Yes ***/
		case 86: dest.x = 167; dest.y = 317; break; /*** No ***/
		case 87: dest.x = 629; dest.y = 63; break; /*** rooms broken ***/
		case 88: dest.x = 656; dest.y = 387; break; /*** custom ***/
		case 89:
			switch (iChangeForeRand)
			{
				case 0: dest.x = 33; break;
				case 1: dest.x = 48; break;
				case 2: dest.x = 63; break;
				case 3: dest.x = 78; break;
			}
			dest.y = 114;
			break;
		case 90:
			if (iChangeForeMod == 0) { dest.x = 114; dest.y = 114; }
			if (iChangeForeMod == 1) { dest.x = 129; dest.y = 114; }
			break;
		case 91: dest.x = 76; dest.y = 241; break; /*** current custom ***/
		case 92: dest.x = 77; dest.y = 146; break; /*** event target ***/
		case 93: dest.x = 620; dest.y = 3; break; /*** extras ***/
	}
	if ((iLocation >= 2010) && (iLocation <= 2244)) /*** adjacent rooms ***/
	{
		snprintf (sAdj, MAX_ADJ, "%i", iLocation);
		snprintf (sAdjR, MAX_ADJ, "%c%c", sAdj[1], sAdj[2]);
		snprintf (sAdjL, MAX_ADJ, "%c", sAdj[3]);

		switch (atoi (sAdjL))
		{
			case 0:
				dest.x = BROKEN_ROOM_X;
				dest.y = BROKEN_ROOM_Y;
				break;
			case 1:
				dest.x = BROKEN_LEFT_X;
				dest.y = BROKEN_LEFT_Y;
				break;
			case 2:
				dest.x = BROKEN_RIGHT_X;
				dest.y = BROKEN_RIGHT_Y;
				break;
			case 3:
				dest.x = BROKEN_UP_X;
				dest.y = BROKEN_UP_Y;
				break;
			case 4:
				dest.x = BROKEN_DOWN_X;
				dest.y = BROKEN_DOWN_Y;
				break;
		}

		switch (atoi (sAdjR))
		{
			case 1: dest.x += (63 * 0); dest.y += (63 * 0); break;
			case 2: dest.x += (63 * 1); dest.y += (63 * 0); break;
			case 3: dest.x += (63 * 2); dest.y += (63 * 0); break;
			case 4: dest.x += (63 * 3); dest.y += (63 * 0); break;
			case 5: dest.x += (63 * 0); dest.y += (63 * 1); break;
			case 6: dest.x += (63 * 1); dest.y += (63 * 1); break;
			case 7: dest.x += (63 * 2); dest.y += (63 * 1); break;
			case 8: dest.x += (63 * 3); dest.y += (63 * 1); break;
			case 9: dest.x += (63 * 0); dest.y += (63 * 2); break;
			case 10: dest.x += (63 * 1); dest.y += (63 * 2); break;
			case 11: dest.x += (63 * 2); dest.y += (63 * 2); break;
			case 12: dest.x += (63 * 3); dest.y += (63 * 2); break;
			case 13: dest.x += (63 * 0); dest.y += (63 * 3); break;
			case 14: dest.x += (63 * 1); dest.y += (63 * 3); break;
			case 15: dest.x += (63 * 2); dest.y += (63 * 3); break;
			case 16: dest.x += (63 * 3); dest.y += (63 * 3); break;
			case 17: dest.x += (63 * 0); dest.y += (63 * 4); break;
			case 18: dest.x += (63 * 1); dest.y += (63 * 4); break;
			case 19: dest.x += (63 * 2); dest.y += (63 * 4); break;
			case 20: dest.x += (63 * 3); dest.y += (63 * 4); break;
			case 21: dest.x += (63 * 0); dest.y += (63 * 5); break;
			case 22: dest.x += (63 * 1); dest.y += (63 * 5); break;
			case 23: dest.x += (63 * 2); dest.y += (63 * 5); break;
			case 24: dest.x += (63 * 3); dest.y += (63 * 5); break;
		}
	}
	if ((iLocation >= 10101) && (iLocation <= 12524)) /*** room grid ***/
	{
		snprintf (sGrid, MAX_GRID, "%i", iLocation);
		snprintf (sGridX, MAX_GRID, "%c%c", sGrid[1], sGrid[2]);
		snprintf (sGridY, MAX_GRID, "%c%c", sGrid[3], sGrid[4]);

		/*** save room location ***/
		if (iThing != -7)
		{
			iRoomArray[atoi (sGridX)][atoi (sGridY)] = iModifier;
		}

		if (strcmp (sGridX, "25") == 0)
		{
			dest.x = 272; /*** side pane ***/
		} else {
			dest.x = 282 + (atoi (sGridX) * 15); /*** grid, 24x24 ***/
		}
		dest.y = 46 + (atoi (sGridY) * 15);
	}

	/*** For some reason I don't need to set ***/
	/*** dest.w = img->w and dest.h = img->h ***/

	/*** black background ***/
	if ((iThing == -4) && (iModifier == 1))
		CustomBlitSurface (imgblack, &loc, screen, &dest, arImage);

	/*** hourglass ***/
	else if ((iThing == -4) && (iModifier == 2))
		CustomBlitSurface (imghourglass, &loc, screen, &dest, arImage);

	/*** room links and selection ***/
	else if ((iThing == -6) && (iModifier == 0))
		CustomBlitSurface (imgrl, &loc, screen, &dest, arImage);
	else if ((iThing == -6) && (iModifier == 25))
		CustomBlitSurface (imgbrl, &loc, screen, &dest, arImage);
	else if ((iThing == -7) && (iModifier == 0))
		CustomBlitSurface (imgsrc, &loc, screen, &dest, arImage);
	else if ((iThing == -7) && (iModifier == 1))
		CustomBlitSurface (imgsrs, &loc, screen, &dest, arImage);
	else if ((iThing == -7) && (iModifier == 2))
		CustomBlitSurface (imgsrm, &loc, screen, &dest, arImage);
	else if ((iThing == -7) && (iModifier == 3))
		CustomBlitSurface (imgsrp, &loc, screen, &dest, arImage);
	else if ((iThing == -7) && (iModifier == 4))
		CustomBlitSurface (imgsrb, &loc, screen, &dest, arImage);

	/*** rooms ***/
	else if ((iThing == -6) && (iModifier == 1))
		CustomBlitSurface (imgroom1, &loc, screen, &dest, arImage);
	else if ((iThing == -6) && (iModifier == 2))
		CustomBlitSurface (imgroom2, &loc, screen, &dest, arImage);
	else if ((iThing == -6) && (iModifier == 3))
		CustomBlitSurface (imgroom3, &loc, screen, &dest, arImage);
	else if ((iThing == -6) && (iModifier == 4))
		CustomBlitSurface (imgroom4, &loc, screen, &dest, arImage);
	else if ((iThing == -6) && (iModifier == 5))
		CustomBlitSurface (imgroom5, &loc, screen, &dest, arImage);
	else if ((iThing == -6) && (iModifier == 6))
		CustomBlitSurface (imgroom6, &loc, screen, &dest, arImage);
	else if ((iThing == -6) && (iModifier == 7))
		CustomBlitSurface (imgroom7, &loc, screen, &dest, arImage);
	else if ((iThing == -6) && (iModifier == 8))
		CustomBlitSurface (imgroom8, &loc, screen, &dest, arImage);
	else if ((iThing == -6) && (iModifier == 9))
		CustomBlitSurface (imgroom9, &loc, screen, &dest, arImage);
	else if ((iThing == -6) && (iModifier == 10))
		CustomBlitSurface (imgroom10, &loc, screen, &dest, arImage);
	else if ((iThing == -6) && (iModifier == 11))
		CustomBlitSurface (imgroom11, &loc, screen, &dest, arImage);
	else if ((iThing == -6) && (iModifier == 12))
		CustomBlitSurface (imgroom12, &loc, screen, &dest, arImage);
	else if ((iThing == -6) && (iModifier == 13))
		CustomBlitSurface (imgroom13, &loc, screen, &dest, arImage);
	else if ((iThing == -6) && (iModifier == 14))
		CustomBlitSurface (imgroom14, &loc, screen, &dest, arImage);
	else if ((iThing == -6) && (iModifier == 15))
		CustomBlitSurface (imgroom15, &loc, screen, &dest, arImage);
	else if ((iThing == -6) && (iModifier == 16))
		CustomBlitSurface (imgroom16, &loc, screen, &dest, arImage);
	else if ((iThing == -6) && (iModifier == 17))
		CustomBlitSurface (imgroom17, &loc, screen, &dest, arImage);
	else if ((iThing == -6) && (iModifier == 18))
		CustomBlitSurface (imgroom18, &loc, screen, &dest, arImage);
	else if ((iThing == -6) && (iModifier == 19))
		CustomBlitSurface (imgroom19, &loc, screen, &dest, arImage);
	else if ((iThing == -6) && (iModifier == 20))
		CustomBlitSurface (imgroom20, &loc, screen, &dest, arImage);
	else if ((iThing == -6) && (iModifier == 21))
		CustomBlitSurface (imgroom21, &loc, screen, &dest, arImage);
	else if ((iThing == -6) && (iModifier == 22))
		CustomBlitSurface (imgroom22, &loc, screen, &dest, arImage);
	else if ((iThing == -6) && (iModifier == 23))
		CustomBlitSurface (imgroom23, &loc, screen, &dest, arImage);
	else if ((iThing == -6) && (iModifier == 24))
		CustomBlitSurface (imgroom24, &loc, screen, &dest, arImage);

	/*** events ***/
	else if ((iThing == -10) && (iModifier == 0))
		CustomBlitSurface (imgevents, &loc, screen, &dest, arImage);
	else if ((iThing == -10) && (iModifier == 1))
		CustomBlitSurface (imgsele, &loc, screen, &dest, arImage);

	/*** faded tile ***/
	else if ((iThing == -11) && (iModifier == 0))
		CustomBlitSurface (imgfadeds, &loc, screen, &dest, arImage);

	/*** changing ***/
	else if ((iThing == -12) && (iModifier == 0))
		CustomBlitSurface (imgdungeon, &loc, screen, &dest, arImage);
	else if ((iThing == -12) && (iModifier == 1))
		CustomBlitSurface (imgpalace, &loc, screen, &dest, arImage);
	else if ((iThing == -12) && (iModifier == 2))
		CustomBlitSurface (imgclose_0, &loc, screen, &dest, arImage);
	else if ((iThing == -12) && (iModifier == 3))
		CustomBlitSurface (imgclose_1, &loc, screen, &dest, arImage);
	else if ((iThing == -12) && (iModifier == 4))
	{
		if (IsDisabled (iOnTile) == 0)
		{
			/*** Show bright green border. ***/
			if (iOnTile <= 78)
				CustomBlitSurface (imgborderb, &loc, screen, &dest, arImage);
			else CustomBlitSurface (imgborders, &loc, screen, &dest, arImage);
		}
	}
	else if ((iThing == -12) && (iModifier == 5))
		CustomBlitSurface (imgdother, &loc, screen, &dest, arImage);
	else if ((iThing == -12) && (iModifier == 6))
		CustomBlitSurface (imgdother, &loc, screen, &dest, arImage);
	else if ((iThing == -12) && (iModifier == 7))
		CustomBlitSurface (imgdjaffar, &loc, screen, &dest, arImage);
	else if ((iThing == -12) && (iModifier == 8))
		CustomBlitSurface (imgdother, &loc, screen, &dest, arImage);
	else if ((iThing == -12) && (iModifier == 9))
		CustomBlitSurface (imgdguards, &loc, screen, &dest, arImage);
	else if ((iThing == -12) && (iModifier == 10))
		CustomBlitSurface (imgbordersl, &loc, screen, &dest, arImage);
	else if ((iThing == -12) && (iModifier == 11))
	{
		if (iIsCustom == 0)
			CustomBlitSurface (imgborderbl, &loc, screen, &dest, arImage);
	}
	else if ((iThing == -12) && (iModifier == 12))
		CustomBlitSurface (imgsell, &loc, screen, &dest, arImage);
	else if ((iThing == -12) && (iModifier == 13))
		CustomBlitSurface (imgcustoma0, &loc, screen, &dest, arImage);
	else if ((iThing == -12) && (iModifier == 14))
		CustomBlitSurface (imgcustomi0, &loc, screen, &dest, arImage);
	else if ((iThing == -12) && (iModifier == 15))
		CustomBlitSurface (imgcustoma1, &loc, screen, &dest, arImage);
	else if ((iThing == -12) && (iModifier == 16))
		CustomBlitSurface (imgcustomi1, &loc, screen, &dest, arImage);
	else if ((iThing == -12) && (iModifier == 17))
		CustomBlitSurface (imgcustom, &loc, screen, &dest, arImage);
	else if ((iThing == -12) && (iModifier == 18))
		CustomBlitSurface (imgcustom_0, &loc, screen, &dest, arImage);
	else if ((iThing == -12) && (iModifier == 19))
		CustomBlitSurface (imgcustom_1, &loc, screen, &dest, arImage);

	/*** arrows ***/
	else if ((iThing == -1) && (iModifier == 1))
	{
		if (iDownAt == 1)
		{
			CustomBlitSurface (imgleft_1, &loc, screen, &dest, arImage);
		} else {
			CustomBlitSurface (imgleft_0, &loc, screen, &dest, arImage);
		}
	}
	else if ((iThing == -1) && (iModifier == 2))
	{
		if (iDownAt == 2)
		{
			CustomBlitSurface (imgright_1, &loc, screen, &dest, arImage);
		} else {
			CustomBlitSurface (imgright_0, &loc, screen, &dest, arImage);
		}
	}
	else if ((iThing == -1) && (iModifier == 3))
	{
		if (iDownAt == 3)
		{
			CustomBlitSurface (imgup_1, &loc, screen, &dest, arImage);
		} else {
			CustomBlitSurface (imgup_0, &loc, screen, &dest, arImage);
		}
	}
	else if ((iThing == -1) && (iModifier == 4))
	{
		if (iDownAt == 4)
		{
			CustomBlitSurface (imgdown_1, &loc, screen, &dest, arImage);
		} else {
			CustomBlitSurface (imgdown_0, &loc, screen, &dest, arImage);
		}
	}
	else if ((iThing == -1) && (iModifier == 5))
		CustomBlitSurface (imgudno, &loc, screen, &dest, arImage);
	else if ((iThing == -1) && (iModifier == 6))
		CustomBlitSurface (imglrno, &loc, screen, &dest, arImage);
	else if ((iThing == -1) && (iModifier == 7))
		CustomBlitSurface (imgudnonfo, &loc, screen, &dest, arImage);

	/*** buttons ***/
	else if ((iThing == -2) && (iModifier == 1)) /*** rooms on ***/
	{
		if (iDownAt == 5)
		{
			CustomBlitSurface (imgroomson_1, &loc, screen, &dest, arImage);
		} else {
			CustomBlitSurface (imgroomson_0, &loc, screen, &dest, arImage);
		}
	}
	else if ((iThing == -2) && (iModifier == 8)) /*** rooms broken on ***/
	{
		if (iDownAt == 11)
		{
			CustomBlitSurface (imgbroomson_1, &loc, screen, &dest, arImage);
		} else {
			CustomBlitSurface (imgbroomson_0, &loc, screen, &dest, arImage);
		}
	}
	else if ((iThing == -2) && (iModifier == 6)) /*** rooms off ***/
	{
		CustomBlitSurface (imgroomsoff, &loc, screen, &dest, arImage);
	}
	else if ((iThing == -2) && (iModifier == 9)) /*** rooms broken off ***/
	{
		CustomBlitSurface (imgbroomsoff, &loc, screen, &dest, arImage);
	}
	else if ((iThing == -2) && (iModifier == 2)) /*** events on ***/
	{
		if (iDownAt == 6)
		{
			CustomBlitSurface (imgeventson_1, &loc, screen, &dest, arImage);
		} else {
			CustomBlitSurface (imgeventson_0, &loc, screen, &dest, arImage);
		}
	}
	else if ((iThing == -2) && (iModifier == 7)) /*** events off ***/
		CustomBlitSurface (imgeventsoff, &loc, screen, &dest, arImage);
	else if ((iThing == -2) && (iModifier == 3)) /*** save on ***/
	{
		if (iDownAt == 7)
		{
			CustomBlitSurface (imgsaveon_1, &loc, screen, &dest, arImage);
		} else {
			CustomBlitSurface (imgsaveon_0, &loc, screen, &dest, arImage);
		}
	}
	else if ((iThing == -2) && (iModifier == 4)) /*** quit ***/
	{
		if (iDownAt == 8)
		{
			CustomBlitSurface (imgquit_1, &loc, screen, &dest, arImage);
		} else {
			CustomBlitSurface (imgquit_0, &loc, screen, &dest, arImage);
		}
	}
	else if ((iThing == -2) && (iModifier == 5)) /*** save off ***/
		CustomBlitSurface (imgsaveoff, &loc, screen, &dest, arImage);

	/*** previous and next ***/
	else if ((iThing == -5) && (iModifier == 1))
	{
		if (iDownAt == 9)
		{
			CustomBlitSurface (imgprevon_1, &loc, screen, &dest, arImage);
		} else {
			CustomBlitSurface (imgprevon_0, &loc, screen, &dest, arImage);
		}
	}
	else if ((iThing == -5) && (iModifier == 2))
	{
		if (iDownAt == 10)
		{
			CustomBlitSurface (imgnexton_1, &loc, screen, &dest, arImage);
		} else {
			CustomBlitSurface (imgnexton_0, &loc, screen, &dest, arImage);
		}
	}
	else if ((iThing == -5) && (iModifier == 3))
		CustomBlitSurface (imgbar, &loc, screen, &dest, arImage);
	else if ((iThing == -5) && (iModifier == 4))
		CustomBlitSurface (imgprevoff, &loc, screen, &dest, arImage);
	else if ((iThing == -5) && (iModifier == 5))
		CustomBlitSurface (imgnextoff, &loc, screen, &dest, arImage);
	else if ((iThing == -5) && (iModifier == 10))
		CustomBlitSurface (extras_0, &loc, screen, &dest, arImage);
	else if ((iThing == -5) && (iModifier == 11))
		CustomBlitSurface (extras_1, &loc, screen, &dest, arImage);
	else if ((iThing == -5) && (iModifier == 12))
		CustomBlitSurface (extras_2, &loc, screen, &dest, arImage);
	else if ((iThing == -5) && (iModifier == 13))
		CustomBlitSurface (extras_3, &loc, screen, &dest, arImage);
	else if ((iThing == -5) && (iModifier == 14))
		CustomBlitSurface (extras_4, &loc, screen, &dest, arImage);
	else if ((iThing == -5) && (iModifier == 15))
		CustomBlitSurface (extras_5, &loc, screen, &dest, arImage);
	else if ((iThing == -5) && (iModifier == 16))
		CustomBlitSurface (extras_6, &loc, screen, &dest, arImage);
	else if ((iThing == -5) && (iModifier == 17))
		CustomBlitSurface (extras_7, &loc, screen, &dest, arImage);
	else if ((iThing == -5) && (iModifier == 18))
		CustomBlitSurface (extras_8, &loc, screen, &dest, arImage);

	/*** positions ***/
	else if (((iThing == 0) || (iThing == 32)) && (iModifier == 0))
		ShowPos (loc, dest, iLocation, imgd1_2, imgp1_2);
	else if (((iThing == 0) || (iThing == 32)) && (iModifier == 1))
		ShowPos (loc, dest, iLocation, imgd1_12, imgp1_12);
	else if (((iThing == 0) || (iThing == 32)) && (iModifier == 2))
		ShowPos (loc, dest, iLocation, imgd1_22, imgp1_22);
	else if (((iThing == 0) || (iThing == 32)) && (iModifier == 3))
		ShowPos (loc, dest, iLocation, imgd1_32, imgp1_32);
	else if (((iThing == 0) || (iThing == 32)) && (iModifier == 255))
		ShowPos (loc, dest, iLocation, imgd1_2551, imgp1_2551);
	else if (((iThing == 1) || (iThing == 33)) && (iModifier == 0))
		ShowPos (loc, dest, iLocation, imgd2_2, imgp2_2);
	else if (((iThing == 1) || (iThing == 33)) && (iModifier == 1))
		ShowPos (loc, dest, iLocation, imgd2_12, imgp2_12);
	else if (((iThing == 1) || (iThing == 33)) && (iModifier == 2))
		ShowPos (loc, dest, iLocation, imgd2_22, imgp2_22);
	else if (((iThing == 1) || (iThing == 33)) && (iModifier == 3))
		ShowPos (loc, dest, iLocation, imgd2_32, imgp2_32);
	else if (((iThing == 1) || (iThing == 33)) && (iModifier == 255))
		ShowPos (loc, dest, iLocation, imgd2_2551, imgp2_2551);
	else if (((iThing == 2) || (iThing == 34)) && (iModifier == 0))
		ShowPos (loc, dest, iLocation, imgd3_6, imgp3_6);
	else if (((iThing == 2) || (iThing == 34)) && (iModifier == 1))
		ShowPos (loc, dest, iLocation, imgd3_15, imgp3_15);
	else if (((iThing == 2) || (iThing == 34)) && (iModifier == 2))
		ShowPos (loc, dest, iLocation, imgd3_24, imgp3_24);
	else if (((iThing == 2) || (iThing == 34)) && (iModifier == 3))
		ShowPos (loc, dest, iLocation, imgd3_34, imgp3_34);
	else if (((iThing == 2) || (iThing == 34)) && (iModifier == 4))
		ShowPos (loc, dest, iLocation, imgd3_43, imgp3_43);
	else if (((iThing == 2) || (iThing == 34)) && (iModifier == 5))
		ShowPos (loc, dest, iLocation, imgd3_53, imgp3_53);
	else if (((iThing == 2) || (iThing == 34)) && (iModifier == 6))
		ShowPos (loc, dest, iLocation, imgd3_62, imgp3_62);
	else if (((iThing == 2) || (iThing == 34)) && (iModifier == 7))
		ShowPos (loc, dest, iLocation, imgd3_71, imgp3_71);
	else if (((iThing == 2) || (iThing == 34)) && (iModifier == 8))
		ShowPos (loc, dest, iLocation, imgd3_81, imgp3_81);
	else if (((iThing == 2) || (iThing == 34)) && (iModifier == 9))
		ShowPos (loc, dest, iLocation, imgd3_91, imgp3_91);
	else if (((iThing == 3) || (iThing == 35)) && (iModifier == 0))
		ShowPos (loc, dest, iLocation, imgd4_4, imgp4_4);
	else if (((iThing == 4) || (iThing == 36)) && (iModifier == 1))
		ShowPos (loc, dest, iLocation, imgd5_1, imgp5_1);
	/*** 0 = default for level editors, 2 = default of the game ***/
	else if (((iThing == 4) || (iThing == 36)) &&
		((iModifier == 2) || (iModifier == 0)))
		ShowPos (loc, dest, iLocation, imgd5_0, imgp5_0);
	else if (((iThing == 5) || (iThing == 37)) && (iModifier == 0))
		ShowPos (loc, dest, iLocation, imgd6_4, imgp6_4);
	else if ((iThing == 6) || (iThing == 38)) /*** drop button ***/
	{
		if (iCurType == 1)
		{
			CustomBlitSurface (imgd7[1], &loc, screen, &dest, arImage);
		} else {
			CustomBlitSurface (imgp7[1], &loc, screen, &dest, arImage);
		}
		snprintf (sText, MAX_TEXT, "E:%i", iModifier + 1);
		message = TTF_RenderText_Shaded (font2, sText, color_bl, color_wh);
		offset.x = dest.x + 7;
		offset.y = dest.y + 145 - FONT_SIZE_2;
		CustomBlitSurface (message, NULL, screen, &offset, arFont);
		if (iSelected == iLocation)
		{
			if (iCurType == 1)
			{
				CustomBlitSurface (imgd7[2], &loc, screen, &dest, arImage);
			} else {
				CustomBlitSurface (imgp7[2], &loc, screen, &dest, arImage);
			}
		}
	}
	else if (((iThing == 7) || (iThing == 39)) && (iModifier == 0))
		ShowPos (loc, dest, iLocation, imgd8_7, imgp8_7);
	else if (((iThing == 7) || (iThing == 39)) && (iModifier == 1))
		ShowPos (loc, dest, iLocation, imgd8_16, imgp8_16);
	else if (((iThing == 7) || (iThing == 39)) && (iModifier == 2))
		ShowPos (loc, dest, iLocation, imgd8_25, imgp8_25);
	else if (((iThing == 7) || (iThing == 39)) && (iModifier == 3))
		ShowPos (loc, dest, iLocation, imgd8_35, imgp8_35);
	else if (((iThing == 8) || (iThing == 40)) && (iModifier == 0))
		ShowPos (loc, dest, iLocation, imgd9_4, imgp9_4);
	else if (((iThing == 9) || (iThing == 41)) && (iModifier == 0))
		ShowPos (loc, dest, iLocation, imgd10_4, imgp10_4);
	else if (((iThing == 10) || (iThing == 42)) && (iModifier == 0))
		ShowPos (loc, dest, iLocation, imgd11_5, imgp11_5);
	else if (((iThing == 10) || (iThing == 42)) && (iModifier == 1))
		ShowPos (loc, dest, iLocation, imgd11_14, imgp11_14);		
	else if (((iThing == 10) || (iThing == 42)) && (iModifier == 2))
		ShowPos (loc, dest, iLocation, imgd11_23, imgp11_23);
	else if (((iThing == 10) || (iThing == 42)) && (iModifier == 3))
		ShowPos (loc, dest, iLocation, imgd11_33, imgp11_33);
	else if (((iThing == 10) || (iThing == 42)) && (iModifier == 4))
		ShowPos (loc, dest, iLocation, imgd11_42, imgp11_42);
	else if (((iThing == 10) || (iThing == 42)) && (iModifier == 5))
		ShowPos (loc, dest, iLocation, imgd11_52, imgp11_52);
	else if (((iThing == 10) || (iThing == 42)) && (iModifier == 6))
		ShowPos (loc, dest, iLocation, imgd11_61, imgp11_61);
	else if ((iThing == 11) && (iModifier == 0)) /*** loose tile ***/
		ShowPos (loc, dest, iLocation, imgd12_4, imgp12_4);
	else if (((iThing == 12) || (iThing == 44)) && (iModifier == 0))
		ShowPos (loc, dest, iLocation, imgd13_8, imgp13_8);
	else if (((iThing == 12) || (iThing == 44)) && (iModifier == 1))
		ShowPos (loc, dest, iLocation, imgd13_17, imgp13_17);
	else if (((iThing == 12) || (iThing == 44)) && (iModifier == 2))
		ShowPos (loc, dest, iLocation, imgd13_26, imgp13_26);
	else if (((iThing == 12) || (iThing == 44)) && (iModifier == 3))
		ShowPos (loc, dest, iLocation, imgd13_36, imgp13_36);
	else if (((iThing == 12) || (iThing == 44)) && (iModifier == 4))
		ShowPos (loc, dest, iLocation, imgd13_44, imgp13_44);
	else if (((iThing == 12) || (iThing == 44)) && (iModifier == 5))
		ShowPos (loc, dest, iLocation, imgd13_54, imgp13_54);
	else if (((iThing == 12) || (iThing == 44)) && (iModifier == 6))
		ShowPos (loc, dest, iLocation, imgd13_63, imgp13_63);
	else if (((iThing == 12) || (iThing == 44)) && (iModifier == 7))
		ShowPos (loc, dest, iLocation, imgd13_36, imgp13_72);
	else if (((iThing == 13) || (iThing == 45)) && (iModifier == 0))
		ShowPos (loc, dest, iLocation, imgd14_4, imgp14_4);
	else if (((iThing == 14) || (iThing == 46)) && (iModifier == 0))
		ShowPos (loc, dest, iLocation, imgd15_4, imgp15_4);
	else if ((iThing == 15) || (iThing == 47)) /*** raise button ***/
	{
		if (iCurType == 1)
		{
			CustomBlitSurface (imgd16[1], &loc, screen, &dest, arImage);
		} else {
			CustomBlitSurface (imgp16[1], &loc, screen, &dest, arImage);
		}
		snprintf (sText, MAX_TEXT, "E:%i", iModifier + 1);
		message = TTF_RenderText_Shaded (font2, sText, color_bl, color_wh);
		offset.x = dest.x + 7;
		offset.y = dest.y + 143 - FONT_SIZE_2;
		CustomBlitSurface (message, NULL, screen, &offset, arFont);
		if (iSelected == iLocation)
		{
			if (iCurType == 1)
			{
				CustomBlitSurface (imgd16[2], &loc, screen, &dest, arImage);
			} else {
				CustomBlitSurface (imgp16[2], &loc, screen, &dest, arImage);
			}
		}
	}
	else if (((iThing == 16) || (iThing == 48)) && (iModifier == 0))
		ShowPos (loc, dest, iLocation, imgd17_4, imgp17_4);
	else if (((iThing == 17) || (iThing == 49)) && (iModifier == 0))
		ShowPos (loc, dest, iLocation, imgd18_4, imgp18_4);
	else if (((iThing == 18) || (iThing == 50)) && (iModifier == 0))
		ShowPos (loc, dest, iLocation, imgd19_1, imgp19_1);
	else if (((iThing == 18) || (iThing == 50)) && (iModifier == 1))
		ShowPos (loc, dest, iLocation, imgd19_11, imgp19_11);
	else if (((iThing == 18) || (iThing == 50)) && (iModifier == 2))
		ShowPos (loc, dest, iLocation, imgd19_21, imgp19_21);
	else if (((iThing == 18) || (iThing == 50)) && (iModifier == 3))
		ShowPos (loc, dest, iLocation, imgd19_31, imgp19_31);
	else if (((iThing == 18) || (iThing == 50)) && (iModifier == 4))
		ShowPos (loc, dest, iLocation, imgd19_41, imgp19_41);
	else if (((iThing == 18) || (iThing == 50)) && (iModifier == 5))
		ShowPos (loc, dest, iLocation, imgd19_51, imgp19_51);
	else if (((iThing == 19) || (iThing == 51)) && (iModifier == 0))
		ShowPos (loc, dest, iLocation, imgd20_4, imgp20_4);
	else if (((iThing == 20) || (iThing == 52)) && (iModifier == 0))
		ShowPos (loc, dest, iLocation, imgd21_9, imgp21_9);
	else if (((iThing == 20) || (iThing == 52)) && (iModifier == 1))
		ShowPos (loc, dest, iLocation, imgd21_18, imgp21_18);
	else if (((iThing == 21) || (iThing == 53)) && (iModifier == 0))
		ShowPos (loc, dest, iLocation, imgd22_4, imgp22_4);
	else if (((iThing == 22) || (iThing == 54)) && (iModifier == 0))
		ShowPos (loc, dest, iLocation, imgd23_4, imgp23_4);
	else if (((iThing == 23) || (iThing == 55)) && (iModifier == 0))
		ShowPos (loc, dest, iLocation, imgd24_4, imgp24_4);
	else if (((iThing == 24) || (iThing == 56)) && (iModifier == 0))
		ShowPos (loc, dest, iLocation, imgd25_4, imgp25_4);
	else if (((iThing == 25) || (iThing == 57)) && (iModifier == 0))
		ShowPos (loc, dest, iLocation, imgd26_4, imgp26_4);
	else if (((iThing == 26) || (iThing == 58)) && (iModifier == 0))
		ShowPos (loc, dest, iLocation, imgd27_4, imgp27_4);
	else if (((iThing == 27) || (iThing == 59)) && (iModifier == 0))
		ShowPos (loc, dest, iLocation, imgd28_4, imgp28_4);
	else if (((iThing == 28) || (iThing == 60)) && (iModifier == 0))
		ShowPos (loc, dest, iLocation, imgd29_4, imgp29_4);
	else if (((iThing == 29) || (iThing == 61)) && (iModifier == 0))
		ShowPos (loc, dest, iLocation, imgd30_4, imgp30_4);
	else if (((iThing == 30) || (iThing == 62)) && (iModifier == 0))
		ShowPos (loc, dest, iLocation, imgd31_4, imgp31_4);
	else if ((iThing == 43) && (iModifier == 0)) /*** loose tile, stuck ***/
		ShowPos (loc, dest, iLocation, imgd44_4, imgp44_4);

	else if ((iThing == -8) && (iModifier == 1)) /*** kid ***/
	{
		dest.y = dest.y + 51;
		switch (luKidDir)
		{
			case 0: dest.x = dest.x + 74;
				CustomBlitSurface (imgkidr[1], &loc, screen, &dest, arImage);
				if (iSelected == iLocation)
				{ CustomBlitSurface (imgkidr[2], &loc, screen, &dest, arImage); }
				break;
			case 255: dest.x = dest.x + 36;
				CustomBlitSurface (imgkidl[1], &loc, screen, &dest, arImage);
				if (iSelected == iLocation)
				{ CustomBlitSurface (imgkidl[2], &loc, screen, &dest, arImage); }
				break;
			default: printf ("[FAILED] Wrong dir.!\n"); exit (EXIT_ERROR); break;
		}
	}
	else if ((iThing == -8) && (iModifier == 2)) /*** guard ***/
	{
		iTemp = dest.x;
		iTemp2 = dest.y;
		if (((int)luLevelNr == 3) || ((int)luLevelNr == 12))
		{
			dest.y = dest.y + 63; /*** skeleton + shadow ***/
		} else if ((int)luLevelNr == 13) {
			dest.y = dest.y + 43; /*** jaffar ***/
		} else {
			dest.y = dest.y + 51; /*** normal guards + fat ***/
		}
		if (sGuardDirections[iCurRoom - 1] == 255) /*** looking to the left ***/
		{
			if (((int)luLevelNr == 3) || ((int)luLevelNr == 12))
			{
				dest.x = dest.x + 34; /*** skeleton + shadow ***/
			} else {
				dest.x = dest.x + 54; /*** normal guards + fat + jaffar ***/
			}
			switch ((int)luLevelNr)
			{
				case 0:
				case 1:
				case 2:
				case 4:
				case 5:
				case 7:
				case 8:
				case 9:
				case 10:
				case 11:
					switch (sGuardColors[iCurRoom - 1] - 1)
					{
						case 0: CustomBlitSurface (imgglp[1], &loc,
							screen, &dest, arImage); break;
						case 1: CustomBlitSurface (imggrp[1], &loc,
							screen, &dest, arImage); break;
						case 2: CustomBlitSurface (imggoy[1], &loc,
							screen, &dest, arImage); break;
						case 3: CustomBlitSurface (imgggy[1], &loc,
							screen, &dest, arImage); break;
						case 4: CustomBlitSurface (imggdb[1], &loc,
							screen, &dest, arImage); break;
						case 5: CustomBlitSurface (imggpb[1], &loc,
							screen, &dest, arImage); break;
						case 6: CustomBlitSurface (imggyo[1], &loc,
							screen, &dest, arImage); break;
						default:
							CustomBlitSurface (imggyo[1], &loc, screen, &dest, arImage);
							if (iDebug == 1)
							{
								printf ("[ INFO ] Strange guard color: %i (jaffar, shadow, "
									"fat, skeleton?)\n", sGuardColors[iCurRoom - 1] - 1);
							}
							break;
					}
					break;
				case 3:
					CustomBlitSurface (imgskel[1], &loc, screen, &dest, arImage); break;
				case 6:
					CustomBlitSurface (imgfat[1], &loc, screen, &dest, arImage); break;
				case 13: CustomBlitSurface (imgjaffar[1], &loc,
					screen, &dest, arImage); break;
				case 12: CustomBlitSurface (imgshadow[1], &loc,
					screen, &dest, arImage); break;
			}
		} else {
			if (((int)luLevelNr == 3) || ((int)luLevelNr == 12))
			{
				dest.x = dest.x + 8; /*** skeleton + shadow ***/
			} else if ((int)luLevelNr == 13) {
				dest.x = dest.x + 20; /*** jaffar ***/
			} else {
				dest.x = dest.x + 6; /*** normal guards + fat ***/
			}
			switch ((int)luLevelNr)
			{
				case 0:
				case 1:
				case 2:
				case 4:
				case 5:
				case 7:
				case 8:
				case 9:
				case 10:
				case 11:
					switch (sGuardColors[iCurRoom - 1] - 1)
					{
						case 0: CustomBlitSurface (imgglp[2], &loc,
							screen, &dest, arImage); break;
						case 1: CustomBlitSurface (imggrp[2], &loc,
							screen, &dest, arImage); break;
						case 2: CustomBlitSurface (imggoy[2], &loc,
							screen, &dest, arImage); break;
						case 3: CustomBlitSurface (imgggy[2], &loc,
							screen, &dest, arImage); break;
						case 4: CustomBlitSurface (imggdb[2], &loc,
							screen, &dest, arImage); break;
						case 5: CustomBlitSurface (imggpb[2], &loc,
							screen, &dest, arImage); break;
						case 6: CustomBlitSurface (imggyo[2], &loc,
							screen, &dest, arImage); break;
						default:
							CustomBlitSurface (imggyo[2], &loc, screen, &dest, arImage);
							if (iDebug == 1)
							{
								printf ("[ INFO ] Strange guard color: %i (jaffar, shadow, "
									"fat, skeleton?)\n", sGuardColors[iCurRoom - 1] - 1);
							}
							break;
					}
					break;
				case 3:
					CustomBlitSurface (imgskel[2], &loc, screen, &dest, arImage); break;
				case 6:
					CustomBlitSurface (imgfat[2], &loc, screen, &dest, arImage); break;
				case 13: CustomBlitSurface (imgjaffar[2], &loc,
					screen, &dest, arImage); break;
				case 12: CustomBlitSurface (imgshadow[2], &loc,
					screen, &dest, arImage); break;
			}
		}
		snprintf (sText, MAX_TEXT, "LVL:%i", sGuardSkills[iCurRoom - 1]);
		message = TTF_RenderText_Shaded (font2, sText, color_bl, color_wh);
		offset.x = iTemp + 7;
		offset.y = iTemp2 + 145 - FONT_SIZE_2;
		CustomBlitSurface (message, NULL, screen, &offset, arFont);
		if (iSelected == iLocation)
		{
			if (sGuardDirections[iCurRoom - 1] == 255) /*** looking to the left ***/
			{
				if ((int)luLevelNr == 3)
				{ CustomBlitSurface (imgskelsel[1], &loc, screen, &dest, arImage); }
				else if ((int)luLevelNr == 6)
				{ CustomBlitSurface (imgfatsel[1], &loc, screen, &dest, arImage); }
				else if ((int)luLevelNr == 13)
				{ CustomBlitSurface (imgjaffarsel[1], &loc, screen, &dest, arImage); }
				else if ((int)luLevelNr == 12)
				{ CustomBlitSurface (imgshadowsel[1], &loc, screen, &dest, arImage); }
				else if (((int)luLevelNr != 14) && ((int)luLevelNr != 15))
				{ CustomBlitSurface (imggsel[1], &loc, screen, &dest, arImage); }
			} else {
				if ((int)luLevelNr == 3)
				{ CustomBlitSurface (imgskelsel[2], &loc, screen, &dest, arImage); }
				else if ((int)luLevelNr == 6)
				{ CustomBlitSurface (imgfatsel[2], &loc, screen, &dest, arImage); }
				else if ((int)luLevelNr == 13)
				{ CustomBlitSurface (imgjaffarsel[2], &loc, screen, &dest, arImage); }
				else if ((int)luLevelNr == 12)
				{ CustomBlitSurface (imgshadowsel[2], &loc, screen, &dest, arImage); }
				else if (((int)luLevelNr != 14) && ((int)luLevelNr != 15))
				{ CustomBlitSurface (imggsel[2], &loc, screen, &dest, arImage); }
			}
		}
	}

	else if ((iThing == -9) && (iModifier == 1)) /*** faded background ***/
		CustomBlitSurface (imgfadedl, &loc, screen, &dest, arImage);
	else if ((iThing == -9) && (iModifier == 2)) /*** popup ***/
		CustomBlitSurface (imgpopup, &loc, screen, &dest, arImage);
	else if ((iThing == -9) && (iModifier == 3)) /*** OK off ***/
		CustomBlitSurface (imgok[1], &loc, screen, &dest, arImage);
	else if ((iThing == -9) && (iModifier == 4)) /*** OK on ***/
		CustomBlitSurface (imgok[2], &loc, screen, &dest, arImage);
	else if ((iThing == -9) && (iModifier == 5)) /*** popup_yn ***/
		CustomBlitSurface (imgpopup_yn, &loc, screen, &dest, arImage);
	else if ((iThing == -9) && (iModifier == 6)) /*** Yes off ***/
		CustomBlitSurface (imgyes[1], &loc, screen, &dest, arImage);
	else if ((iThing == -9) && (iModifier == 7)) /*** Yes on ***/
		CustomBlitSurface (imgyes[2], &loc, screen, &dest, arImage);
	else if ((iThing == -9) && (iModifier == 8)) /*** No off ***/
		CustomBlitSurface (imgno[1], &loc, screen, &dest, arImage);
	else if ((iThing == -9) && (iModifier == 9)) /*** No on ***/
		CustomBlitSurface (imgno[2], &loc, screen, &dest, arImage);
	else if ((iThing == -9) && (iModifier == 10)) /*** help ***/
		CustomBlitSurface (imghelp, &loc, screen, &dest, arImage);

	else /*** Custom tile. ***/
	{
		ShowPos (loc, dest, iLocation, imgunk, imgunk);
		CustomTile (iThing, iModifier, sCustomTile);
		snprintf (sText, MAX_TEXT, "%s", sCustomTile);
		message = TTF_RenderText_Shaded (font2, sText, color_bl, color_wh);
		offset.x = dest.x;
		offset.y = dest.y + 145 - FONT_SIZE_2;
		CustomBlitSurface (message, NULL, screen, &offset, arFont);
	}
}
/*****************************************************************************/
int InArea (SDL_Event event, int iUpperLeftX, int iUpperLeftY, int
	iLowerRightX, int iLowerRightY)
/*****************************************************************************/
{
	if ((iUpperLeftX * iScale <= event.motion.x) &&
		(iLowerRightX * iScale >= event.motion.x) &&
		(iUpperLeftY * iScale <= event.motion.y) &&
		(iLowerRightY * iScale >= event.motion.y))
	{
		return (1);
	} else {
		return (0);
	}
}
/*****************************************************************************/
void InitScreen (void)
/*****************************************************************************/
{
	SDL_Event event;
	SDL_Surface *loaded_img;
	SDL_Surface *imgicon;
	SDL_AudioSpec fmt;
	int iTemp2;
	int iEventRoom;
	int iEventDoor;
	int iOldXPos, iOldYPos;
	Uint8 *keystate;

	/*** The following are used for changing bits. ***/
	int iTemp;

	if (SDL_Init (SDL_INIT_AUDIO|SDL_INIT_VIDEO) < 0)
	{
		printf ("[FAILED] Unable to init SDL: %s!\n", SDL_GetError());
		exit (EXIT_ERROR);
	}
	atexit(SDL_Quit);

	PUTENV ("SDL_VIDEO_CENTERED=1");
	screen = SDL_SetVideoMode ((SCREEN_WIDTH) * iScale, (SCREEN_HEIGHT) * iScale,
		SCREEN_BPP, SDL_DOUBLEBUF|SDL_SWSURFACE);
	if ( screen == NULL )
	{
		printf ("[FAILED] Unable to set video mode: %s!\n", SDL_GetError());
		exit (EXIT_ERROR);
	}

	SDL_EnableKeyRepeat (SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);

	if (TTF_Init() == -1)
	{
		exit (EXIT_ERROR);
	}

	LoadFonts();

	if (iNoAudio != 1)
	{
		PrIfDe ("[  OK  ] Initializing Audio\n");
		fmt.freq = 22050;
		fmt.format = AUDIO_S16;
		fmt.channels = 2;
		fmt.samples = 512;
		fmt.callback = MixAudio;
		fmt.userdata = NULL;
		if (SDL_OpenAudio (&fmt, NULL) < 0)
		{
			printf ("[FAILED] Unable to open audio: %s!\n", SDL_GetError());
			exit (EXIT_ERROR);
		}
		SDL_PauseAudio (0);
	}

	/*** caption, icon, loading ***/
	loaded_img = IMG_Load ("png/apoplexy_icon.png");
	imgicon = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/black.png");
	imgblack = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/hourglass.png");
	imghourglass = SDL_DisplayFormatAlpha (loaded_img);
	SDL_WM_SetCaption (EDITOR_NAME " " EDITOR_VERSION, NULL);
	SDL_WM_SetIcon (imgicon, NULL);

	ShowImage (-4, 1, screen, 31, 0, 0, 692, 430); /*** black background ***/
	ShowImage (-4, 2, screen, 46, 0, 0, 93, 89); /*** hourglass ***/

	/*** refresh screen ***/
	if (SDL_Flip (screen) == -1)
		{ printf ("[FAILED] SDL_Flip error!\n"); exit (EXIT_ERROR); }

	/*****************************/
	/* Preload all other images. */
	/*****************************/

	/*** locations ***/
	loaded_img = IMG_Load ("png/d_1_2.png");
	imgd1_2[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_1_2.png");
	imgd1_2[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_1_2.png");
	imgp1_2[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_1_2.png");
	imgp1_2[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_1_12.png");
	imgd1_12[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_1_12.png");
	imgd1_12[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_1_12.png");
	imgp1_12[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_1_12.png");
	imgp1_12[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_1_22.png");
	imgd1_22[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_1_22.png");
	imgd1_22[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_1_22.png");
	imgp1_22[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_1_22.png");
	imgp1_22[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_1_32.png");
	imgd1_32[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_1_32.png");
	imgd1_32[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_1_32.png");
	imgp1_32[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_1_32.png");
	imgp1_32[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_1_2551.png");
	imgd1_2551[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_1_2551.png");
	imgd1_2551[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_1_2551.png");
	imgp1_2551[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_1_2551.png");
	imgp1_2551[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_2_2.png");
	imgd2_2[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_2_2.png");
	imgd2_2[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_2_2.png");
	imgp2_2[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_2_2.png");
	imgp2_2[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_2_12.png");
	imgd2_12[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_2_12.png");
	imgd2_12[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_2_12.png");
	imgp2_12[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_2_12.png");
	imgp2_12[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_2_22.png");
	imgd2_22[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_2_22.png");
	imgd2_22[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_2_22.png");
	imgp2_22[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_2_22.png");
	imgp2_22[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_2_32.png");
	imgd2_32[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_2_32.png");
	imgd2_32[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_2_32.png");
	imgp2_32[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_2_32.png");
	imgp2_32[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_2_2551.png");
	imgd2_2551[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_2_2551.png");
	imgd2_2551[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_2_2551.png");
	imgp2_2551[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_2_2551.png");
	imgp2_2551[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_3_6.png");
	imgd3_6[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_3_6.png");
	imgd3_6[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_3_6.png");
	imgp3_6[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_3_6.png");
	imgp3_6[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_3_15.png");
	imgd3_15[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_3_15.png");
	imgd3_15[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_3_15.png");
	imgp3_15[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_3_15.png");
	imgp3_15[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_3_24.png");
	imgd3_24[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_3_24.png");
	imgd3_24[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_3_24.png");
	imgp3_24[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_3_24.png");
	imgp3_24[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_3_34.png");
	imgd3_34[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_3_34.png");
	imgd3_34[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_3_34.png");
	imgp3_34[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_3_34.png");
	imgp3_34[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_3_43.png");
	imgd3_43[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_3_43.png");
	imgd3_43[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_3_43.png");
	imgp3_43[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_3_43.png");
	imgp3_43[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_3_53.png");
	imgd3_53[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_3_53.png");
	imgd3_53[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_3_53.png");
	imgp3_53[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_3_53.png");
	imgp3_53[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_3_62.png");
	imgd3_62[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_3_62.png");
	imgd3_62[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_3_62.png");
	imgp3_62[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_3_62.png");
	imgp3_62[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_3_71.png");
	imgd3_71[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_3_71.png");
	imgd3_71[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_3_71.png");
	imgp3_71[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_3_71.png");
	imgp3_71[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_3_81.png");
	imgd3_81[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_3_81.png");
	imgd3_81[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_3_81.png");
	imgp3_81[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_3_81.png");
	imgp3_81[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_3_91.png");
	imgd3_91[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_3_91.png");
	imgd3_91[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_3_91.png");
	imgp3_91[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_3_91.png");
	imgp3_91[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_4_4.png");
	imgd4_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_4_4.png");
	imgd4_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_4_4.png");
	imgp4_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_4_4.png");
	imgp4_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_5_0.png");
	imgd5_0[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_5_0.png");
	imgd5_0[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_5_0.png");
	imgp5_0[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_5_0.png");
	imgp5_0[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_5_1.png");
	imgd5_1[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_5_1.png");
	imgd5_1[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_5_1.png");
	imgp5_1[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_5_1.png");
	imgp5_1[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_6_4.png");
	imgd6_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_6_4.png");
	imgd6_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_6_4.png");
	imgp6_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_6_4.png");
	imgp6_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_7.png");
	imgd7[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_7.png");
	imgd7[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_7.png");
	imgp7[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_7.png");
	imgp7[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_8_7.png");
	imgd8_7[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_8_7.png");
	imgd8_7[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_8_7.png");
	imgp8_7[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_8_7.png");
	imgp8_7[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_8_16.png");
	imgd8_16[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_8_16.png");
	imgd8_16[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_8_16.png");
	imgp8_16[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_8_16.png");
	imgp8_16[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_8_25.png");
	imgd8_25[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_8_25.png");
	imgd8_25[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_8_25.png");
	imgp8_25[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_8_25.png");
	imgp8_25[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_8_35.png");
	imgd8_35[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_8_35.png");
	imgd8_35[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_8_35.png");
	imgp8_35[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_8_35.png");
	imgp8_35[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_9_4.png");
	imgd9_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_9_4.png");
	imgd9_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_9_4.png");
	imgp9_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_9_4.png");
	imgp9_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_10_4.png");
	imgd10_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_10_4.png");
	imgd10_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_10_4.png");
	imgp10_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_10_4.png");
	imgp10_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_11_5.png");
	imgd11_5[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_11_5.png");
	imgd11_5[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_11_5.png");
	imgp11_5[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_11_5.png");
	imgp11_5[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_11_14.png");
	imgd11_14[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_11_14.png");
	imgd11_14[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_11_14.png");
	imgp11_14[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_11_14.png");
	imgp11_14[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_11_23.png");
	imgd11_23[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_11_23.png");
	imgd11_23[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_11_23.png");
	imgp11_23[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_11_23.png");
	imgp11_23[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_11_33.png");
	imgd11_33[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_11_33.png");
	imgd11_33[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_11_33.png");
	imgp11_33[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_11_33.png");
	imgp11_33[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_11_42.png");
	imgd11_42[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_11_42.png");
	imgd11_42[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_11_42.png");
	imgp11_42[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_11_42.png");
	imgp11_42[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_11_52.png");
	imgd11_52[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_11_52.png");
	imgd11_52[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_11_52.png");
	imgp11_52[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_11_52.png");
	imgp11_52[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_11_61.png");
	imgd11_61[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_11_61.png");
	imgd11_61[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_11_61.png");
	imgp11_61[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_11_61.png");
	imgp11_61[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_12_4.png");
	imgd12_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_12_4.png");
	imgd12_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_12_4.png");
	imgp12_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_12_4.png");
	imgp12_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_13_8.png");
	imgd13_8[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_13_8.png");
	imgd13_8[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_13_8.png");
	imgp13_8[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_13_8.png");
	imgp13_8[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_13_17.png");
	imgd13_17[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_13_17.png");
	imgd13_17[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_13_17.png");
	imgp13_17[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_13_17.png");
	imgp13_17[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_13_26.png");
	imgd13_26[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_13_26.png");
	imgd13_26[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_13_26.png");
	imgp13_26[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_13_26.png");
	imgp13_26[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_13_36.png");
	imgd13_36[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_13_36.png");
	imgd13_36[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_13_36.png");
	imgp13_36[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_13_36.png");
	imgp13_36[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_13_44.png");
	imgd13_44[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_13_44.png");
	imgd13_44[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_13_44.png");
	imgp13_44[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_13_44.png");
	imgp13_44[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_13_54.png");
	imgd13_54[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_13_54.png");
	imgd13_54[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_13_54.png");
	imgp13_54[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_13_54.png");
	imgp13_54[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_13_63.png");
	imgd13_63[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_13_63.png");
	imgd13_63[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_13_63.png");
	imgp13_63[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_13_63.png");
	imgp13_63[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_13_72.png");
	imgd13_72[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_13_72.png");
	imgd13_72[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_13_72.png");
	imgp13_72[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_13_72.png");
	imgp13_72[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_14_4.png");
	imgd14_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_14_4.png");
	imgd14_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_14_4.png");
	imgp14_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_14_4.png");
	imgp14_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_15_4.png");
	imgd15_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_15_4.png");
	imgd15_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_15_4.png");
	imgp15_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_15_4.png");
	imgp15_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_16.png");
	imgd16[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_16.png");
	imgd16[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_16.png");
	imgp16[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_16.png");
	imgp16[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_17_4.png");
	imgd17_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_17_4.png");
	imgd17_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_17_4.png");
	imgp17_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_17_4.png");
	imgp17_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_18_4.png");
	imgd18_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_18_4.png");
	imgd18_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_18_4.png");
	imgp18_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_18_4.png");
	imgp18_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_19_1.png");
	imgd19_1[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_19_1.png");
	imgd19_1[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_19_1.png");
	imgp19_1[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_19_1.png");
	imgp19_1[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_19_11.png");
	imgd19_11[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_19_11.png");
	imgd19_11[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_19_11.png");
	imgp19_11[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_19_11.png");
	imgp19_11[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_19_21.png");
	imgd19_21[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_19_21.png");
	imgd19_21[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_19_21.png");
	imgp19_21[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_19_21.png");
	imgp19_21[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_19_31.png");
	imgd19_31[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_19_31.png");
	imgd19_31[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_19_31.png");
	imgp19_31[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_19_31.png");
	imgp19_31[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_19_41.png");
	imgd19_41[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_19_41.png");
	imgd19_41[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_19_41.png");
	imgp19_41[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_19_41.png");
	imgp19_41[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_19_51.png");
	imgd19_51[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_19_51.png");
	imgd19_51[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_19_51.png");
	imgp19_51[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_19_51.png");
	imgp19_51[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_20_4.png");
	imgd20_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_20_4.png");
	imgd20_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_20_4.png");
	imgp20_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_20_4.png");
	imgp20_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_21_9.png");
	imgd21_9[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_21_9.png");
	imgd21_9[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_21_9.png");
	imgp21_9[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_21_9.png");
	imgp21_9[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_21_18.png");
	imgd21_18[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_21_18.png");
	imgd21_18[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_21_18.png");
	imgp21_18[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_21_18.png");
	imgp21_18[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_22_4.png");
	imgd22_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_22_4.png");
	imgd22_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_22_4.png");
	imgp22_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_22_4.png");
	imgp22_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_23_4.png");
	imgd23_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_23_4.png");
	imgd23_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_23_4.png");
	imgp23_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_23_4.png");
	imgp23_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_24_4.png");
	imgd24_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_24_4.png");
	imgd24_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_24_4.png");
	imgp24_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_24_4.png");
	imgp24_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_25_4.png");
	imgd25_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_25_4.png");
	imgd25_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_25_4.png");
	imgp25_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_25_4.png");
	imgp25_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_26_4.png");
	imgd26_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_26_4.png");
	imgd26_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_26_4.png");
	imgp26_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_26_4.png");
	imgp26_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_27_4.png");
	imgd27_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_27_4.png");
	imgd27_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_27_4.png");
	imgp27_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_27_4.png");
	imgp27_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_28_4.png");
	imgd28_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_28_4.png");
	imgd28_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_28_4.png");
	imgp28_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_28_4.png");
	imgp28_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_29_4.png");
	imgd29_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_29_4.png");
	imgd29_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_29_4.png");
	imgp29_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_29_4.png");
	imgp29_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_30_4.png");
	imgd30_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_30_4.png");
	imgd30_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_30_4.png");
	imgp30_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_30_4.png");
	imgp30_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_31_4.png");
	imgd31_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_31_4.png");
	imgd31_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_31_4.png");
	imgp31_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_31_4.png");
	imgp31_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_44_4.png");
	imgd44_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/d_sel_44_4.png");
	imgd44_4[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_44_4.png");
	imgp44_4[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/p_sel_44_4.png");
	imgp44_4[2] = SDL_DisplayFormatAlpha (loaded_img);

	/*** guards ***/
	loaded_img = IMG_Load ("png/guard_darkblue_beige_l.png");
	imggdb[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/guard_darkblue_beige_r.png");
	imggdb[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/guard_green_yellow_l.png");
	imgggy[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/guard_green_yellow_r.png");
	imgggy[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/guard_lightblue_pink_l.png");
	imgglp[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/guard_lightblue_pink_r.png");
	imgglp[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/guard_orange_yellow_l.png");
	imggoy[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/guard_orange_yellow_r.png");
	imggoy[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/guard_purple_beige_l.png");
	imggpb[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/guard_purple_beige_r.png");
	imggpb[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/guard_red_purple_l.png");
	imggrp[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/guard_red_purple_r.png");
	imggrp[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/guard_yellow_orange_l.png");
	imggyo[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/guard_yellow_orange_r.png");
	imggyo[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/skeleton_left.png");
	imgskel[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/skeleton_right.png");
	imgskel[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/jaffar_left.png");
	imgjaffar[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/jaffar_right.png");
	imgjaffar[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/guard_fat_l.png");
	imgfat[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/guard_fat_r.png");
	imgfat[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/shadow_left.png");
	imgshadow[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/shadow_right.png");
	imgshadow[2] = SDL_DisplayFormatAlpha (loaded_img);

	/*** guards, selected ***/
	loaded_img = IMG_Load ("png/sel_guard_left.png");
	imggsel[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/sel_guard_right.png");
	imggsel[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/sel_skeleton_left.png");
	imgskelsel[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/sel_skeleton_right.png");
	imgskelsel[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/sel_jaffar_left.png");
	imgjaffarsel[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/sel_jaffar_right.png");
	imgjaffarsel[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/sel_guard_fat_l.png");
	imgfatsel[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/sel_guard_fat_r.png");
	imgfatsel[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/sel_shadow_left.png");
	imgshadowsel[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/sel_shadow_right.png");
	imgshadowsel[2] = SDL_DisplayFormatAlpha (loaded_img);

	/*** various ***/
	loaded_img = IMG_Load ("png/no_other.png");
	imgdother = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/no_jaffar.png");
	imgdjaffar = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/no_guards.png");
	imgdguards = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/unknown.png");
	imgunk[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/sel_unknown.png");
	imgunk[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/up_0.png");
	imgup_0 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/up_1.png");
	imgup_1 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/down_0.png");
	imgdown_0 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/down_1.png");
	imgdown_1 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/left_0.png");
	imgleft_0 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/left_1.png");
	imgleft_1 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/right_0.png");
	imgright_0 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/right_1.png");
	imgright_1 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/up_down_no.png");
	imgudno = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/left_right_no.png");
	imglrno = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/up_down_no_nfo.png");
	imgudnonfo = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/previous_on_0.png");
	imgprevon_0 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/previous_on_1.png");
	imgprevon_1 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/next_on_0.png");
	imgnexton_0 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/next_on_1.png");
	imgnexton_1 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/previous_off.png");
	imgprevoff = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/next_off.png");
	imgnextoff = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/level_bar.png");
	imgbar = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/extras_0.png");
	extras_0 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/extras_1.png");
	extras_1 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/extras_2.png");
	extras_2 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/extras_3.png");
	extras_3 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/extras_4.png");
	extras_4 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/extras_5.png");
	extras_5 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/extras_6.png");
	extras_6 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/extras_7.png");
	extras_7 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/extras_8.png");
	extras_8 = SDL_DisplayFormatAlpha (loaded_img);
	/***/
	loaded_img = IMG_Load ("png/rooms_on_0.png");
	imgroomson_0 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/rooms_on_1.png");
	imgroomson_1 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/rooms_off.png");
	imgroomsoff = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/broken_rooms_on_0.png");
	imgbroomson_0 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/broken_rooms_on_1.png");
	imgbroomson_1 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/broken_rooms_off.png");
	imgbroomsoff = SDL_DisplayFormatAlpha (loaded_img);
	/***/
	loaded_img = IMG_Load ("png/events_on_0.png");
	imgeventson_0 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/events_on_1.png");
	imgeventson_1 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/events_off.png");
	imgeventsoff = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/save_on_0.png");
	imgsaveon_0 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/save_on_1.png");
	imgsaveon_1 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/save_off.png");
	imgsaveoff = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/quit_0.png");
	imgquit_0 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/quit_1.png");
	imgquit_1 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/room_links.png");
	imgrl = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/broken_room_links.png");
	imgbrl = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/sel_room_current.png");
	imgsrc = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/sel_room_start.png");
	imgsrs = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/sel_room_moving.png");
	imgsrm = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/sel_room_cross.png");
	imgsrp = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/sel_room_broken.png");
	imgsrb = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/kid_right.png");
	imgkidr[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/sel_kid_right.png");
	imgkidr[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/kid_left.png");
	imgkidl[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/sel_kid_left.png");
	imgkidl[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/events.png");
	imgevents = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/sel_event.png");
	imgsele = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/sel_level.png");
	imgsell = SDL_DisplayFormatAlpha (loaded_img);

	/*** changing ***/
	loaded_img = IMG_Load ("png/dungeon.png");
	imgdungeon = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/palace.png");
	imgpalace = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/close_0.png");
	imgclose_0 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/close_1.png");
	imgclose_1 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/border_big.png");
	imgborderb = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/border_small.png");
	imgborders = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/border_small_live.png");
	imgbordersl = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/border_big_live.png");
	imgborderbl = SDL_DisplayFormatAlpha (loaded_img);
	/*** custom ***/
	loaded_img = IMG_Load ("png/custom_active_0.png");
	imgcustoma0 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/custom_inactive_0.png");
	imgcustomi0 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/custom_active_1.png");
	imgcustoma1 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/custom_inactive_1.png");
	imgcustomi1 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/custom.png");
	imgcustom = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/custom_0.png");
	imgcustom_0 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/custom_1.png");
	imgcustom_1 = SDL_DisplayFormatAlpha (loaded_img);

	/*** popup things ***/
	loaded_img = IMG_Load ("png/faded_l.png");
	imgfadedl = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/popup.png");
	imgpopup = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/OK.png");
	imgok[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/sel_OK.png");
	imgok[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/popup_yn.png");
	imgpopup_yn = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/Yes.png");
	imgyes[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/sel_Yes.png");
	imgyes[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/No.png");
	imgno[1] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/sel_No.png");
	imgno[2] = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/help.png");
	imghelp = SDL_DisplayFormatAlpha (loaded_img);

	/*** faded tiles ***/
	loaded_img = IMG_Load ("png/faded_s.png");
	imgfadeds = SDL_DisplayFormatAlpha (loaded_img);

	/*** room thingies, 14x14 ***/
	loaded_img = IMG_Load ("png/room1.png");
	imgroom1 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/room2.png");
	imgroom2 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/room3.png");
	imgroom3 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/room4.png");
	imgroom4 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/room5.png");
	imgroom5 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/room6.png");
	imgroom6 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/room7.png");
	imgroom7 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/room8.png");
	imgroom8 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/room9.png");
	imgroom9 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/room10.png");
	imgroom10 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/room11.png");
	imgroom11 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/room12.png");
	imgroom12 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/room13.png");
	imgroom13 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/room14.png");
	imgroom14 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/room15.png");
	imgroom15 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/room16.png");
	imgroom16 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/room17.png");
	imgroom17 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/room18.png");
	imgroom18 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/room19.png");
	imgroom19 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/room20.png");
	imgroom20 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/room21.png");
	imgroom21 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/room22.png");
	imgroom22 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/room23.png");
	imgroom23 = SDL_DisplayFormatAlpha (loaded_img);
	loaded_img = IMG_Load ("png/room24.png");
	imgroom24 = SDL_DisplayFormatAlpha (loaded_img);

	iDownAt = 0;
	iSelected = 1; /*** Start with the upper left selected. ***/
	iScreen = 1;
	iOnTile = 1;

	ShowScreen (iScreen);
	InitPopUp (1, 1);
	while (1)
	{
		while (SDL_PollEvent (&event))
		{
			switch (event.type)
			{
				case SDL_KEYDOWN: /*** http://wiki.libsdl.org/SDLKeycodeLookup ***/
					switch (event.key.keysym.sym)
					{
						case SDLK_F1: Help (iScreen); break;
						case SDLK_LEFTBRACKET:
							switch (iScreen)
							{
								case 2:
									if (iBrokenRoomLinks == 0)
									{
										iMovingNewBusy = 0;
										switch (iMovingRoom)
										{
											case 0: iMovingRoom = 24; break;
											case 1: iMovingRoom = 24; break;
											default: iMovingRoom--; break;
										}
									}
									break;
								case 3:
									iEventRoom = EventInfo (iChangeEvent, 1);
									if (iEventRoom > 1)
									{
										EventRoom (iEventRoom - 1);
									} else {
										EventRoom (24);
									}
									break;
							}
							break;
						case SDLK_RIGHTBRACKET:
							switch (iScreen)
							{
								case 2:
									if (iBrokenRoomLinks == 0)
									{
										iMovingNewBusy = 0;
										switch (iMovingRoom)
										{
											case 0: iMovingRoom = 1; break;
											case 24: iMovingRoom = 1; break;
											default: iMovingRoom++; break;
										}
									}
									break;
								case 3:
									iEventRoom = EventInfo (iChangeEvent, 1);
									if (iEventRoom < 24)
									{
										EventRoom (iEventRoom + 1);
									} else {
										EventRoom (1);
									}
									break;
							}
							break;
						case SDLK_d:
							RunLevel ((int)luLevelNr);
							break;
						case SDLK_SLASH:
						case SDLK_QUESTION:
							if (iScreen == 1)
							{
								for (iTemp = 1; iTemp <= 30; iTemp++)
								{
									SetLocation (iCurRoom, iTemp, 0, 0);
								}
								sGuardLocations[iCurRoom - 1] = 30;
								iChanged = 1;
								PlaySound ("wav/ok_close.wav");
							}
							break;
						case SDLK_BACKSLASH:
							if (iScreen == 1)
							{
								for (iTemp2 = 1; iTemp2 <= ROOMS; iTemp2++)
								{
									for (iTemp = 1; iTemp <= 30; iTemp++)
									{
										UseTile (-1, iTemp, iTemp2);
									}
								}
								iChanged = 1;
								PlaySound ("wav/ok_close.wav");
							}
							break;
						case SDLK_KP_ENTER:
						case SDLK_RETURN:
						case SDLK_SPACE:
							switch (iScreen)
							{
								case 1:
									ChangePos (iSelected, screen);
									break;
								case 2:
									if (iBrokenRoomLinks == 0)
									{
										if (iMovingRoom != 0)
										{
											if (iRoomArray[iMovingNewX][iMovingNewY] == 0)
											{
												RemoveOldRoom();
												AddNewRoom (iMovingNewX, iMovingNewY, iMovingRoom);
												iChanged = 1;
											}
											iMovingRoom = 0; iMovingNewBusy = 0;
										}
									} else {
										iRoomConnections[iChangingBrokenRoom]
											[iChangingBrokenSide]++;
										if (iRoomConnections[iChangingBrokenRoom]
											[iChangingBrokenSide] == 25)
										{
											iRoomConnections[iChangingBrokenRoom]
												[iChangingBrokenSide] = 0;
										}
										iChanged = 1;
										iBrokenRoomLinks = BrokenRoomLinks (iRoomConnections, 0);
										PlaySound ("wav/hum_adj.wav");
									}
									break;
							}
							break;
						case SDLK_BACKSPACE:
							if ((iScreen == 2) && (iBrokenRoomLinks == 1))
							{
								iRoomConnections[iChangingBrokenRoom]
									[iChangingBrokenSide]--;
								if (iRoomConnections[iChangingBrokenRoom]
									[iChangingBrokenSide] == -1)
								{
									iRoomConnections[iChangingBrokenRoom]
										[iChangingBrokenSide] = 24;
								}
								iChanged = 1;
								iBrokenRoomLinks = BrokenRoomLinks (iRoomConnections, 0);
								PlaySound ("wav/hum_adj.wav");
							}
							break;
						case SDLK_ESCAPE:
						case SDLK_q:
							switch (iScreen)
							{
								case 1:
									Quit(); break;
								case 2:
									iBrokenRoomLinks = BrokenRoomLinks (iRoomConnections, 0);
									/*** no break ***/
								case 3:
									iScreen = 1; break;
							}
							break;
						case SDLK_LEFT:
							if ((event.key.keysym.mod & KMOD_LSHIFT) ||
								(event.key.keysym.mod & KMOD_RSHIFT))
							{
								switch (iScreen)
								{
									case 1:
										if (iRoomConnections[iCurRoom][1] != 0)
										{
											iCurRoom = iRoomConnections[iCurRoom][1];
											PlaySound ("wav/scroll.wav");
										}
										break;
									case 3:
										ChangeEvent (-1, 0);
										break;
								}
							} else if ((event.key.keysym.mod & KMOD_LCTRL) ||
								(event.key.keysym.mod & KMOD_RCTRL))
							{
								if (iScreen == 3)
								{
									ChangeEvent (-10, 0);
								}
							} else {
								switch (iScreen)
								{
									case 1:
										iSelected--;
										switch (iSelected)
										{
											case 0: iSelected = 10; break;
											case 10: iSelected = 20; break;
											case 20: iSelected = 30; break;
										}
										break;
									case 2:
										if (iBrokenRoomLinks == 0)
										{
											if (iMovingRoom != 0)
											{
												if (iMovingNewX != 1) { iMovingNewX--; }
													else { iMovingNewX = 25; }
												PlaySound ("wav/cross.wav");
											}
										} else {
											if (iChangingBrokenSide != 1)
											{
												iChangingBrokenSide = 1;
											} else {
												iChangingBrokenRoom--;
												switch (iChangingBrokenRoom)
												{
													case 0: iChangingBrokenRoom = 4; break;
													case 4: iChangingBrokenRoom = 8; break;
													case 8: iChangingBrokenRoom = 12; break;
													case 12: iChangingBrokenRoom = 16; break;
													case 16: iChangingBrokenRoom = 20; break;
													case 20: iChangingBrokenRoom = 24; break;
												}
											}
										}
										break;
									case 3:
										iEventDoor = EventInfo (iChangeEvent, 2);
										switch (iEventDoor)
										{
											case 1: iEventDoor = 10; break;
											case 11: iEventDoor = 20; break;
											case 21: iEventDoor = 30; break;
											default: iEventDoor--; break;
										}
										if (iEventDoor > 20) { EventDoor (iEventDoor - 20, 3); }
											else if (iEventDoor > 10)
												{ EventDoor (iEventDoor - 10, 2); }
											else { EventDoor (iEventDoor, 1); }
										break;
								}
							}
							break;
						case SDLK_RIGHT:
							if ((event.key.keysym.mod & KMOD_LSHIFT) ||
								(event.key.keysym.mod & KMOD_RSHIFT))
							{
								switch (iScreen)
								{
									case 1:
										if (iRoomConnections[iCurRoom][2] != 0)
										{
											iCurRoom = iRoomConnections[iCurRoom][2];
											PlaySound ("wav/scroll.wav");
										}
										break;
									case 3:
										ChangeEvent (1, 0);
										break;
								}
							} else if ((event.key.keysym.mod & KMOD_LCTRL) ||
								(event.key.keysym.mod & KMOD_RCTRL))
							{
								if (iScreen == 3)
								{
									ChangeEvent (10, 0);
								}
							} else {
								switch (iScreen)
								{
									case 1:
										iSelected++;
										switch (iSelected)
										{
											case 11: iSelected = 1; break;
											case 21: iSelected = 11; break;
											case 31: iSelected = 21; break;
										}
										break;
									case 2:
										if (iBrokenRoomLinks == 0)
										{
											if (iMovingRoom != 0)
											{
												if (iMovingNewX != 25) { iMovingNewX++; }
													else { iMovingNewX = 1; }
												PlaySound ("wav/cross.wav");
											}
										} else {
											if (iChangingBrokenSide != 2)
											{
												iChangingBrokenSide = 2;
											} else {
												iChangingBrokenRoom++;
												switch (iChangingBrokenRoom)
												{
													case 5: iChangingBrokenRoom = 1; break;
													case 9: iChangingBrokenRoom = 5; break;
													case 13: iChangingBrokenRoom = 9; break;
													case 17: iChangingBrokenRoom = 13; break;
													case 21: iChangingBrokenRoom = 17; break;
													case 25: iChangingBrokenRoom = 21; break;
												}
											}
										}
										break;
									case 3:
										iEventDoor = EventInfo (iChangeEvent, 2);
										switch (iEventDoor)
										{
											case 10: iEventDoor = 1; break;
											case 20: iEventDoor = 11; break;
											case 30: iEventDoor = 21; break;
											default: iEventDoor++; break;
										}
										if (iEventDoor > 20) { EventDoor (iEventDoor - 20, 3); }
											else if (iEventDoor > 10)
												{ EventDoor (iEventDoor - 10, 2); }
											else { EventDoor (iEventDoor, 1); }
										break;
								}
							}
							break;
						case SDLK_UP:
							if ((event.key.keysym.mod & KMOD_LSHIFT) ||
								(event.key.keysym.mod & KMOD_RSHIFT))
							{
								if (iScreen == 1)
								{
									if (iRoomConnections[iCurRoom][3] != 0)
									{
										iCurRoom = iRoomConnections[iCurRoom][3];
										PlaySound ("wav/scroll.wav");
									}
								}
							} else {
								switch (iScreen)
								{
									case 1:
										if (iSelected > 10) { iSelected-=10; }
											else { iSelected+=20; }
										break;
									case 2:
										if (iBrokenRoomLinks == 0)
										{
											if (iMovingRoom != 0)
											{
												if (iMovingNewY != 1) { iMovingNewY--; }
													else { iMovingNewY = 24; }
												PlaySound ("wav/cross.wav");
											}
										} else {
											if (iChangingBrokenSide != 3)
											{
												iChangingBrokenSide = 3;
											} else {
												iChangingBrokenRoom -= 4;
												switch (iChangingBrokenRoom)
												{
													case 0: iChangingBrokenRoom = 24; break;
													case -1: iChangingBrokenRoom = 23; break;
													case -2: iChangingBrokenRoom = 22; break;
													case -3: iChangingBrokenRoom = 21; break;
												}
											}
										}
										break;
									case 3:
										iEventDoor = EventInfo (iChangeEvent, 2);
										if (iEventDoor > 10) { iEventDoor-=10; }
											else { iEventDoor+=20; }
										if (iEventDoor > 20) { EventDoor (iEventDoor - 20, 3); }
											else if (iEventDoor > 10)
												{ EventDoor (iEventDoor - 10, 2); }
											else { EventDoor (iEventDoor, 1); }
										break;
								}
							}
							break;
						case SDLK_DOWN:
							if ((event.key.keysym.mod & KMOD_LSHIFT) ||
								(event.key.keysym.mod & KMOD_RSHIFT))
							{
								if (iScreen == 1)
								{
									if (iRoomConnections[iCurRoom][4] != 0)
									{
										iCurRoom = iRoomConnections[iCurRoom][4];
										PlaySound ("wav/scroll.wav");
									}
								}
							} else {
								switch (iScreen)
								{
									case 1:
										if (iSelected <= 20) { iSelected+=10; }
											else { iSelected-=20; }
										break;
									case 2:
										if (iBrokenRoomLinks == 0)
										{
											if (iMovingRoom != 0)
											{
												if (iMovingNewY != 24) { iMovingNewY++; }
													else { iMovingNewY = 1; }
												PlaySound ("wav/cross.wav");
											}
										} else {
											if (iChangingBrokenSide != 4)
											{
												iChangingBrokenSide = 4;
											} else {
												iChangingBrokenRoom += 4;
												switch (iChangingBrokenRoom)
												{
													case 25: iChangingBrokenRoom = 1; break;
													case 26: iChangingBrokenRoom = 2; break;
													case 27: iChangingBrokenRoom = 3; break;
													case 28: iChangingBrokenRoom = 4; break;
												}
											}
										}
										break;
									case 3:
										iEventDoor = EventInfo (iChangeEvent, 2);
										if (iEventDoor <= 20) { iEventDoor+=10; }
											else { iEventDoor-=20; }
										if (iEventDoor > 20) { EventDoor (iEventDoor - 20, 3); }
											else if (iEventDoor > 10)
												{ EventDoor (iEventDoor - 10, 2); }
											else { EventDoor (iEventDoor, 1); }
										break;
								}
							}
							break;
						case SDLK_y:
							if (iScreen == 3) { EventNext (0); }
							break;
						case SDLK_n:
							if (iScreen == 3) { EventNext (1); }
							break;
						case SDLK_t:
							if (iCurType == 1) { iCurType = 2; } else { iCurType = 1; }
							PlaySound ("wav/extras.wav");
							break;
						case SDLK_MINUS:
						case SDLK_KP_MINUS:
						case SDLK_UNDERSCORE:
							if (iChanged != 0) { InitPopUpSave (iScreen); }
							Prev ((int)luLevelNr); break;
						case SDLK_PLUS:
						case SDLK_KP_PLUS:
						case SDLK_EQUALS:
							if (iChanged != 0) { InitPopUpSave (iScreen); }
							Next ((int)luLevelNr); break;
						case SDLK_r:
							if (iScreen != 2)
							{
								iScreen = 2;
								iMovingRoom = 0;
								iMovingNewBusy = 0;
								iChangingBrokenRoom = iCurRoom;
								iChangingBrokenSide = 1;
								PlaySound ("wav/screen2or3.wav");
							} else if (iBrokenRoomLinks == 0) {
								iBrokenRoomLinks = 1;
								PlaySound ("wav/screen2or3.wav");
							}
							break;
						case SDLK_e:
							if (iScreen == 2)
							{
								iBrokenRoomLinks = BrokenRoomLinks (iRoomConnections, 0);
							}
							if (iScreen != 3)
							{
								iScreen = 3;
								PlaySound ("wav/screen2or3.wav");
							}
							break;
						case SDLK_s:
							if (iChanged != 0) { CallSave (0); } break;
						case SDLK_z:
							Zoom();
							iExtras = 0;
							PlaySound ("wav/extras.wav");
							break;
						case SDLK_QUOTE:
							if ((event.key.keysym.mod & KMOD_LSHIFT) ||
								(event.key.keysym.mod & KMOD_RSHIFT))
							{
								Sprinkle();
								PlaySound ("wav/extras.wav");
								iChanged = 1;
							} else {
								SetLocation (iCurRoom, iSelected, iLastThing, iLastModifier);
								PlaySound ("wav/ok_close.wav");
								iChanged = 1;
							}
							break;
						case SDLK_h:
							FlipRoom (iCurRoom, 1);
							PlaySound ("wav/extras.wav");
							iChanged = 1;
							break;
						case SDLK_v:
							if ((event.key.keysym.mod & KMOD_LCTRL) ||
								(event.key.keysym.mod & KMOD_RCTRL))
							{
								CopyPaste (iCurRoom, 2);
								PlaySound ("wav/extras.wav");
								iChanged = 1;
							} else {
								FlipRoom (iCurRoom, 2);
								PlaySound ("wav/extras.wav");
								iChanged = 1;
							}
							break;
						case SDLK_c:
							if ((event.key.keysym.mod & KMOD_LCTRL) ||
								(event.key.keysym.mod & KMOD_RCTRL))
							{
								CopyPaste (iCurRoom, 1);
								PlaySound ("wav/extras.wav");
							}
							break;
						default: break;
					}
					ShowScreen (iScreen);
					break;
				case SDL_MOUSEMOTION:
					iOldXPos = iXPos;
					iOldYPos = iYPos;
					SDL_GetMouseState (&iXPos, &iYPos);
					if ((iOldXPos == iXPos) && (iOldYPos == iYPos)) { break; }

					if (iScreen == 2)
					{
						if (iMovingRoom != 0) { ShowScreen (2); }
					}

					if (iScreen == 1)
					{
						/*** User hovers over tiles in the upper row. ***/
						if ((InArea (event, HOR1, VER1 + TTP, HOR2, VER2 + TTP)
							== 1) && (iSelected != 1)) { iSelected = 1; ShowScreen (1); }
						else if ((InArea (event, HOR2, VER1 + TTP, HOR3, VER2 + TTP)
							== 1) && (iSelected != 2)) { iSelected = 2; ShowScreen (1); }
						else if ((InArea (event, HOR3, VER1 + TTP, HOR4, VER2 + TTP)
							== 1) && (iSelected != 3)) { iSelected = 3; ShowScreen (1); }
						else if ((InArea (event, HOR4, VER1 + TTP, HOR5, VER2 + TTP)
							== 1) && (iSelected != 4)) { iSelected = 4; ShowScreen (1); }
						else if ((InArea (event, HOR5, VER1 + TTP, HOR6, VER2 + TTP)
							== 1) && (iSelected != 5)) { iSelected = 5; ShowScreen (1); }
						else if ((InArea (event, HOR6, VER1 + TTP, HOR7, VER2 + TTP)
							== 1) && (iSelected != 6)) { iSelected = 6; ShowScreen (1); }
						else if ((InArea (event, HOR7, VER1 + TTP, HOR8, VER2 + TTP)
							== 1) && (iSelected != 7)) { iSelected = 7; ShowScreen (1); }
						else if ((InArea (event, HOR8, VER1 + TTP, HOR9, VER2 + TTP)
							== 1) && (iSelected != 8)) { iSelected = 8; ShowScreen (1); }
						else if ((InArea (event, HOR9, VER1 + TTP, HOR10, VER2 + TTP)
							== 1) && (iSelected != 9)) { iSelected = 9; ShowScreen (1); }
						else if ((InArea (event, HOR10, VER1 + TTP, HOR10 + 64,
							VER2 + TTP) == 1) && (iSelected != 10))
						{ iSelected = 10; ShowScreen (1); }

						/*** User hovers over tiles in the middle row. ***/
						else if ((InArea (event, HOR1, VER2 + TTP, HOR2, VER3 + TTP)
							== 1) && (iSelected != 11)) { iSelected = 11; ShowScreen (1); }
						else if ((InArea (event, HOR2, VER2 + TTP, HOR3, VER3 + TTP)
							== 1) && (iSelected != 12)) { iSelected = 12; ShowScreen (1); }
						else if ((InArea (event, HOR3, VER2 + TTP, HOR4, VER3 + TTP)
							== 1) && (iSelected != 13)) { iSelected = 13; ShowScreen (1); }
						else if ((InArea (event, HOR4, VER2 + TTP, HOR5, VER3 + TTP)
							== 1) && (iSelected != 14)) { iSelected = 14; ShowScreen (1); }
						else if ((InArea (event, HOR5, VER2 + TTP, HOR6, VER3 + TTP)
							== 1) && (iSelected != 15)) { iSelected = 15; ShowScreen (1); }
						else if ((InArea (event, HOR6, VER2 + TTP, HOR7, VER3 + TTP)
							== 1) && (iSelected != 16)) { iSelected = 16; ShowScreen (1); }
						else if ((InArea (event, HOR7, VER2 + TTP, HOR8, VER3 + TTP)
							== 1) && (iSelected != 17)) { iSelected = 17; ShowScreen (1); }
						else if ((InArea (event, HOR8, VER2 + TTP, HOR9, VER3 + TTP)
							== 1) && (iSelected != 18)) { iSelected = 18; ShowScreen (1); }
						else if ((InArea (event, HOR9, VER2 + TTP, HOR10, VER3 + TTP)
							== 1) && (iSelected != 19)) { iSelected = 19; ShowScreen (1); }
						else if ((InArea (event, HOR10, VER2 + TTP, HOR10 + 64,
							VER3 + TTP) == 1) && (iSelected != 20))
						{ iSelected = 20; ShowScreen (1); }

						/*** User hovers over tiles in the bottom row. ***/
						else if ((InArea (event, HOR1, VER3 + TTP, HOR2, VER3 + 126 + TTP)
							== 1) && (iSelected != 21)) { iSelected = 21; ShowScreen (1); }
						else if ((InArea (event, HOR2, VER3 + TTP, HOR3, VER3 + 126 + TTP)
							== 1) && (iSelected != 22)) { iSelected = 22; ShowScreen (1); }
						else if ((InArea (event, HOR3, VER3 + TTP, HOR4, VER3 + 126 + TTP)
							== 1) && (iSelected != 23)) { iSelected = 23; ShowScreen (1); }
						else if ((InArea (event, HOR4, VER3 + TTP, HOR5, VER3 + 126 + TTP)
							== 1) && (iSelected != 24)) { iSelected = 24; ShowScreen (1); }
						else if ((InArea (event, HOR5, VER3 + TTP, HOR6, VER3 + 126 + TTP)
							== 1) && (iSelected != 25)) { iSelected = 25; ShowScreen (1); }
						else if ((InArea (event, HOR6, VER3 + TTP, HOR7, VER3 + 126 + TTP)
							== 1) && (iSelected != 26)) { iSelected = 26; ShowScreen (1); }
						else if ((InArea (event, HOR7, VER3 + TTP, HOR8, VER3 + 126 + TTP)
							== 1) && (iSelected != 27)) { iSelected = 27; ShowScreen (1); }
						else if ((InArea (event, HOR8, VER3 + TTP, HOR9, VER3 + 126 + TTP)
							== 1) && (iSelected != 28)) { iSelected = 28; ShowScreen (1); }
						else if ((InArea (event, HOR9, VER3 + TTP, HOR10, VER3 + 126 + TTP)
							== 1) && (iSelected != 29)) { iSelected = 29; ShowScreen (1); }
						else if ((InArea (event, HOR10, VER3 + TTP, HOR10 + 64,
							VER3 + 126 + TTP) == 1) && (iSelected != 30))
						{ iSelected = 30; ShowScreen (1); }

						/*** extras ***/
						if ((InArea (event, 620, 3, 629, 12) == 1) && (iExtras != 1))
							{ iExtras = 1; ShowScreen (1); }
						else if ((InArea (event, 630, 3, 639, 12) == 1) && (iExtras != 2))
							{ iExtras = 2; ShowScreen (1); }
						else if ((InArea (event, 640, 3, 649, 12) == 1) && (iExtras != 3))
							{ iExtras = 3; ShowScreen (1); }
						else if ((InArea (event, 650, 3, 659, 12) == 1) && (iExtras != 4))
							{ iExtras = 4; ShowScreen (1); }
						else if ((InArea (event, 620, 13, 629, 22) == 1) && (iExtras != 5))
							{ iExtras = 5; ShowScreen (1); }
						else if ((InArea (event, 630, 13, 639, 22) == 1) && (iExtras != 6))
							{ iExtras = 6; ShowScreen (1); }
						else if ((InArea (event, 640, 13, 649, 22) == 1) && (iExtras != 7))
							{ iExtras = 7; ShowScreen (1); }
						else if ((InArea (event, 650, 13, 659, 22) == 1) && (iExtras != 8))
							{ iExtras = 8; ShowScreen (1); }
						else if ((InArea (event, 620, 3, 659, 22) == 0) && (iExtras != 0))
							{ iExtras = 0; ShowScreen (1); }
					}

					break;
				case SDL_MOUSEBUTTONDOWN:
					if (event.button.button == 1)
					{
						if (InArea (event, 0, 50, 25, 430) == 1) /*** left arrow ***/
						{
							if (iRoomConnections[iCurRoom][1] != 0)
								iDownAt = 1;
						}
						if (InArea (event, 667, 50, 692, 430) == 1) /*** right arrow ***/
						{
							if (iRoomConnections[iCurRoom][2] != 0)
								iDownAt = 2;
						}
						if (InArea (event, 25, 25, 667, 50) == 1) /*** up arrow ***/
						{
							if (iRoomConnections[iCurRoom][3] != 0)
								iDownAt = 3;
						}
						if (InArea (event, 25, 430, 667, 455) == 1) /*** down arrow ***/
						{
							if (iRoomConnections[iCurRoom][4] != 0)
								iDownAt = 4;
						}
						if (InArea (event, 0, 25, 25, 50) == 1) /*** rooms ***/
						{
							if (iBrokenRoomLinks == 0)
							{
								iDownAt = 5;
							} else {
								iDownAt = 11;
							}
						}
						if (InArea (event, 667, 25, 692, 50) == 1) /*** events ***/
						{
							iDownAt = 6;
						}
						if (InArea (event, 0, 430, 25, 455) == 1) /*** save ***/
						{
							iDownAt = 7;
						}
						if (InArea (event, 667, 430, 692, 455) == 1) /*** quit ***/
						{
							iDownAt = 8;
						}
						if (InArea (event, 0, 0, 25, 25) == 1) /*** previous ***/
						{
							iDownAt = 9;
						}
						if (InArea (event, 667, 0, 692, 25) == 1) /*** next ***/
						{
							iDownAt = 10;
						}

						if (iScreen == 2)
						{
							if (iBrokenRoomLinks == 0)
							{
								for (iTemp = 0; iTemp < ROOMS; iTemp++) /*** x ***/
								{
									for (iTemp2 = 0; iTemp2 < ROOMS; iTemp2++) /*** y ***/
									{
										if (InArea (event, 297 + (iTemp * 15), 61 + (iTemp2 * 15),
											310 + (iTemp * 15), 74 + (iTemp2 * 15)) == 1)
										{
											if (iRoomArray[iTemp + 1][iTemp2 + 1] != 0)
											{
												iMovingNewBusy = 0;
												iMovingRoom = iRoomArray[iTemp + 1][iTemp2 + 1];
											}
										}
									}
								}
								for (iTemp2 = 0; iTemp2 < ROOMS; iTemp2++) /*** y ***/
								{
									if (InArea (event, 272, 61 + (iTemp2 * 15), 285,
										74 + (iTemp2 * 15)) == 1)
									{
										if (iRoomArray[25][iTemp2 + 1] != 0)
										{
											iMovingNewBusy = 0;
											iMovingRoom = iRoomArray[25][iTemp2 + 1];
										}
									}
								}

								/*** rooms broken ***/
								if (InArea (event, 629, 63, 654, 88) == 1)
								{
									iDownAt = 11;
								}
							} else {
								MouseSelectAdj (event);
							}
						}
					}
					ShowScreen (iScreen);
					break;
				case SDL_MOUSEBUTTONUP:
					iDownAt = 0;
					if (event.button.button == 1) /*** left mouse button, change ***/
					{
						if (InArea (event, 0, 50, 25, 430) == 1) /*** left arrow ***/
						{
							if (iRoomConnections[iCurRoom][1] != 0) {
								iCurRoom = iRoomConnections[iCurRoom][1];
								PlaySound ("wav/scroll.wav"); }
						}
						if (InArea (event, 667, 50, 692, 430) == 1) /*** right arrow ***/
						{
							if (iRoomConnections[iCurRoom][2] != 0) {
								iCurRoom = iRoomConnections[iCurRoom][2];
								PlaySound ("wav/scroll.wav"); }
						}
						if (InArea (event, 25, 25, 667, 50) == 1) /*** up arrow ***/
						{
							if (iRoomConnections[iCurRoom][3] != 0) {
								iCurRoom = iRoomConnections[iCurRoom][3];
								PlaySound ("wav/scroll.wav"); }
						}
						if (InArea (event, 25, 430, 667, 455) == 1) /*** down arrow ***/
						{
							if (iRoomConnections[iCurRoom][4] != 0) {
								iCurRoom = iRoomConnections[iCurRoom][4];
								PlaySound ("wav/scroll.wav"); }
						}
						if (InArea (event, 0, 25, 25, 50) == 1) /*** rooms ***/
						{
							if (iScreen != 2)
							{
								iScreen = 2; iMovingRoom = 0; iMovingNewBusy = 0;
								iChangingBrokenRoom = iCurRoom;
								iChangingBrokenSide = 1;
								PlaySound ("wav/screen2or3.wav");
							}
						}
						if (InArea (event, 667, 25, 692, 50) == 1) /*** events ***/
						{
							if (iScreen == 2)
							{
								iBrokenRoomLinks = BrokenRoomLinks (iRoomConnections, 0);
							}
							if (iScreen != 3)
							{
								iScreen = 3;
								PlaySound ("wav/screen2or3.wav");
							}
						}
						if (InArea (event, 0, 430, 25, 455) == 1) /*** save ***/
						{
							if (iChanged != 0) { CallSave (0); }
						}
						if (InArea (event, 667, 430, 692, 455) == 1) /*** quit ***/
						{
							switch (iScreen)
							{
								case 1:
									Quit(); break;
								case 2:
									iBrokenRoomLinks = BrokenRoomLinks (iRoomConnections, 0);
									/*** no break ***/
								case 3:
									iScreen = 1; break;
							}
						}
						if (InArea (event, 0, 0, 25, 25) == 1) /*** previous ***/
						{
							if (iChanged != 0) { InitPopUpSave (iScreen); }
							Prev ((int)luLevelNr);
						}
						if (InArea (event, 667, 0, 692, 25) == 1) /*** next ***/
						{
							if (iChanged != 0) { InitPopUpSave (iScreen); }
							Next ((int)luLevelNr);
						}
						if (InArea (event, 25, 0, 612, 24) == 1) /*** level bar ***/
						{
							RunLevel ((int)luLevelNr);
						}

						if (iScreen == 1)
						{
							if (InArea (event, HOR1, VER1 + TTP, HOR10 + 64,
								VER3 + 126 + TTP) == 1)
							{
								keystate = SDL_GetKeyState (NULL);
								if ((keystate[SDLK_LSHIFT]) || (keystate[SDLK_RSHIFT]))
								{
									SetLocation (iCurRoom, iSelected, iLastThing, iLastModifier);
									PlaySound ("wav/ok_close.wav");
									iChanged = 1;
								} else {
									ChangePos (iSelected, screen);
								}
							}

							if (InArea (event, 620, 3, 629, 12) == 1)
							{
								Zoom();
								iExtras = 0;
								PlaySound ("wav/extras.wav");
							}
							if (InArea (event, 650, 3, 659, 12) == 1)
							{
								if (iCurType == 1) { iCurType = 2; } else { iCurType = 1; }
								PlaySound ("wav/extras.wav");
							}
							if (InArea (event, 620, 13, 629, 22) == 1)
							{
								Sprinkle();
								PlaySound ("wav/extras.wav");
								iChanged = 1;
							}
							if (InArea (event, 640, 13, 649, 22) == 1)
							{
								FlipRoom (iCurRoom, 1);
								PlaySound ("wav/extras.wav");
								iChanged = 1;
							}
							if (InArea (event, 640, 3, 649, 12) == 1)
							{
								FlipRoom (iCurRoom, 2);
								PlaySound ("wav/extras.wav");
								iChanged = 1;
							}
							if (InArea (event, 630, 3, 639, 12) == 1)
							{
								CopyPaste (iCurRoom, 1);
								PlaySound ("wav/extras.wav");
							}
							if (InArea (event, 630, 13, 639, 22) == 1)
							{
								CopyPaste (iCurRoom, 2);
								PlaySound ("wav/extras.wav");
								iChanged = 1;
							}
						}

						if (iScreen == 2) /*** room links screen ***/
						{
							if (iBrokenRoomLinks == 0)
							{
								for (iTemp = 0; iTemp < ROOMS; iTemp++) /*** x ***/
								{
									for (iTemp2 = 0; iTemp2 < ROOMS; iTemp2++) /*** y ***/
									{
										if (InArea (event, 297 + (iTemp * 15), 61 + (iTemp2 * 15),
											310 + (iTemp * 15), 74 + (iTemp2 * 15)) == 1)
										{
											if (iMovingRoom != 0)
											{
												if (iRoomArray[iTemp + 1][iTemp2 + 1] == 0)
												{
													RemoveOldRoom();
													AddNewRoom (iTemp + 1, iTemp2 + 1, iMovingRoom);
													iChanged = 1;
												}
												iMovingRoom = 0; iMovingNewBusy = 0;
											}
										}
									}
								}
								for (iTemp2 = 0; iTemp2 < ROOMS; iTemp2++) /*** y ***/
								{
									if (InArea (event, 272, 61 + (iTemp2 * 15), 285,
										74 + (iTemp2 * 15)) == 1)
									{
										if (iMovingRoom != 0)
										{
											if (iRoomArray[25][iTemp2 + 1] == 0)
											{
												RemoveOldRoom();
												AddNewRoom (25, iTemp2 + 1, iMovingRoom);
												iChanged = 1;
											}
											iMovingRoom = 0; iMovingNewBusy = 0;
										}
									}
								}

								/*** rooms broken ***/
								if (InArea (event, 629, 63, 654, 88) == 1)
								{
									iBrokenRoomLinks = 1;
									PlaySound ("wav/screen2or3.wav");
								}
							} else {
								if (MouseSelectAdj (event) == 1)
								{
									iRoomConnections[iChangingBrokenRoom]
										[iChangingBrokenSide]++;
									if (iRoomConnections[iChangingBrokenRoom]
										[iChangingBrokenSide] == 25)
									{
										iRoomConnections[iChangingBrokenRoom]
											[iChangingBrokenSide] = 0;
									}
									iChanged = 1;
									iBrokenRoomLinks = BrokenRoomLinks (iRoomConnections, 0);
									PlaySound ("wav/hum_adj.wav");
								}
							}
						}

						if (iScreen == 3) /*** events screen ***/
						{
							/*** Change event number ***/
							if (InArea (event, 343, 60, 355, 79) == 1)
								{ ChangeEvent (-10, 0); }
							if (InArea (event, 358, 60, 370, 79) == 1)
								{ ChangeEvent (-1, 0); }
							if (InArea (event, 428, 60, 440, 79) == 1)
								{ ChangeEvent (1, 0); }
							if (InArea (event, 443, 60, 455, 79) == 1)
								{ ChangeEvent (10, 0); }

							/*** room ***/
							if ((event.motion.y >= 115 * iScale) &&
								(event.motion.y <= 128 * iScale))
							{
								for (iTemp = 1; iTemp <= ROOMS; iTemp++)
								{
									if ((event.motion.x >= (297 + ((iTemp - 1) * 15)) * iScale)
										&& (event.motion.x <= (310 + ((iTemp - 1) * 15)) * iScale))
									{
										EventRoom (iTemp);
									}
								}
							}

							/*** location ***/
							for (iTemp2 = 1; iTemp2 <= 3; iTemp2++)
							{
								for (iTemp = 1; iTemp <= 10; iTemp++)
								{
									if ((event.motion.x >= (462 + ((iTemp - 1) * 15)) * iScale)
										&& (event.motion.x <= (475 + ((iTemp - 1) * 15)) * iScale))
									{
										if ((event.motion.y >=
											(155 + ((iTemp2 - 1) * 15)) * iScale)
											&& (event.motion.y <=
											(168 + ((iTemp2 - 1) * 15)) * iScale))
										{
											EventDoor (iTemp, iTemp2);
										}
									}
								}
							}

							/*** next ***/
							if (InArea (event, 582, 225, 595, 238) == 1) { EventNext (1); }
							if (InArea (event, 597, 225, 610, 238) == 1) { EventNext (0); }
						}
					}
					if (event.button.button == 2) /*** middle mouse button, clear ***/
					{
						if (iScreen == 1)
						{
							for (iTemp = 1; iTemp <= 30; iTemp++)
							{
								SetLocation (iCurRoom, iTemp, 0, 0);
							}
							sGuardLocations[iCurRoom - 1] = 30;
							PlaySound ("wav/ok_close.wav");
							iChanged = 1;
						}
					}
					if (event.button.button == 3) /*** right mouse button, randomize ***/
					{
						if (iScreen == 1)
						{
							for (iTemp2 = 1; iTemp2 <= ROOMS; iTemp2++)
							{
								for (iTemp = 1; iTemp <= 30; iTemp++)
								{
									UseTile (-1, iTemp, iTemp2);
								}
							}
							PlaySound ("wav/ok_close.wav");
							iChanged = 1;
						}
						if (iScreen == 2)
						{
							if (iBrokenRoomLinks == 1)
							{
								if (MouseSelectAdj (event) == 1)
								{
									iRoomConnections[iChangingBrokenRoom]
										[iChangingBrokenSide]--;
									if (iRoomConnections[iChangingBrokenRoom]
										[iChangingBrokenSide] == -1)
									{
										iRoomConnections[iChangingBrokenRoom]
											[iChangingBrokenSide] = 24;
									}
									iChanged = 1;
									iBrokenRoomLinks = BrokenRoomLinks (iRoomConnections, 0);
									PlaySound ("wav/hum_adj.wav");
								}
							}
						}
					}
					if (event.button.button == 4) /*** scroll wheel up ***/
					{
						if (InArea (event, HOR1, VER1 + TTP, HOR10 + 64,
							VER3 + 126 + TTP) == 1) /*** middle field ***/
						{
							keystate = SDL_GetKeyState (NULL);
							if ((keystate[SDLK_LSHIFT]) || (keystate[SDLK_RSHIFT]))
							{ /*** right ***/
								if (iRoomConnections[iCurRoom][2] != 0) {
									iCurRoom = iRoomConnections[iCurRoom][2];
									PlaySound ("wav/scroll.wav"); }
							} else { /*** up ***/
								if (iRoomConnections[iCurRoom][3] != 0) {
									iCurRoom = iRoomConnections[iCurRoom][3];
									PlaySound ("wav/scroll.wav"); }
							}
						}
					}
					if (event.button.button == 5) /*** scroll wheel down ***/
					{
						if (InArea (event, HOR1, VER1 + TTP, HOR10 + 64,
							VER3 + 126 + TTP) == 1) /*** middle field ***/
						{
							keystate = SDL_GetKeyState (NULL);
							if ((keystate[SDLK_LSHIFT]) || (keystate[SDLK_RSHIFT]))
							{ /*** left ***/
								if (iRoomConnections[iCurRoom][1] != 0) {
									iCurRoom = iRoomConnections[iCurRoom][1];
									PlaySound ("wav/scroll.wav"); }
							} else { /*** down ***/
								if (iRoomConnections[iCurRoom][4] != 0) {
									iCurRoom = iRoomConnections[iCurRoom][4];
									PlaySound ("wav/scroll.wav"); }
							}
						}
					}
					ShowScreen (iScreen);
					break;
				case SDL_QUIT:
					Quit(); break;
				default:
					break;
			}
		}

		/*** prevent CPU eating ***/
		gamespeed = REFRESH;
		while ((SDL_GetTicks() - looptime) < gamespeed)
		{
			SDL_Delay (10);
		}
		looptime = SDL_GetTicks();
	}
}
/*****************************************************************************/
void EventDoor (int iX, int iY)
/*****************************************************************************/
{
	char sBinaryFDoors[9 + 2]; /*** 8 chars, plus \0 ***/
	char sTempBinary[9 + 2]; /*** 8 chars, plus \0 ***/

	/* This part is kind of complicated. Only some bits have
	 * to be changed, so we first convert the event to bits,
	 * change some of them, and then convert them all back.
	 */
	GetAsEightBits (sFirstDoorEvents[iChangeEvent],
		sBinaryFDoors);
	GetAsEightBits ((((iY - 1) * 10) + iX - 1),
		sTempBinary);
	sBinaryFDoors[3] = sTempBinary[3];
	sBinaryFDoors[4] = sTempBinary[4];
	sBinaryFDoors[5] = sTempBinary[5];
	sBinaryFDoors[6] = sTempBinary[6];
	sBinaryFDoors[7] = sTempBinary[7];
	sFirstDoorEvents[iChangeEvent] =
		BitsToInt (sBinaryFDoors);
	PlaySound ("wav/check_box.wav");
	iChanged = 1;
}
/*****************************************************************************/
void EventNext (int iNoNext)
/*****************************************************************************/
{
	char sBinaryFDoors[9 + 2]; /*** 8 chars, plus \0 ***/

	/* This part is kind of complicated. Only one bit has
	 * to be changed, so we first convert the event to bits,
	 * change one of them, and then convert them all back.
	 */
	GetAsEightBits (sFirstDoorEvents[iChangeEvent],
		sBinaryFDoors);
	switch (iNoNext)
	{
		case 1: sBinaryFDoors[0] = '1'; break;
		case 0: sBinaryFDoors[0] = '0'; break;
	}
	sFirstDoorEvents[iChangeEvent] =
		BitsToInt (sBinaryFDoors);
	PlaySound ("wav/check_box.wav");
	iChanged = 1;
}
/*****************************************************************************/
void RunLevel (int iLevel)
/*****************************************************************************/
{
	/*** The following are used for the batch file. ***/
	int iFd;
	char sToWrite[MAX_TOWRITE + 2];
	SDL_Thread *princethread;

	if ((iLevel != 15) && (iLevel != 0))
	{
		/*** Create batch file. ***/
		iFd = open (BATCH_FILE, O_WRONLY|O_TRUNC|O_CREAT|O_BINARY, 0600);
		if (iFd == -1)
		{
			printf ("[FAILED] Could not create batch file: %s!\n",
				strerror (errno));
			exit (EXIT_ERROR);
		}
		snprintf (sToWrite, MAX_TOWRITE, "PRINCE %s %i\n", sCheat, iLevel);
		write (iFd, sToWrite, strlen (sToWrite));
		close (iFd);
		princethread = SDL_CreateThread (StartGame, NULL);
		if (princethread == NULL)
		{
			printf ("[FAILED] Could not create thread!\n");
			exit (EXIT_ERROR);
		}
	}
}
/*****************************************************************************/
void EventRoom (int iRoom)
/*****************************************************************************/
{
	char sBinarySDoors[9 + 2]; /*** 8 chars, plus \0 ***/
	char sBinaryFDoors[9 + 2]; /*** 8 chars, plus \0 ***/
	char sTempBinary[9 + 2]; /*** 8 chars, plus \0 ***/

	/* This part is kind of complicated. Only some bits have
	 * to be changed, so we first convert the event to bits,
	 * change some of them, and then convert all of them back.
	 */
	GetAsEightBits (sFirstDoorEvents[iChangeEvent],
		sBinaryFDoors);
	GetAsEightBits (sSecondDoorEvents[iChangeEvent],
		sBinarySDoors);
	GetAsEightBits (iRoom, sTempBinary);
	sBinarySDoors[0] = sTempBinary[3];
	sBinarySDoors[1] = sTempBinary[4];
	sBinarySDoors[2] = sTempBinary[5];
	sBinaryFDoors[1] = sTempBinary[6];
	sBinaryFDoors[2] = sTempBinary[7];
	sFirstDoorEvents[iChangeEvent] =
		BitsToInt (sBinaryFDoors);
	sSecondDoorEvents[iChangeEvent] =
		BitsToInt (sBinarySDoors);
	PlaySound ("wav/check_box.wav");
	iChanged = 1;
}
/*****************************************************************************/
void RemoveOldRoom (void)
/*****************************************************************************/
{
	iRoomArray[iMovingOldX][iMovingOldY] = 0;

	/* Change the links of the rooms around
	 * the removed room.
	 */

	/*** left of removed ***/
	if ((iMovingOldX - 1 >= 1) && (iMovingOldX - 1 <= 23))
	{
		if (iRoomArray[iMovingOldX - 1][iMovingOldY] != 0)
		{
			iRoomConnections[iRoomArray[iMovingOldX - 1]
				[iMovingOldY]][2] = 0; /*** remove right ***/
		}
	}

	/*** right of removed ***/
	if ((iMovingOldX + 1 >= 2) && (iMovingOldX + 1 <= 24))
	{
		if (iRoomArray[iMovingOldX + 1][iMovingOldY] != 0)
		{
			iRoomConnections[iRoomArray[iMovingOldX + 1]
				[iMovingOldY]][1] = 0; /*** remove left ***/
		}
	}

	/*** above removed ***/
	if ((iMovingOldY - 1 >= 1) && (iMovingOldY - 1 <= 23))
	{
		if (iRoomArray[iMovingOldX][iMovingOldY - 1] != 0)
		{
			iRoomConnections[iRoomArray[iMovingOldX]
				[iMovingOldY - 1]][4] = 0; /*** remove below ***/
		}
	}

	/*** below removed ***/
	if ((iMovingOldY + 1 >= 2) && (iMovingOldY + 1 <= 24))
	{
		if (iRoomArray[iMovingOldX][iMovingOldY + 1] != 0)
		{
			iRoomConnections[iRoomArray[iMovingOldX]
				[iMovingOldY + 1]][3] = 0; /*** remove above ***/
		}
	}
}
/*****************************************************************************/
void AddNewRoom (int iX, int iY, int iRoom)
/*****************************************************************************/
{
	iRoomArray[iX][iY] = iRoom;

	/* Change the links of the rooms around
	 * the new room and the room itself.
	 */

	iRoomConnections[iRoom][1] = 0;
	iRoomConnections[iRoom][2] = 0;
	iRoomConnections[iRoom][3] = 0;
	iRoomConnections[iRoom][4] = 0;
	if ((iX - 1 >= 1) && (iX - 1 <= 23)) /*** left of added ***/
	{
		if (iRoomArray[iX - 1][iY] != 0)
		{
			iRoomConnections[iRoomArray[iX - 1]
				[iY]][2] = iRoom; /*** add room right ***/
			iRoomConnections[iRoom][1] = iRoomArray[iX - 1][iY];
		}
	}
	if ((iX + 1 >= 2) && (iX + 1 <= 24)) /*** right of added ***/
	{
		if (iRoomArray[iX + 1][iY] != 0)
		{
			iRoomConnections[iRoomArray[iX + 1]
				[iY]][1] = iRoom; /*** add room left ***/
			iRoomConnections[iRoom][2] = iRoomArray[iX + 1][iY];
		}
	}
	if ((iY - 1 >= 1) && (iY - 1 <= 23)) /*** above added ***/
	{
		if (iRoomArray[iX][iY - 1] != 0)
		{
			iRoomConnections[iRoomArray[iX]
				[iY - 1]][4] = iRoom; /*** add room below ***/
			iRoomConnections[iRoom][3] = iRoomArray[iX][iY - 1];
		}
	}
	if ((iY + 1 >= 2) && (iY + 1 <= 24)) /*** below added ***/
	{
		if (iRoomArray[iX][iY + 1] != 0)
		{
			iRoomConnections[iRoomArray[iX]
				[iY + 1]][3] = iRoom; /*** add room above ***/
			iRoomConnections[iRoom][4] = iRoomArray[iX][iY + 1];
		}
	}
	PlaySound ("wav/move_room.wav");
}
/*****************************************************************************/
void InitRooms (void)
/*****************************************************************************/
{
	int iTemp;
	int iTemp2;

	for (iTemp = 0; iTemp <= ROOMS + 1; iTemp++) /*** x ***/
	{
		for (iTemp2 = 0; iTemp2 <= ROOMS; iTemp2++) /*** y ***/
		{
			iRoomArray[iTemp][iTemp2] = 0;
		}
	}
}
/*****************************************************************************/
int StartGame (void *unused)
/*****************************************************************************/
{
	PlaySound ("wav/dosbox.wav");
	if (system ("dosbox " BATCH_FILE " > " DEVNULL) == -1)
	{
		printf ("[FAILED] Could not execute batch file!\n");
	}
	return (EXIT_NORMAL);
}
/*****************************************************************************/
void Quit (void)
/*****************************************************************************/
{
	if (iChanged != 0) { InitPopUpSave (iScreen); }
	TTF_CloseFont (font1);
	TTF_CloseFont (font2);
	TTF_CloseFont (font3);
	TTF_Quit();
	SDL_Quit();
	exit (EXIT_NORMAL);
}
/*****************************************************************************/
void ShowScreen (int iScreen)
/*****************************************************************************/
{
	char sText[MAX_TEXT + 2];
	char sTemp[MAX_TEXT + 2];
	int iTemp;
	int iLoc;
	int iUnusedRooms;
	int iRoom;
	int iSide;
	int iTargetRoom;
	int iTargetTile;

	ShowImage (-4, 1, screen, 31, 0, 0, 692, 430); /*** black background ***/

	if (iScreen == 1)
	{
		/*** under this room ***/
		if (iRoomConnections[iCurRoom][4] != 0)
		{
			for (iTemp = 1; iTemp <= 10; iTemp++)
			{
				ShowImage (iThingA[iRoomConnections[iCurRoom][4]][iTemp - 1],
					iModifierA[iRoomConnections[iCurRoom][4]][iTemp - 1], screen,
					-4 - iTemp, 0, 0, 120, 155);
				/*** ShowImage (-11, 0, screen, -4 - iTemp, 0, 0, 120, 155); ***/
			}
		}
		for (iTemp = 1; iTemp <= 30; iTemp++)
		{
			/*** locations ***/
			if ((iTemp >= 1) && (iTemp <= 10)) { iLoc = 20 + iTemp; }
			if ((iTemp >= 11) && (iTemp <= 20)) { iLoc = iTemp; }
			if ((iTemp >= 21) && (iTemp <= 30)) { iLoc = -20 + iTemp; }
			ShowImage (iThingA[iCurRoom][iLoc - 1], iModifierA[iCurRoom][iLoc - 1],
				screen, iLoc, 0, 0, 120, 155);
			/*** kid ***/
			if ((iCurRoom == (int)luKidRoom) && (iLoc == (int)luKidPos))
			{
				ShowImage (-8, 1, screen, iLoc, 0, 0, 22, 84);
			}
			/*** guard ***/
			if (sGuardLocations[iCurRoom - 1] + 1 == iLoc)
			{
				ShowImage (-8, 2, screen, iLoc, 0, 0, 70, 86);
			}
		}
	}
	if (iScreen == 2) /*** R ***/
	{
		if (iBrokenRoomLinks == 0)
		{
			InitRooms();
			ShowImage (-6, 0, screen, 42, 0, 0, 642, 380); /*** room links ***/
			ShowImage (-2, 8, screen, 87, 0, 0, 25, 25); /*** rooms broken on ***/
			WhereToStart ();
			for (iTemp = 1; iTemp <= ROOMS; iTemp++)
			{
				iDone[iTemp] = 0;
			}
			ShowRooms ((int)luKidRoom, iStartRoomsX, iStartRoomsY, 1);
			iUnusedRooms = 0;
			for (iTemp = 1; iTemp <= ROOMS; iTemp++)
			{
				if (iDone[iTemp] != 1)
				{
					iUnusedRooms++;
					ShowRooms (iTemp, 25, iUnusedRooms, 0);
				}
			}
			if (iMovingRoom != 0)
			{
				ShowImage (-6, iMovingRoom, screen, 83, 0, 0, 14, 14);
				if (iCurRoom == iMovingRoom)
				{
					ShowImage (-7, 0, screen, 83, 0, 0, 14, 14);
				}
				if ((int)luKidRoom == iMovingRoom)
				{
					ShowImage (-7, 1, screen, 83, 0, 0, 14, 14);
				}
				ShowImage (-7, 2, screen, 83, 0, 0, 14, 14);
				ShowRooms (-1, iMovingNewX, iMovingNewY, 0);
			}
		} else {
			/*** broken room links ***/
			ShowImage (-6, 25, screen, 42, 0, 0, 642, 380);
			for (iRoom = 1; iRoom <= ROOMS; iRoom++)
			{
				if (iCurRoom == iRoom)
				{
					ShowImage (-7, 0, screen, BrokenRoomChange (iRoom, 0),
						0, 0, 14, 14);
				}
				if ((int)luKidRoom == iRoom)
				{
					ShowImage (-7, 1, screen, BrokenRoomChange (iRoom, 0),
						0, 0, 14, 14);
				}
				for (iSide = 1; iSide <= 4; iSide++)
				{
					if (iRoomConnections[iRoom][iSide] != 0)
					{
						ShowImage (-6, iRoomConnections[iRoom][iSide], screen,
							BrokenRoomChange (iRoom, iSide), 0, 0, 14, 14);
						if (iRoomConnectionsBroken[iRoom][iSide] == 1)
						{
							ShowImage (-7, 4, screen, BrokenRoomChange (iRoom, iSide),
								0, 0, 14, 14);
						}
					}
					if ((iChangingBrokenRoom == iRoom) && (iChangingBrokenSide == iSide))
					{
						ShowImage (-7, 2, screen, BrokenRoomChange (iRoom, iSide),
							0, 0, 14, 14);
					}
				}
			}
		}
	}
	if (iScreen == 3) /*** E ***/
	{
		ShowImage (-10, 0, screen, 42, 0, 0, 642, 380); /*** events ***/
		snprintf (sText, MAX_TEXT, "%i", iChangeEvent + 1);
		message = TTF_RenderText_Shaded (font3, sText, color_wh, color_bl);
		if ((iChangeEvent + 1 >= 1) && (iChangeEvent + 1 <= 9))
		{
			offset.x = 392;
		} else if ((iChangeEvent + 1 >= 10) && (iChangeEvent + 1 <= 99)) {
			offset.x = 385;
		} else {
			offset.x = 378;
		}
		offset.y = 59;
		CustomBlitSurface (message, NULL, screen, &offset, arFont);
		if ((EventInfo (iChangeEvent, 1) >= 1) &&
			(EventInfo (iChangeEvent, 1) <= 24))
		{
			ShowImage (-10, 1, screen, 49, 0, 0, 14, 14); /*** sel event, room ***/
		}
		if ((EventInfo (iChangeEvent, 2) >= 1) &&
			(EventInfo (iChangeEvent, 2) <= 30))
		{
			/*** sel event, location ***/
			ShowImage (-10, 1, screen, 50, 0, 0, 14, 14);
		}
		if ((EventInfo (iChangeEvent, 3) == 0) ||
			(EventInfo (iChangeEvent, 3) == 1))
		{
			ShowImage (-10, 1, screen, 51, 0, 0, 14, 14); /*** sel event, next ***/
		}

		/*** Show target. ***/
		iTargetRoom = EventInfo (iChangeEvent, 1);
		iTargetTile = EventInfo (iChangeEvent, 2) - 1;
		ShowImage (iThingA[iTargetRoom][iTargetTile], iModifierA[iTargetRoom]
			[iTargetTile], screen, 92, 0, 0, 120, 155);
	}

	if (iRoomConnections[iCurRoom][1] != 0)
	{
		ShowImage (-1, 1, screen, 32, 0, 0, 25, 380); /*** left ***/
	} else {
		ShowImage (-1, 6, screen, 32, 0, 0, 25, 380); /*** no left ***/
	}
	if (iRoomConnections[iCurRoom][2] != 0)
	{
		ShowImage (-1, 2, screen, 33, 0, 0, 25, 380); /*** right ***/
	} else {
		ShowImage (-1, 6, screen, 33, 0, 0, 25, 380); /*** no right ***/
	}
	if (iRoomConnections[iCurRoom][3] != 0)
	{
		ShowImage (-1, 3, screen, 34, 0, 0, 642, 25); /*** up ***/
	} else {
		if (iScreen != 1)
		{
			ShowImage (-1, 5, screen, 34, 0, 0, 642, 25); /*** no up ***/
		} else {
			ShowImage (-1, 7, screen, 34, 0, 0, 642, 25); /*** no up, with info ***/
		}
	}
	if (iRoomConnections[iCurRoom][4] != 0)
	{
		ShowImage (-1, 4, screen, 35, 0, 0, 642, 25); /*** down ***/
	} else {
		ShowImage (-1, 5, screen, 35, 0, 0, 642, 25); /*** no down ***/
	}
	switch (iScreen)
	{
		case 1:
			if (iBrokenRoomLinks == 1)
			{
				ShowImage (-2, 8, screen, 36, 0, 0, 25, 25); /*** rooms broken on ***/
			} else {
				ShowImage (-2, 1, screen, 36, 0, 0, 25, 25); /*** rooms on ***/
			}
			ShowImage (-2, 2, screen, 37, 0, 0, 25, 25); /*** events on ***/
			break;
		case 2:
			if (iBrokenRoomLinks == 1)
			{
				ShowImage (-2, 9, screen, 36, 0, 0, 25, 25); /*** rooms broken off ***/
			} else {
				ShowImage (-2, 6, screen, 36, 0, 0, 25, 25); /*** rooms off ***/
			}
			ShowImage (-2, 2, screen, 37, 0, 0, 25, 25); /*** events on ***/
			break;
		case 3:
			if (iBrokenRoomLinks == 1)
			{
				ShowImage (-2, 8, screen, 36, 0, 0, 25, 25); /*** rooms broken on ***/
			} else {
				ShowImage (-2, 1, screen, 36, 0, 0, 25, 25); /*** rooms on ***/
			}
			ShowImage (-2, 7, screen, 37, 0, 0, 25, 25); /*** events off ***/
			break;
	}
	if (iChanged == 1)
	{
		ShowImage (-2, 3, screen, 38, 0, 0, 25, 25); /*** save on ***/
	} else {
		ShowImage (-2, 5, screen, 38, 0, 0, 25, 25); /*** save off ***/
	}
	ShowImage (-2, 4, screen, 39, 0, 0, 25, 25); /*** quit ***/
	if ((int)luLevelNr != 1)
	{
		ShowImage (-5, 1, screen, 31, 0, 0, 25, 25); /*** previous on ***/
	} else {
		ShowImage (-5, 4, screen, 31, 0, 0, 25, 25); /*** previous off ***/
	}
	if ((int)luLevelNr != 0)
	{
		ShowImage (-5, 2, screen, 40, 0, 0, 25, 25); /*** next on ***/
	} else {
		ShowImage (-5, 5, screen, 40, 0, 0, 25, 25); /*** next off ***/
	}
	ShowImage (-5, 3, screen, 41, 0, 0, 642, 25); /*** level bar ***/

	switch (luLevelNr)
	{
		case 1:
			snprintf (sTemp, MAX_TEXT, "%s", "level 1 (dungeon; prison),"); break;
		case 2:
			snprintf (sTemp, MAX_TEXT, "%s", "level 2 (dungeon; guards),"); break;
		case 3:
			snprintf (sTemp, MAX_TEXT, "level 3 (dungeon; skeleton),"); break;
		case 4:
			snprintf (sTemp, MAX_TEXT, "level 4 (palace; mirror),"); break;
		case 5:
			snprintf (sTemp, MAX_TEXT, "level 5 (palace; thief),"); break;
		case 6:
			snprintf (sTemp, MAX_TEXT, "level 6 (palace; plunge),"); break;
		case 7:
			snprintf (sTemp, MAX_TEXT, "level 7 (dungeon; weightless),"); break;
		case 8:
			snprintf (sTemp, MAX_TEXT, "level 8 (dungeon; mouse),"); break;
		case 9:
			snprintf (sTemp, MAX_TEXT, "level 9 (dungeon; twisty),"); break;
		case 10:
			snprintf (sTemp, MAX_TEXT, "level 10 (palace; quad),"); break;
		case 11:
			snprintf (sTemp, MAX_TEXT, "level 11 (palace; fragile),"); break;
		case 12:
			snprintf (sTemp, MAX_TEXT, "level 12 (12a; dungeon; tower),"); break;
		case 13:
			snprintf (sTemp, MAX_TEXT, "level 13 (12b; dungeon; jaffar),"); break;
		case 14:
			snprintf (sTemp, MAX_TEXT, "level 14 (palace; rescue),"); break;
		case 15:
			snprintf (sTemp, MAX_TEXT, "level 15 (dungeon; potions),"); break;
		case 0:
			snprintf (sTemp, MAX_TEXT, "level 0 (dungeon; demo),"); break;
	}
	switch (iScreen)
	{
		case 1:
			snprintf (sText, MAX_TEXT, "%s room %i", sTemp, iCurRoom);
			ShowImage (-5, iExtras + 10, screen, 93, 0, 0, 39, 19);
			break;
		case 2:
			snprintf (sText, MAX_TEXT, "%s room links", sTemp); break;
		case 3:
			snprintf (sText, MAX_TEXT, "%s events", sTemp); break;
	}

	message = TTF_RenderText_Shaded (font1, sText, color_bl, color_wh);
	offset.x = 31;
	offset.y = 5;
	CustomBlitSurface (message, NULL, screen, &offset, arFont);

	/*** refresh screen ***/
	if (SDL_Flip (screen) == -1)
		{ printf ("[FAILED] SDL_Flip error!\n"); exit (EXIT_ERROR); }
}
/*****************************************************************************/
void Prev (int iCurLevel)
/*****************************************************************************/
{
	int iToLoad;

	if (iCurLevel != 1)
	{
		iToLoad = iCurLevel - 1;
		if (iToLoad == -1) { iToLoad = 15; }
		LoadLevel (iToLoad);
		PlaySound ("wav/level_change.wav");
	}
}
/*****************************************************************************/
void Next (int iCurLevel)
/*****************************************************************************/
{
	int iToLoad;

	if (iCurLevel != 0)
	{
		iToLoad = iCurLevel + 1;
		if (iToLoad == 16) { iToLoad = 0; }
		LoadLevel (iToLoad);
		PlaySound ("wav/level_change.wav");
	}
}
/*****************************************************************************/
void WhereToStart (void)
/*****************************************************************************/
{
	int iTemp;
	iMinX = 0;
	iMaxX = 0;
	iMinY = 0;
	iMaxY = 0;

	for (iTemp = 1; iTemp <= ROOMS; iTemp++)
	{
		iDone[iTemp] = 0;
	}
	CheckSides ((int)luKidRoom, 0, 0);

	iStartRoomsX = 12 - round ((iMinX + iMaxX) / 2);
	iStartRoomsY = 12 - round ((iMinY + iMaxY) / 2);
}
/*****************************************************************************/
void CheckSides (int iRoom, int iX, int iY)
/*****************************************************************************/
{
	if (iX < iMinX) { iMinX = iX; }
	if (iY < iMinY) { iMinY = iY; }
	if (iX > iMaxX) { iMaxX = iX; }
	if (iY > iMaxY) { iMaxY = iY; }

	iDone[iRoom] = 1;

	if ((iRoomConnections[iRoom][1] != 0) && (iDone[iRoomConnections[iRoom][1]]
		!= 1)) { CheckSides (iRoomConnections[iRoom][1], iX - 1, iY); }
	if ((iRoomConnections[iRoom][2] != 0) && (iDone[iRoomConnections[iRoom][2]]
		!= 1)) { CheckSides (iRoomConnections[iRoom][2], iX + 1, iY); }
	if ((iRoomConnections[iRoom][3] != 0) && (iDone[iRoomConnections[iRoom][3]]
		!= 1)) { CheckSides (iRoomConnections[iRoom][3], iX, iY - 1); }
	if ((iRoomConnections[iRoom][4] != 0) && (iDone[iRoomConnections[iRoom][4]]
		!= 1)) { CheckSides (iRoomConnections[iRoom][4], iX, iY + 1); }
}
/*****************************************************************************/
void ShowRooms (int iRoom, int iX, int iY, int iNext)
/*****************************************************************************/
{
	char sOnGrid[MAX_GRID + 2];
	char sTemp[MAX_GRID + 2];

	if (iX < 10)
	{
		snprintf (sTemp, MAX_GRID, "10%i", iX);
	} else {
		snprintf (sTemp, MAX_GRID, "1%i", iX);
	}
	if (iY < 10)
	{
		snprintf (sOnGrid, MAX_GRID, "%s0%i", sTemp, iY);
	} else {
		snprintf (sOnGrid, MAX_GRID, "%s%i", sTemp, iY);
	}
	if (iRoom != -1)
	{
		ShowImage (-6, iRoom, screen, atoi (sOnGrid), 0, 0, 14, 14);
		if (iCurRoom == iRoom)
		{
			ShowImage (-7, 0, screen, atoi (sOnGrid), 0, 0, 14, 14);
		}
		if ((int)luKidRoom == iRoom)
		{
			ShowImage (-7, 1, screen, atoi (sOnGrid), 0, 0, 14, 14);
		}
		if (iRoom == iMovingRoom)
		{
			ShowImage (-7, 2, screen, atoi (sOnGrid), 0, 0, 14, 14);
		}
	} else {
		ShowImage (-7, 3, screen, atoi (sOnGrid), 0, 0, 14, 14);
	}
	if (iRoom == iMovingRoom)
	{
		iMovingOldX = iX;
		iMovingOldY = iY;
		if (iMovingNewBusy == 0)
		{
			iMovingNewX = iMovingOldX;
			iMovingNewY = iMovingOldY;
			iMovingNewBusy = 1;
		}
	}

	iDone[iRoom] = 1;

	if (iNext == 1)
	{
		if ((iRoomConnections[iRoom][1] != 0) && (iDone[iRoomConnections[iRoom][1]]
			!= 1)) { ShowRooms (iRoomConnections[iRoom][1], iX - 1, iY, 1); }
		if ((iRoomConnections[iRoom][2] != 0) && (iDone[iRoomConnections[iRoom][2]]
			!= 1)) { ShowRooms (iRoomConnections[iRoom][2], iX + 1, iY, 1); }
		if ((iRoomConnections[iRoom][3] != 0) && (iDone[iRoomConnections[iRoom][3]]
			!= 1)) { ShowRooms (iRoomConnections[iRoom][3], iX, iY - 1, 1); }
		if ((iRoomConnections[iRoom][4] != 0) && (iDone[iRoomConnections[iRoom][4]]
			!= 1)) { ShowRooms (iRoomConnections[iRoom][4], iX, iY + 1, 1); }
	}
}
/*****************************************************************************/
void InitPopUp (int iAtScreen, int iStart)
/*****************************************************************************/
{
	SDL_Event event;
	int iPopUp;

	iPopUp = 1;

	if (iStart == 1) { PlaySound ("wav/popup.wav"); }
	ShowPopUp();
	while (iPopUp == 1)
	{
		while (SDL_PollEvent (&event))
		{
			switch (event.type)
			{
				case SDL_KEYDOWN:
					switch (event.key.keysym.sym)
					{
						case SDLK_ESCAPE:
						case SDLK_KP_ENTER:
						case SDLK_RETURN:
						case SDLK_SPACE:
						case SDLK_o:
							iPopUp = 0;
						default: break;
					}
					break;
				case SDL_MOUSEMOTION:
					SDL_GetMouseState (&iXPos, &iYPos);
					break;
				case SDL_MOUSEBUTTONDOWN:
					if (event.button.button == 1)
					{
						if (InArea (event, 440, 317, 440 + 85, 317 + 32) == 1) /*** OK ***/
						{
							if (iOKOn != 1) { iOKOn = 1; }
							ShowPopUp();
						}
					}
					break;
				case SDL_MOUSEBUTTONUP:
					iOKOn = 0;
					if (event.button.button == 1)
					{
						if (InArea (event, 440, 317, 440 + 85, 317 + 32) == 1) /*** OK ***/
						{
							iPopUp = 0;
						}
					}
					ShowPopUp(); break;
				case SDL_QUIT:
					Quit(); break;
			}
		}

		/*** prevent CPU eating ***/
		gamespeed = REFRESH;
		while ((SDL_GetTicks() - looptime) < gamespeed)
		{
			SDL_Delay (10);
		}
		looptime = SDL_GetTicks();
	}
	PlaySound ("wav/popup_close.wav");
	ShowScreen (iAtScreen);
}
/*****************************************************************************/
void ShowPopUp (void)
/*****************************************************************************/
{
	char sText[9 + 2][MAX_TEXT + 2];

	ShowImage (-9, 1, screen, 31, 0, 0, 692, 455); /*** faded background ***/
	ShowImage (-9, 2, screen, 44, 0, 0, 524, 361); /*** popup ***/
	switch (iOKOn)
	{
		case 0:
			ShowImage (-9, 3, screen, 45, 0, 0, 85, 32); break; /*** OK off ***/
		case 1:
			ShowImage (-9, 4, screen, 45, 0, 0, 85, 32); break; /*** OK on ***/
	}

	snprintf (sText[0], MAX_TEXT, "%s %s", EDITOR_NAME, EDITOR_VERSION);
	snprintf (sText[1], MAX_TEXT, "%s", COPYRIGHT);
	snprintf (sText[2], MAX_TEXT, "%s", "");
	snprintf (sText[3], MAX_TEXT, "%s", "single tile (change or select)");
	snprintf (sText[4], MAX_TEXT, "%s", "entire room (clear or fill)");
	snprintf (sText[5], MAX_TEXT, "%s", "entire level (randomize or fill)");
	snprintf (sText[6], MAX_TEXT, "%s", "");
	snprintf (sText[7], MAX_TEXT, "%s", "You may use one guard per room.");
	snprintf (sText[8], MAX_TEXT, "%s", "The tile behavior may differ per"
		" level.");

	DisplayText (180, 118, FONT_SIZE_1, sText, 9);

	/*** refresh screen ***/
	if (SDL_Flip (screen) == -1)
		{ printf ("[FAILED] SDL_Flip error!\n"); exit (EXIT_ERROR); }
}
/*****************************************************************************/
void InitPopUpSave (int iAtScreen)
/*****************************************************************************/
{
	SDL_Event event;
	int iPopUp;

	iPopUp = 1;

	PlaySound ("wav/popup_yn.wav");
	ShowPopUpSave();
	while (iPopUp == 1)
	{
		while (SDL_PollEvent (&event))
		{
			switch (event.type)
			{
				case SDL_KEYDOWN:
					switch (event.key.keysym.sym)
					{
						case SDLK_ESCAPE:
						case SDLK_n:
							iPopUp = 0;
							break;
						case SDLK_y:
							CallSave (0);
							iPopUp = 0;
							break;
						default: break;
					}
					break;
				case SDL_MOUSEMOTION:
					SDL_GetMouseState (&iXPos, &iYPos);
					break;
				case SDL_MOUSEBUTTONDOWN:
					if (event.button.button == 1)
					{
						if (InArea (event, 440, 317, 440 + 85, 317 + 32) == 1)
						{ /*** Yes ***/
							if (iYesOn != 1) { iYesOn = 1; }
							ShowPopUpSave();
						}
						if (InArea (event, 167, 317, 167 + 85, 317 + 32) == 1)
						{ /*** No ***/
							if (iNoOn != 1) { iNoOn = 1; }
							ShowPopUpSave();
						}
					}
					break;
				case SDL_MOUSEBUTTONUP:
					iYesOn = 0;
					iNoOn = 0;
					if (event.button.button == 1)
					{
						if (InArea (event, 440, 317, 440 + 85, 317 + 32) == 1)
						{ /*** Yes ***/
							CallSave (0);
							iPopUp = 0;
						}
						if (InArea (event, 167, 317, 167 + 85, 317 + 32) == 1)
						{ /*** No ***/
							iPopUp = 0;
						}
					}
					ShowPopUpSave(); break;
				case SDL_QUIT:
					Quit(); break;
			}
		}

		/*** prevent CPU eating ***/
		gamespeed = REFRESH;
		while ((SDL_GetTicks() - looptime) < gamespeed)
		{
			SDL_Delay (10);
		}
		looptime = SDL_GetTicks();
	}
	PlaySound ("wav/popup_close.wav");
	ShowScreen (iAtScreen);
}
/*****************************************************************************/
void ShowPopUpSave (void)
/*****************************************************************************/
{
	char sText[2 + 2][MAX_TEXT + 2];

	ShowImage (-9, 1, screen, 31, 0, 0, 692, 455); /*** faded background ***/
	ShowImage (-9, 5, screen, 84, 0, 0, 392, 277); /*** popup_yn ***/
	switch (iYesOn)
	{
		case 0:
			ShowImage (-9, 6, screen, 85, 0, 0, 85, 32); break; /*** Yes off ***/
		case 1:
			ShowImage (-9, 7, screen, 85, 0, 0, 85, 32); break; /*** Yes on ***/
	}
	switch (iNoOn)
	{
		case 0:
			ShowImage (-9, 8, screen, 86, 0, 0, 85, 32); break; /*** No off ***/
		case 1:
			ShowImage (-9, 9, screen, 86, 0, 0, 85, 32); break; /*** No on ***/
	}

	snprintf (sText[0], MAX_TEXT, "%s", "There are unsaved changes.");
	snprintf (sText[1], MAX_TEXT, "%s", "Do you wish to save these?");

	DisplayText (180, 118, FONT_SIZE_1, sText, 2);

	/*** refresh screen ***/
	if (SDL_Flip (screen) == -1)
		{ printf ("[FAILED] SDL_Flip error!\n"); exit (EXIT_ERROR); }
}
/*****************************************************************************/
void DisplayText (int iStartX, int iStartY, int iFontSize,
	char sText[9 + 2][MAX_TEXT + 2], int iLines)
/*****************************************************************************/
{
	int iTemp;

	for (iTemp = 0; iTemp <= (iLines - 1); iTemp++)
	{
		message = TTF_RenderText_Shaded (font1, sText[iTemp], color_bl, color_wh);
		if ((iTemp == 3) || (iTemp == 4) || (iTemp == 5))
		{
			offset.x = iStartX + 20;
		} else {
			offset.x = iStartX;
		}
		offset.y = iStartY + (iTemp * (iFontSize + 4));
		CustomBlitSurface (message, NULL, screen, &offset, arFont);
	}
}
/*****************************************************************************/
void ShowPos (SDL_Rect loc, SDL_Rect dest, int iLocation,
	SDL_Surface *img_first[2 + 2], SDL_Surface *img_second[2 + 2])
/*****************************************************************************/
{
	if (iCurType == 1)
	{
		CustomBlitSurface (img_first[1], &loc, screen, &dest, arImage);
		if (iSelected == iLocation)
		{
			CustomBlitSurface (img_first[2], &loc, screen, &dest, arImage);
		}
	} else {
		CustomBlitSurface (img_second[1], &loc, screen, &dest, arImage);
		if (iSelected == iLocation)
		{
			CustomBlitSurface (img_second[2], &loc, screen, &dest, arImage);
		}
	}
}
/*****************************************************************************/
void ChangePos (int iLocation, SDL_Surface *screen)
/*****************************************************************************/
{
	SDL_Event event;
	int iChanging;
	int iTemp;
	int iTemp2;
	int iNowOn;
	int iOldXPos;
	int iOldYPos;

	iChanging = 1;
	iGuardSkill = 0;

	ShowChange (iLocation);
	while (iChanging == 1)
	{
		while (SDL_PollEvent (&event))
		{
			switch (event.type)
			{
				case SDL_KEYDOWN:
					iTemp = 0;
					switch (event.key.keysym.sym)
					{
						case SDLK_KP_ENTER:
						case SDLK_RETURN:
						case SDLK_SPACE:
							if ((event.key.keysym.mod & KMOD_LCTRL) ||
								(event.key.keysym.mod & KMOD_RCTRL))
							{
								if ((iOnTile >= 1) && (iOnTile <= 72))
								{
									for (iTemp2 = 1; iTemp2 <= ROOMS; iTemp2++)
									{
										for (iTemp = 1; iTemp <= 30; iTemp++)
											{ UseTile (iOnTile, iTemp, iTemp2); }
									}
									iChanging = 0;
									iChanged = 1;
								}
							} else if ((event.key.keysym.mod & KMOD_LSHIFT) ||
								(event.key.keysym.mod & KMOD_RSHIFT))
							{
								if ((iOnTile >= 1) && (iOnTile <= 72))
								{
									for (iTemp = 1; iTemp <= 30; iTemp++)
										{ UseTile (iOnTile, iTemp, iCurRoom); }
									iChanging = 0;
									iChanged = 1;
								}
							} else {
								if ((iOnTile >= 1) && (iOnTile <= 96))
								{
									UseTile (iOnTile, iLocation, iCurRoom);
									if (iOnTile <= 72) { iChanging = 0; }
									iChanged = 1;
								}
							}
							break;
						case SDLK_ESCAPE:
						case SDLK_q:
						case SDLK_c:
							iChanging = 0; break;
						case SDLK_0:
						case SDLK_KP0:
							if (iGuardSkill != 0) { iGuardSkill = 0; iTemp = 1; } break;
						case SDLK_1:
						case SDLK_KP1:
							if (iGuardSkill != 1) { iGuardSkill = 1; iTemp = 1; } break;
						case SDLK_2:
						case SDLK_KP2:
							if (iGuardSkill != 2) { iGuardSkill = 2; iTemp = 1; } break;
						case SDLK_3:
						case SDLK_KP3:
							if (iGuardSkill != 3) { iGuardSkill = 3; iTemp = 1; } break;
						case SDLK_4:
						case SDLK_KP4:
							if (iGuardSkill != 4) { iGuardSkill = 4; iTemp = 1; } break;
						case SDLK_5:
						case SDLK_KP5:
							if (iGuardSkill != 5) { iGuardSkill = 5; iTemp = 1; } break;
						case SDLK_6:
						case SDLK_KP6:
							if (iGuardSkill != 6) { iGuardSkill = 6; iTemp = 1; } break;
						case SDLK_7:
						case SDLK_KP7:
							if (iGuardSkill != 7) { iGuardSkill = 7; iTemp = 1; } break;
						case SDLK_8:
						case SDLK_KP8:
							if (iGuardSkill != 8) { iGuardSkill = 8; iTemp = 1; } break;
						case SDLK_9:
						case SDLK_KP9:
							if (iGuardSkill != 9) { iGuardSkill = 9; iTemp = 1; } break;
						case SDLK_a:
							if (iGuardSkill != 10) { iGuardSkill = 10; iTemp = 1; } break;
						case SDLK_b:
							if (iGuardSkill != 11) { iGuardSkill = 11; iTemp = 1; } break;
						case SDLK_LEFT:
							if ((event.key.keysym.mod & KMOD_LCTRL) ||
								(event.key.keysym.mod & KMOD_RCTRL))
							{
								ChangeEvent (-10, 1);
							} else if ((event.key.keysym.mod & KMOD_LSHIFT) ||
								(event.key.keysym.mod & KMOD_RSHIFT))
							{
								ChangeEvent (-1, 1);
							} else {
								do { iOnTile--; } while (IsDisabled (iOnTile) == 1);
								switch (iOnTile)
								{
									case 0:
									case 13:
									case 26:
									case 39:
									case 52:
									case 65:
										iOnTile+=13; break;
									case 78:
										iOnTile = 96; break;
								}
								while (IsDisabled (iOnTile) == 1) { iOnTile--; }
							}
							break;
						case SDLK_RIGHT:
							if ((event.key.keysym.mod & KMOD_LCTRL) ||
								(event.key.keysym.mod & KMOD_RCTRL))
							{
								ChangeEvent (10, 1);
							} else if ((event.key.keysym.mod & KMOD_LSHIFT) ||
								(event.key.keysym.mod & KMOD_RSHIFT))
							{
								ChangeEvent (1, 1);
							} else {
								do { iOnTile++; } while (IsDisabled (iOnTile) == 1);
								switch (iOnTile)
								{
									case 14:
									case 27:
									case 40:
									case 53:
									case 66:
									case 79:
										iOnTile-=13; break;
									case 97:
										iOnTile = 79; break;
								}
								while (IsDisabled (iOnTile) == 1) { iOnTile++; }
							}
							break;
						case SDLK_UP:
							switch (iOnTile)
							{
								case 1: iOnTile = 79; break;
								case 2: iOnTile = 81; break;
								case 3: iOnTile = 83; break;
								case 4: iOnTile = 85; break;
								case 5: iOnTile = 87; break;
								case 6: iOnTile = 89; break;
								case 7: iOnTile = 91; break;
								case 8: iOnTile = 93; break;
								case 9: iOnTile = 95; break;
								case 10: iOnTile = 96; break;
								case 11: iOnTile = 96; break;
								case 12: iOnTile = 96; break;
								case 13: iOnTile = 96; break;
								case 79: iOnTile = 66; break;
								case 80: iOnTile = 66; break;
								case 81: iOnTile = 67; break;
								case 82: iOnTile = 67; break;
								case 83: iOnTile = 68; break;
								case 84: iOnTile = 68; break;
								case 85: iOnTile = 69; break;
								case 86: iOnTile = 69; break;
								case 87: iOnTile = 70; break;
								case 88: iOnTile = 70; break;
								case 89: iOnTile = 71; break;
								case 90: iOnTile = 71; break;
								case 91: iOnTile = 72; break;
								case 92: iOnTile = 72; break;
								case 93: iOnTile = 73; break;
								case 94: iOnTile = 73; break;
								case 95: iOnTile = 74; break;
								case 96: iOnTile = 74; break;
								default: iOnTile-=13; break;
							}
							while (IsDisabled (iOnTile) == 1) { iOnTile--; }
							break;
						case SDLK_DOWN:
							switch (iOnTile)
							{
								case 66: iOnTile = 79; break;
								case 67: iOnTile = 81; break;
								case 68: iOnTile = 83; break;
								case 69: iOnTile = 85; break;
								case 70: iOnTile = 87; break;
								case 71: iOnTile = 89; break;
								case 72: iOnTile = 91; break;
								case 73: iOnTile = 93; break;
								case 74: iOnTile = 95; break;
								case 75: iOnTile = 96; break;
								case 76: iOnTile = 96; break;
								case 77: iOnTile = 96; break;
								case 78: iOnTile = 96; break;
								case 79: iOnTile = 1; break;
								case 80: iOnTile = 1; break;
								case 81: iOnTile = 2; break;
								case 82: iOnTile = 2; break;
								case 83: iOnTile = 3; break;
								case 84: iOnTile = 3; break;
								case 85: iOnTile = 4; break;
								case 86: iOnTile = 4; break;
								case 87: iOnTile = 5; break;
								case 88: iOnTile = 5; break;
								case 89: iOnTile = 6; break;
								case 90: iOnTile = 6; break;
								case 91: iOnTile = 7; break;
								case 92: iOnTile = 7; break;
								case 93: iOnTile = 8; break;
								case 94: iOnTile = 8; break;
								case 95: iOnTile = 9; break;
								case 96: iOnTile = 9; break;
								default: iOnTile+=13; break;
							}
							while (IsDisabled (iOnTile) == 1) { iOnTile--; }
							break;
						case SDLK_o:
							iChanging = ChangePosCustom (iLocation, screen); break;
						default: break;
					}
					if (iTemp == 1)
					{
						if (sGuardLocations[iCurRoom - 1] + 1 == iSelected)
						{
							sGuardSkills[iCurRoom - 1] = iGuardSkill;
							iChanged = 1;
						}
						PlaySound ("wav/check_box.wav");
					}
					ShowChange (iLocation);
					break;
				case SDL_MOUSEMOTION:
					iOldXPos = iXPos;
					iOldYPos = iYPos;
					SDL_GetMouseState (&iXPos, &iYPos);
					if ((iOldXPos == iXPos) && (iOldYPos == iYPos)) { break; }

					iNowOn = OnTile (event);
					if ((iOnTile != iNowOn) && (iNowOn != 0))
					{
						if (IsDisabled (iNowOn) == 0)
						{
							iOnTile = iNowOn;
							ShowChange (iLocation);
						}
					}
					break;
				case SDL_MOUSEBUTTONDOWN: /*** ChangePos ***/
					if (event.button.button == 1) /*** left mouse button ***/
					{
						if (InArea (event, 656, 0, 692, 387) == 1) /*** close ***/
						{
							if (iCloseOn != 1) { iCloseOn = 1; }
							ShowChange (iLocation);
						}
						if (InArea (event, 656, 387, 692, 455) == 1) /*** custom ***/
						{
							if (iCustomOn != 1) { iCustomOn = 1; }
							ShowChange (iLocation);
						}
					}
					break;
				case SDL_MOUSEBUTTONUP:
					iCloseOn = 0;
					iCustomOn = 0;
					if (event.button.button == 1) /*** left mouse button, tile ***/
					{
						iTemp = 0;
						if (InArea (event, 455, 420, 468, 433) == 1)
							if (iGuardSkill != 0) { iGuardSkill = 0; iTemp = 1; }
						if (InArea (event, 470, 420, 483, 433) == 1)
							if (iGuardSkill != 1) { iGuardSkill = 1; iTemp = 1; }
						if (InArea (event, 485, 420, 498, 433) == 1)
							if (iGuardSkill != 2) { iGuardSkill = 2; iTemp = 1; }
						if (InArea (event, 500, 420, 513, 433) == 1)
							if (iGuardSkill != 3) { iGuardSkill = 3; iTemp = 1; }
						if (InArea (event, 515, 420, 528, 433) == 1)
							if (iGuardSkill != 4) { iGuardSkill = 4; iTemp = 1; }
						if (InArea (event, 455, 435, 468, 448) == 1)
							if (iGuardSkill != 5) { iGuardSkill = 5; iTemp = 1; }
						if (InArea (event, 470, 435, 483, 448) == 1)
							if (iGuardSkill != 6) { iGuardSkill = 6; iTemp = 1; }
						if (InArea (event, 485, 435, 498, 448) == 1)
							if (iGuardSkill != 7) { iGuardSkill = 7; iTemp = 1; }
						if (InArea (event, 500, 435, 513, 448) == 1)
							if (iGuardSkill != 8) { iGuardSkill = 8; iTemp = 1; }
						if (InArea (event, 515, 435, 528, 448) == 1)
							if (iGuardSkill != 9) { iGuardSkill = 9; iTemp = 1; }
						if (InArea (event, 500, 405, 513, 418) == 1)
							if (iGuardSkill != 10) { iGuardSkill = 10; iTemp = 1; }
						if (InArea (event, 515, 405, 528, 418) == 1)
							if (iGuardSkill != 11) { iGuardSkill = 11; iTemp = 1; }
						if (iTemp == 1)
						{
							if (sGuardLocations[iCurRoom - 1] + 1 == iSelected)
							{
								sGuardSkills[iCurRoom - 1] = iGuardSkill;
								iChanged = 1;
							}
							PlaySound ("wav/check_box.wav");
						}

						/*** Changing the event number. ***/
						if (InArea (event, 534, 424, 546, 443) == 1)
							{ ChangeEvent (-10, 1); }
						if (InArea (event, 549, 424, 561, 443) == 1)
							{ ChangeEvent (-1, 1); }
						if (InArea (event, 619, 424, 631, 443) == 1)
							{ ChangeEvent (1, 1); }
						if (InArea (event, 634, 424, 646, 443) == 1)
							{ ChangeEvent (10, 1); }

						/*** On close: exit ChangePos. ***/
						if (InArea (event, 656, 0, 692, 387) == 1) { iChanging = 0; }

						/*** On custom, mouse button up. ***/
						if (InArea (event, 656, 387, 692, 455) == 1)
							{ iChanging = ChangePosCustom (iLocation, screen); }

						/*** Tile or human. ***/
						if ((InArea (event, 2, 2, 652, 386) == 1) ||
							(InArea (event, 2, 386, 454, 452) == 1))
						{
							UseTile (iOnTile, iLocation, iCurRoom);
							if (iOnTile <= 72) { iChanging = 0; }
							iChanged = 1;
						}
					}

					if (event.button.button == 2)
					{
						if ((InArea (event, 656, 0, 692, 455) != 1)
							&& ((iOnTile >= 1) && (iOnTile <= 72)))
						{
							for (iTemp = 1; iTemp <= 30; iTemp++)
							{
								UseTile (iOnTile, iTemp, iCurRoom);
							}
							iChanging = 0;
							iChanged = 1;
						}
					}

					if (event.button.button == 3)
					{
						if ((InArea (event, 656, 0, 692, 455) != 1)
							&& ((iOnTile >= 1) && (iOnTile <= 72)))
						{
							for (iTemp2 = 1; iTemp2 <= ROOMS; iTemp2++)
							{
								for (iTemp = 1; iTemp <= 30; iTemp++)
								{
									UseTile (iOnTile, iTemp, iTemp2);
								}
							}
							iChanging = 0;
							iChanged = 1;
						}
					}

					ShowChange (iLocation);
					break;
				case SDL_QUIT:
					Quit(); break;
			}
		}

		/*** prevent CPU eating ***/
		gamespeed = REFRESH;
		while ((SDL_GetTicks() - looptime) < gamespeed)
		{
			SDL_Delay (10);
		}
		looptime = SDL_GetTicks();
	}
	PlaySound ("wav/ok_close.wav");
}
/*****************************************************************************/
int ChangePosCustom (int iLocation, SDL_Surface *screen)
/*****************************************************************************/
{
	SDL_Event event;
	int iChanging;
	int iChangingCustom;
	int iTemp;

	iChanging = 1;
	iChangingCustom = 1;

	/*** Set iChangeForeRand, iChangeForeMod and iChangeFore. ***/
	iTemp = iThingA[iCurRoom][iLocation - 1];
	iChangeForeRand = 0;
	if (iTemp >= 192) { iTemp-=192; iChangeForeRand = 3; }
	if (iTemp >= 128) { iTemp-=128; iChangeForeRand = 2; }
	if (iTemp >= 64) { iTemp-=64; iChangeForeRand = 1; }
	if (iTemp >= 32) { iTemp-=32; iChangeForeMod = 1; }
		else { iChangeForeMod = 0; }
	iChangeFore = iTemp;

	/*** Set iChangeMod. ***/
	iTemp = iModifierA[iCurRoom][iLocation - 1];
	if (iTemp == -1) { iTemp = 255; }
	iChangeMod = iTemp;

	ShowChangeCustom (iLocation);
	while (iChangingCustom == 1)
	{
		while (SDL_PollEvent (&event))
		{
			switch (event.type)
			{
				case SDL_KEYDOWN:
					switch (event.key.keysym.sym)
					{
						case SDLK_KP_ENTER:
						case SDLK_RETURN:
						case SDLK_SPACE:
							iTemp = 0;
							switch (iChangeForeRand)
							{
								case 0: break;
								case 1: iTemp+=64; break;
								case 2: iTemp+=128; break;
								case 3: iTemp+=192; break;
							}
							if (iChangeForeMod == 1) { iTemp+=32; }
							iTemp+=iChangeFore;
							SetLocation (iCurRoom, iLocation, iTemp, iChangeMod);
							iChanged = 1;
							iChangingCustom = 0;
							iChanging = 0;
							break;
						case SDLK_n:
							if (iChangeForeMod != 0)
							{
								iChangeForeMod = 0;
								PlaySound ("wav/check_box.wav");
							}
							break;
						case SDLK_y:
							if (iChangeForeMod != 1)
							{
								iChangeForeMod = 1;
								PlaySound ("wav/check_box.wav");
							}
							break;
						case SDLK_ESCAPE:
						case SDLK_q:
						case SDLK_c:
							iChangingCustom = 0; break;
						case SDLK_0:
						case SDLK_KP0:
							if (iChangeForeRand != 0)
								{ iChangeForeRand = 0; PlaySound ("wav/check_box.wav"); }
							break;
						case SDLK_1:
						case SDLK_KP1:
							if (iChangeForeRand != 1)
								{ iChangeForeRand = 1; PlaySound ("wav/check_box.wav"); }
							break;
						case SDLK_2:
						case SDLK_KP2:
							if (iChangeForeRand != 2)
								{ iChangeForeRand = 2; PlaySound ("wav/check_box.wav"); }
							break;
						case SDLK_3:
						case SDLK_KP3:
							if (iChangeForeRand != 3)
								{ iChangeForeRand = 3; PlaySound ("wav/check_box.wav"); }
							break;
						case SDLK_LEFT:
							if ((event.key.keysym.mod & KMOD_LCTRL) ||
								(event.key.keysym.mod & KMOD_RCTRL))
							{
								ChangeCustom (-10, 2);
							} else if ((event.key.keysym.mod & KMOD_LSHIFT) ||
								(event.key.keysym.mod & KMOD_RSHIFT))
							{
								ChangeCustom (-1, 2);
							} else {
								ChangeCustom (-1, 1);
							}
							break;
						case SDLK_RIGHT:
							if ((event.key.keysym.mod & KMOD_LCTRL) ||
								(event.key.keysym.mod & KMOD_RCTRL))
							{
								ChangeCustom (10, 2);
							} else if ((event.key.keysym.mod & KMOD_LSHIFT) ||
								(event.key.keysym.mod & KMOD_RSHIFT))
							{
								ChangeCustom (1, 2);
							} else {
								ChangeCustom (1, 1);
							}
							break;
						default: break;
					}
					ShowChangeCustom (iLocation);
					break;
				case SDL_MOUSEMOTION:
					SDL_GetMouseState (&iXPos, &iYPos);
					break;
				case SDL_MOUSEBUTTONDOWN:
					if (event.button.button == 1)
					{
						if (InArea (event, 656, 0, 692, 387) == 1) /*** close ***/
						{
							if (iCloseCustomOn != 1) { iCloseCustomOn = 1; }
							ShowChangeCustom (iLocation);
						}
						if (InArea (event, 656, 387, 692, 455) == 1) /*** use ***/
						{
							if (iCustomUseOn != 1) { iCustomUseOn = 1; }
							ShowChangeCustom (iLocation);
						}
					}
					break;
				case SDL_MOUSEBUTTONUP:
					iCloseCustomOn = 0;
					iCustomUseOn = 0;
					if (event.button.button == 1) /*** left mouse button ***/
					{
						/*** foreground random ***/
						iTemp = 0;
						if (InArea (event, 33, 114, 46, 127) == 1)
							if (iChangeForeRand != 0) { iChangeForeRand = 0; iTemp = 1; }
						if (InArea (event, 48, 114, 61, 127) == 1)
							if (iChangeForeRand != 1) { iChangeForeRand = 1; iTemp = 1; }
						if (InArea (event, 63, 114, 76, 127) == 1)
							if (iChangeForeRand != 2) { iChangeForeRand = 2; iTemp = 1; }
						if (InArea (event, 78, 114, 91, 127) == 1)
							if (iChangeForeRand != 3) { iChangeForeRand = 3; iTemp = 1; }
						if (iTemp == 1) { PlaySound ("wav/check_box.wav"); }

						/*** foreground mod ***/
						if ((InArea (event, 114, 114, 127, 127) == 1) &&
							(iChangeForeMod != 0))
						{
							iChangeForeMod = 0;
							PlaySound ("wav/check_box.wav");
						}
						if ((InArea (event, 129, 114, 142, 127) == 1) &&
							(iChangeForeMod != 1))
						{
							iChangeForeMod = 1;
							PlaySound ("wav/check_box.wav");
						}

						/*** foreground code ***/
						if (InArea (event, 164, 109, 176, 128) == 1)
							{ ChangeCustom (-1, 1); }
						if (InArea (event, 234, 109, 246, 128) == 1)
							{ ChangeCustom (1, 1); }

						/*** modifier code ***/
						if (InArea (event, 409, 109, 421, 128) == 1)
							{ ChangeCustom (-10, 2); }
						if (InArea (event, 424, 109, 436, 128) == 1)
							{ ChangeCustom (-1, 2); }
						if (InArea (event, 494, 109, 506, 128) == 1)
							{ ChangeCustom (1, 2); }
						if (InArea (event, 509, 109, 521, 128) == 1)
							{ ChangeCustom (10, 2); }

						/*** CLOSE ***/
						if (InArea (event, 656, 0, 692, 387) == 1)
							{ iChangingCustom = 0; }

						/*** USE ***/
						if (InArea (event, 656, 387, 692, 455) == 1)
						{
							iTemp = 0;
							switch (iChangeForeRand)
							{
								case 0: break;
								case 1: iTemp+=64; break;
								case 2: iTemp+=128; break;
								case 3: iTemp+=192; break;
							}
							if (iChangeForeMod == 1) { iTemp+=32; }
							iTemp+=iChangeFore;
							SetLocation (iCurRoom, iLocation, iTemp, iChangeMod);
							iChanged = 1;
							iChangingCustom = 0;
							iChanging = 0;
						}
					}

					ShowChangeCustom (iLocation);
					break;
				case SDL_QUIT:
					Quit(); break;
			}
		}

		/*** prevent CPU eating ***/
		gamespeed = REFRESH;
		while ((SDL_GetTicks() - looptime) < gamespeed)
		{
			SDL_Delay (10);
		}
		looptime = SDL_GetTicks();
	}
	if (iChanging == 1) { PlaySound ("wav/ok_close.wav"); }

	return (iChanging);
}
/*****************************************************************************/
void ChangeEvent (int iAmount, int iChangePos)
/*****************************************************************************/
{
	if (((iAmount > 0) && (iChangeEvent != 255)) ||
		((iAmount < 0) && (iChangeEvent != 0)))
	{
		switch (iAmount)
		{
			case -10: iChangeEvent-=10; break;
			case -1: iChangeEvent--; break;
			case 1: iChangeEvent++; break;
			case 10: iChangeEvent+=10; break;
		}
		if (iChangeEvent < 0) { iChangeEvent = 0; }
		if (iChangeEvent > 255) { iChangeEvent = 255; }
		if (iChangePos == 1)
		{
			if ((iThingA[iCurRoom][iSelected - 1] == 6) ||
				(iThingA[iCurRoom][iSelected - 1] == 15) ||
				(iThingA[iCurRoom][iSelected - 1] == 38) ||
				(iThingA[iCurRoom][iSelected - 1] == 47))
			{
				iModifierA[iCurRoom][iSelected - 1] = iChangeEvent;
				iChanged = 1;
			}
		}
		PlaySound ("wav/plus_minus.wav");
	}
}
/*****************************************************************************/
void ChangeCustom (int iAmount, int iForeBack)
/*****************************************************************************/
{
	switch (iForeBack)
	{
		case 1: /*** fore ***/
			if (((iAmount == 1) && (iChangeFore != 31)) ||
				((iAmount == -1) && (iChangeFore != 0)))
			{
				switch (iAmount)
				{
					case -1: iChangeFore--; break;
					case 1: iChangeFore++; break;
				}
				PlaySound ("wav/plus_minus.wav");
			}
			break;
		case 2: /*** back ***/
			if (((iAmount > 0) && (iChangeMod != 255)) ||
				((iAmount < 0) && (iChangeMod != 0)))
			{
				switch (iAmount)
				{
					case -10: iChangeMod-=10; break;
					case -1: iChangeMod--; break;
					case 1: iChangeMod++; break;
					case 10: iChangeMod+=10; break;
				}
				if (iChangeMod < 0) { iChangeMod = 0; }
				if (iChangeMod > 255) { iChangeMod = 255; }
				PlaySound ("wav/plus_minus.wav");
			}
			break;
	}
}
/*****************************************************************************/
int IsDisabled (int iTile)
/*****************************************************************************/
{
	if (((iTile == 73) || (iTile == 74)) && ((int)luLevelNr != 12))
	{
		PrIfDe ("[  OK  ] State of shadow: disabled\n");
		return (1);
	}
	else if (((iTile == 75) || (iTile == 76)) && ((int)luLevelNr != 6))
	{
		PrIfDe ("[  OK  ] State of fat guard: disabled\n");
		return (1);
	}
	else if (((iTile == 77) || (iTile == 78)) && ((int)luLevelNr != 3))
	{
		PrIfDe ("[  OK  ] State of skeleton: disabled\n");
		return (1);
	}
	else if (((iTile == 95) || (iTile == 96)) && ((int)luLevelNr != 13))
	{
		PrIfDe ("[  OK  ] State of jaffar: disabled\n");
		return (1);
	}
	else if (((iTile >= 79) && (iTile <= 92)) &&
		(((int)luLevelNr == 14) || ((int)luLevelNr == 15) || ((int)luLevelNr ==
		3) || ((int)luLevelNr == 6) || ((int)luLevelNr == 13) || ((int)luLevelNr
		== 12)))
	{
		PrIfDe ("[  OK  ] State of guards: disabled\n");
		return (1);
	} else {
		return (0);
	}
}
/*****************************************************************************/
void UseTile (int iTile, int iLocation, int iRoom)
/*****************************************************************************/
{
	int iTemp;

	if (iTile == -1)
	{
		do {
			iTile = 1 + (int) (72.0 * RANDOM() / (RAND_MAX + 1.0));
		} while (((iCurType == 1) && ((iTile == 48) || (iTile == 64) ||
			(iTile == 65) || (iTile == 67) || (iTile == 68) || (iTile == 69) ||
			(iTile == 70))) || ((iCurType == 2) && (iTile == 62)));
	}

	/*** Make sure the disabled guards can't be used. ***/
	if (((int)luLevelNr != 12) && ((iTile == 73) || (iTile == 74))) { return; }
	if (((int)luLevelNr != 6) && ((iTile == 75) || (iTile == 76))) { return; }
	if (((int)luLevelNr != 3) && ((iTile == 77) || (iTile == 78))) { return; }
	if (((int)luLevelNr != 13) && ((iTile == 95) || (iTile == 96))) { return; }
	if (((int)luLevelNr == 12) || ((int)luLevelNr == 6)
		|| ((int)luLevelNr == 3) || ((int)luLevelNr == 13)
		|| ((int)luLevelNr == 14) || ((int)luLevelNr == 15))
	{
		if ((iTile >= 79) && (iTile <= 92)) { return; }
	}

	switch (iTile)
	{
		case 1: SetLocation (iRoom, iLocation, 0, 0); break;
		case 2: SetLocation (iRoom, iLocation, 0, 1); break;
		case 3: SetLocation (iRoom, iLocation, 0, 2); break;
		case 4: SetLocation (iRoom, iLocation, 0, 3); break;
		case 5: SetLocation (iRoom, iLocation, 0, 255); break;
		case 6: SetLocation (iRoom, iLocation, 1, 0); break;
		case 7: SetLocation (iRoom, iLocation, 1, 1); break;
		case 8: SetLocation (iRoom, iLocation, 1, 2); break;
		case 9: SetLocation (iRoom, iLocation, 1, 3); break;
		case 10: SetLocation (iRoom, iLocation, 1, 255); break;
		case 11: SetLocation (iRoom, iLocation, 2, 0); break;
		case 12: SetLocation (iRoom, iLocation, 2, 1); break;
		case 13: SetLocation (iRoom, iLocation, 2, 2); break;
		case 14: SetLocation (iRoom, iLocation, 2, 3); break;
		case 15: SetLocation (iRoom, iLocation, 2, 4); break;
		case 16: SetLocation (iRoom, iLocation, 2, 5); break;
		case 17: SetLocation (iRoom, iLocation, 2, 6); break;
		case 18: SetLocation (iRoom, iLocation, 2, 7); break;
		case 19: SetLocation (iRoom, iLocation, 2, 8); break;
		case 20: SetLocation (iRoom, iLocation, 2, 9); break;
		case 21: SetLocation (iRoom, iLocation, 3, 0); break;
		case 22: SetLocation (iRoom, iLocation, 4, 0); break;
		case 23: SetLocation (iRoom, iLocation, 4, 1); break;
		case 24: SetLocation (iRoom, iLocation, 5, 0); break;
		case 25: SetLocation (iRoom, iLocation, 6, iChangeEvent); break;
		case 26: SetLocation (iRoom, iLocation, 7, 0); break;
		case 27: SetLocation (iRoom, iLocation, 7, 1); break;
		case 28: SetLocation (iRoom, iLocation, 7, 2); break;
		case 29: SetLocation (iRoom, iLocation, 7, 3); break;
		case 30: SetLocation (iRoom, iLocation, 8, 0); break;
		case 31: SetLocation (iRoom, iLocation, 9, 0); break;
		case 32: SetLocation (iRoom, iLocation, 10, 0); break;
		case 33: SetLocation (iRoom, iLocation, 10, 1); break;
		case 34: SetLocation (iRoom, iLocation, 10, 2); break;
		case 35: SetLocation (iRoom, iLocation, 10, 3); break;
		case 36: SetLocation (iRoom, iLocation, 10, 4); break;
		case 37: SetLocation (iRoom, iLocation, 10, 5); break;
		case 38: SetLocation (iRoom, iLocation, 10, 6); break;
		case 39: SetLocation (iRoom, iLocation, 11, 0); break;
		case 40: SetLocation (iRoom, iLocation, 12, 0); break;
		case 41: SetLocation (iRoom, iLocation, 12, 1); break;
		case 42: SetLocation (iRoom, iLocation, 12, 2); break;
		case 43: SetLocation (iRoom, iLocation, 12, 3); break;
		case 44: SetLocation (iRoom, iLocation, 12, 4); break;
		case 45: SetLocation (iRoom, iLocation, 12, 5); break;
		case 46: SetLocation (iRoom, iLocation, 12, 6); break;
		case 47: SetLocation (iRoom, iLocation, 12, 7); break;
		case 48: SetLocation (iRoom, iLocation, 13, 0); break;
		case 49: SetLocation (iRoom, iLocation, 14, 0); break;
		case 50: SetLocation (iRoom, iLocation, 15, iChangeEvent); break;
		case 51: SetLocation (iRoom, iLocation, 16, 0); break;
		case 52: SetLocation (iRoom, iLocation, 17, 0); break;
		case 53: SetLocation (iRoom, iLocation, 18, 0); break;
		case 54: SetLocation (iRoom, iLocation, 18, 1); break;
		case 55: SetLocation (iRoom, iLocation, 18, 2); break;
		case 56: SetLocation (iRoom, iLocation, 18, 3); break;
		case 57: SetLocation (iRoom, iLocation, 18, 4); break;
		case 58: SetLocation (iRoom, iLocation, 18, 5); break;
		case 59: SetLocation (iRoom, iLocation, 19, 0); break;
		case 60: SetLocation (iRoom, iLocation, 20, 0); break;
		case 61: SetLocation (iRoom, iLocation, 20, 1); break;
		case 62: SetLocation (iRoom, iLocation, 21, 0); break;
		case 63: SetLocation (iRoom, iLocation, 22, 0); break;
		case 64: SetLocation (iRoom, iLocation, 23, 0); break;
		case 65: SetLocation (iRoom, iLocation, 24, 0); break;
		case 66: SetLocation (iRoom, iLocation, 25, 0); break;
		case 67: SetLocation (iRoom, iLocation, 26, 0); break;
		case 68: SetLocation (iRoom, iLocation, 27, 0); break;
		case 69: SetLocation (iRoom, iLocation, 28, 0); break;
		case 70: SetLocation (iRoom, iLocation, 29, 0); break;
		case 71: SetLocation (iRoom, iLocation, 30, 0); break;
		case 72: SetLocation (iRoom, iLocation, 43, 0); break;
		case 93: /*** kid, turned right ***/
			if ((luKidRoom != iCurRoom) || (luKidPos != iLocation) ||
				(luKidDir != 0))
			{
				luKidRoom = iCurRoom;
				luKidPos = iLocation;
				luKidDir = 0;
				PlaySound ("wav/hum_adj.wav");
			}
			break;
		case 94: /*** kid, turned left ***/
			if ((luKidRoom != iCurRoom) || (luKidPos != iLocation) ||
				(luKidDir != 255))
			{
				luKidRoom = iCurRoom;
				luKidPos = iLocation;
				luKidDir = 255;
				PlaySound ("wav/hum_adj.wav");
			}
			break;
		case 73: /*** shadow, turned right ***/
			if ((sGuardLocations[iCurRoom - 1] == iLocation - 1)
				&& (sGuardDirections[iCurRoom - 1] == 0))
				sGuardLocations[iCurRoom - 1] = 30; else
				SetGuard (iCurRoom, iLocation - 1, 0, iGuardSkill, -1); break;
		case 74: /*** shadow, turned left ***/
			if ((sGuardLocations[iCurRoom - 1] == iLocation - 1)
				&& (sGuardDirections[iCurRoom - 1] == 255))
				sGuardLocations[iCurRoom - 1] = 30; else
				SetGuard (iCurRoom, iLocation - 1, 255, iGuardSkill, -1); break;
		case 75: /*** fat, turned right ***/
			if ((sGuardLocations[iCurRoom - 1] == iLocation - 1)
				&& (sGuardDirections[iCurRoom - 1] == 0))
				sGuardLocations[iCurRoom - 1] = 30; else
				SetGuard (iCurRoom, iLocation - 1, 0, iGuardSkill, -1); break;
		case 76: /*** fat, turned left ***/
			if ((sGuardLocations[iCurRoom - 1] == iLocation - 1)
				&& (sGuardDirections[iCurRoom - 1] == 255))
				sGuardLocations[iCurRoom - 1] = 30; else
				SetGuard (iCurRoom, iLocation - 1, 255, iGuardSkill, -1); break;
		case 77: /*** skeleton, turned right ***/
			if ((sGuardLocations[iCurRoom - 1] == iLocation - 1)
				&& (sGuardDirections[iCurRoom - 1] == 0))
				sGuardLocations[iCurRoom - 1] = 30; else
				SetGuard (iCurRoom, iLocation - 1, 0, iGuardSkill, -1); break;
		case 78: /*** skeleton, turned left ***/
			if ((sGuardLocations[iCurRoom - 1] == iLocation - 1)
				&& (sGuardDirections[iCurRoom - 1] == 255))
				sGuardLocations[iCurRoom - 1] = 30; else
				SetGuard (iCurRoom, iLocation - 1, 255, iGuardSkill, -1); break;
		case 95: /*** jaffar, turned right ***/
			if ((sGuardLocations[iCurRoom - 1] == iLocation - 1)
				&& (sGuardDirections[iCurRoom - 1] == 0))
				sGuardLocations[iCurRoom - 1] = 30; else
				SetGuard (iCurRoom, iLocation - 1, 0, iGuardSkill, -1); break;
		case 96: /*** jaffar, turned left ***/
			if ((sGuardLocations[iCurRoom - 1] == iLocation - 1)
				&& (sGuardDirections[iCurRoom - 1] == 255))
				sGuardLocations[iCurRoom - 1] = 30; else
				SetGuard (iCurRoom, iLocation - 1, 255, iGuardSkill, -1); break;
		default: /*** normal guards ***/
			switch (iTile)
			{
				case 79: case 80: iTemp = 1; break;
				case 81: case 82: iTemp = 2; break;
				case 83: case 84: iTemp = 3; break;
				case 85: case 86: iTemp = 4; break;
				case 87: case 88: iTemp = 5; break;
				case 89: case 90: iTemp = 6; break;
				case 91: case 92: iTemp = 7; break;
			}
			switch (iTile)
			{
				case 79: case 81: case 83: case 85: case 87: case 89: case 91:
					if ((sGuardLocations[iCurRoom - 1] == iLocation - 1)
						&& (sGuardDirections[iCurRoom - 1] == 0) &&
						(sGuardColors[iCurRoom - 1] == iTemp))
						sGuardLocations[iCurRoom - 1] = 30; else
						SetGuard (iCurRoom, iLocation - 1, 0, iGuardSkill, iTemp); break;
				case 80: case 82: case 84: case 86: case 88: case 90: case 92:
					if ((sGuardLocations[iCurRoom - 1] == iLocation - 1)
						&& (sGuardDirections[iCurRoom - 1] == 255) &&
						(sGuardColors[iCurRoom - 1] == iTemp))
						sGuardLocations[iCurRoom - 1] = 30; else
						SetGuard (iCurRoom, iLocation - 1, 255, iGuardSkill, iTemp); break;
			}
			break;
	}
}
/*****************************************************************************/
void SetGuard (int iRoom, int iLoc, int iDirection, int iSkill, int iColor)
/*****************************************************************************/
{
	sGuardLocations[iCurRoom - 1] = iLoc;
	sGuardDirections[iCurRoom - 1] = iDirection;
	sGuardSkills[iCurRoom - 1] = iSkill;
	sGuardColors[iCurRoom - 1] = iColor;
	PlaySound ("wav/hum_adj.wav");
}
/*****************************************************************************/
void ShowChange (int iLocation)
/*****************************************************************************/
{
	char sText[MAX_TEXT];

	switch (iCurType)
	{
		case 1: ShowImage (-12, 0, screen, 31, 0, 0, 692, 455); break;
		case 2: ShowImage (-12, 1, screen, 31, 0, 0, 692, 455); break;
	}
	switch (iCloseOn)
	{
		case 0: ShowImage (-12, 2, screen, 47, 0, 0, 36, 387); break; /*** off ***/
		case 1: ShowImage (-12, 3, screen, 47, 0, 0, 36, 387); break; /*** on ***/
	}
	DisableSome ((int)luLevelNr);
	ShowImage (-12, 11, screen, 82, 0, 0, 52, 66); /*** old tile ***/
	switch (iCustomOn)
	{
		case 0:
			if (iIsCustom == 1)
				ShowImage (-12, 13, screen, 88, 0, 0, 36, 68);
			if (iIsCustom == 0)
				ShowImage (-12, 14, screen, 88, 0, 0, 36, 68);
			break;
		case 1:
			if (iIsCustom == 1)
				ShowImage (-12, 15, screen, 88, 0, 0, 36, 68);
			if (iIsCustom == 0)
				ShowImage (-12, 16, screen, 88, 0, 0, 36, 68);
			break;
	}
	if ((iCurRoom == (int)luKidRoom) && (iLocation == (int)luKidPos))
	{
		switch ((int)luKidDir)
		{
			case 0: ShowImage (-12, 10, screen, 57, 0, 0, 27, 66); break;
			case 255: ShowImage (-12, 10, screen, 58, 0, 0, 27, 66); break;
		}
	}
	if (iLocation == sGuardLocations[iCurRoom - 1] + 1)
	{
		iGuardSkill = sGuardSkills[iCurRoom - 1];
		switch ((int)luLevelNr)
		{
			case 3: /*** skeleton ***/
				if (sGuardDirections[iCurRoom - 1] == 0)
					ShowImage (-12, 11, screen, 73, 0, 0, 52, 66);
				if (sGuardDirections[iCurRoom - 1] == 255)
					ShowImage (-12, 11, screen, 74, 0, 0, 52, 66);
				break;
			case 6: /*** fat ***/
				if (sGuardDirections[iCurRoom - 1] == 0)
					ShowImage (-12, 11, screen, 75, 0, 0, 52, 66);
				if (sGuardDirections[iCurRoom - 1] == 255)
					ShowImage (-12, 11, screen, 76, 0, 0, 52, 66);
				break;
			case 13: /*** jaffar ***/
				if (sGuardDirections[iCurRoom - 1] == 0)
					ShowImage (-12, 10, screen, 77, 0, 0, 27, 66);
				if (sGuardDirections[iCurRoom - 1] == 255)
					ShowImage (-12, 10, screen, 78, 0, 0, 27, 66);
				break;
			case 12: /*** shadow ***/
				if (sGuardDirections[iCurRoom - 1] == 0)
					ShowImage (-12, 11, screen, 79, 0, 0, 52, 66);
				if (sGuardDirections[iCurRoom - 1] == 255)
					ShowImage (-12, 11, screen, 80, 0, 0, 52, 66);
				break;
			default:
				switch (sGuardColors[iCurRoom - 1] - 1)
				{
					case 0:
						if (sGuardDirections[iCurRoom - 1] == 0)
							ShowImage (-12, 10, screen, 59, 0, 0, 27, 66);
						if (sGuardDirections[iCurRoom - 1] == 255)
							ShowImage (-12, 10, screen, 60, 0, 0, 27, 66);
						break;
					case 1:
						if (sGuardDirections[iCurRoom - 1] == 0)
							ShowImage (-12, 10, screen, 61, 0, 0, 27, 66);
						if (sGuardDirections[iCurRoom - 1] == 255)
							ShowImage (-12, 10, screen, 62, 0, 0, 27, 66);
						break;
					case 2:
						if (sGuardDirections[iCurRoom - 1] == 0)
							ShowImage (-12, 10, screen, 63, 0, 0, 27, 66);
						if (sGuardDirections[iCurRoom - 1] == 255)
							ShowImage (-12, 10, screen, 64, 0, 0, 27, 66);
						break;
					case 3:
						if (sGuardDirections[iCurRoom - 1] == 0)
							ShowImage (-12, 10, screen, 65, 0, 0, 27, 66);
						if (sGuardDirections[iCurRoom - 1] == 255)
							ShowImage (-12, 10, screen, 66, 0, 0, 27, 66);
						break;
					case 4:
						if (sGuardDirections[iCurRoom - 1] == 0)
							ShowImage (-12, 10, screen, 67, 0, 0, 27, 66);
						if (sGuardDirections[iCurRoom - 1] == 255)
							ShowImage (-12, 10, screen, 68, 0, 0, 27, 66);
						break;
					case 5:
						if (sGuardDirections[iCurRoom - 1] == 0)
							ShowImage (-12, 10, screen, 69, 0, 0, 27, 66);
						if (sGuardDirections[iCurRoom - 1] == 255)
							ShowImage (-12, 10, screen, 70, 0, 0, 27, 66);
						break;
					case 6:
					default:
						if (sGuardDirections[iCurRoom - 1] == 0)
							ShowImage (-12, 10, screen, 71, 0, 0, 27, 66);
						if (sGuardDirections[iCurRoom - 1] == 255)
							ShowImage (-12, 10, screen, 72, 0, 0, 27, 66);
						break;
				}
				break;
		}
	}
	ShowImage (-12, 12, screen, 81, 0, 0, 14, 14); /*** guard skill ***/

	if (iOnTile != 0)
	{
		ShowImage (-12, 4, screen, 48, 0, 0, 52, 66);
	}

	/*** show event number ***/
	snprintf (sText, MAX_TEXT, "%i", iChangeEvent + 1);
	message = TTF_RenderText_Shaded (font3, sText, color_bl, color_wh);
	if ((iChangeEvent + 1 >= 1) && (iChangeEvent + 1 <= 9))
	{
		offset.x = 583;
	} else if ((iChangeEvent + 1 >= 10) && (iChangeEvent + 1 <= 99)) {
		offset.x = 576;
	} else {
		offset.x = 569;
	}
	offset.y = 423;
	CustomBlitSurface (message, NULL, screen, &offset, arFont);

	/*** refresh screen ***/
	if (SDL_Flip (screen) == -1)
		{ printf ("[FAILED] SDL_Flip error!\n"); exit (EXIT_ERROR); }
}
/*****************************************************************************/
void ShowChangeCustom (int iLocation)
/*****************************************************************************/
{
	char sText[MAX_TEXT];

	ShowImage (-12, 17, screen, 31, 0, 0, 692, 455);

	switch (iCloseCustomOn)
	{
		case 0: ShowImage (-12, 2, screen, 47, 0, 0, 36, 387); break; /*** off ***/
		case 1: ShowImage (-12, 3, screen, 47, 0, 0, 36, 387); break; /*** on ***/
	}
	switch (iCustomUseOn)
	{
		case 0: ShowImage (-12, 18, screen, 88, 0, 0, 36, 68); break; /*** off ***/
		case 1: ShowImage (-12, 19, screen, 88, 0, 0, 36, 68); break; /*** on ***/
	}

	ShowImage (-10, 1, screen, 89, 0, 0, 14, 14); /*** foreground random ***/
	ShowImage (-10, 1, screen, 90, 0, 0, 14, 14); /*** foreground mod ***/

	/*** foreground code ***/
	snprintf (sText, MAX_TEXT, "%i", iChangeFore);
	message = TTF_RenderText_Shaded (font3, sText, color_wh, color_bl);
	if ((iChangeFore >= 0) && (iChangeFore <= 9))
	{
		offset.x = 198;
	} else if ((iChangeFore >= 10) && (iChangeFore <= 31)) {
		offset.x = 191;
	}
	offset.y = 108;
	CustomBlitSurface (message, NULL, screen, &offset, arFont);

	/*** modifier code ***/
	snprintf (sText, MAX_TEXT, "%i", iChangeMod);
	message = TTF_RenderText_Shaded (font3, sText, color_wh, color_bl);
	if ((iChangeMod >= 0) && (iChangeMod <= 9))
	{
		offset.x = 458;
	} else if ((iChangeMod >= 10) && (iChangeMod <= 99)) {
		offset.x = 451;
	} else {
		offset.x = 444;
	}
	offset.y = 108;
	CustomBlitSurface (message, NULL, screen, &offset, arFont);

	/*** current custom ***/
	ShowCurrentCustom (screen);

	/*** refresh screen ***/
	if (SDL_Flip (screen) == -1)
		{ printf ("[FAILED] SDL_Flip error!\n"); exit (EXIT_ERROR); }
}
/*****************************************************************************/
void DisableSome (int iCurLevel)
/*****************************************************************************/
{
	if (iCurLevel != 3) /*** disable skeleton ***/
	{
		ShowImage (-12, 5, screen, 52, 0, 0, 102, 66);
	}
	if (iCurLevel != 6) /*** disable fat guard ***/
	{
		ShowImage (-12, 6, screen, 53, 0, 0, 102, 66);
	}
	if (iCurLevel != 13) /*** disable jaffar ***/
	{
		ShowImage (-12, 7, screen, 54, 0, 0, 52, 66);
	}
	if (iCurLevel != 12) /*** disable shadow ***/
	{
		ShowImage (-12, 8, screen, 55, 0, 0, 102, 66);
	}
	if ((iCurLevel == 14) || (iCurLevel == 15) /*** disable guards ***/
		|| (iCurLevel == 3) || (iCurLevel == 6)
		|| (iCurLevel == 13) || (iCurLevel == 12))
	{
		ShowImage (-12, 9, screen, 56, 0, 0, 352, 66);
	}
}
/*****************************************************************************/
int OnTile (SDL_Event event)
/*****************************************************************************/
{
	int iTempX;
	int iTempY;

	/*** Top six rows. ***/
	for (iTempY = 0; iTempY <= 5; iTempY++)
	{
		for (iTempX = 0; iTempX <= 12; iTempX++)
		{
			if (InArea (event, TX + (iTempX * 50), TY + (iTempY * 64),
				TX2 + ((iTempX + 1) * 50), TY2 + ((iTempY + 1) * 64)) == 1)
			{
				return ((iTempY * 13) + iTempX + 1);
			}
		}
	}

	/*** Humans in the bottom row. ***/
	for (iTempX = 0; iTempX <= 17; iTempX++)
	{
		if (InArea (event, TX + (iTempX * 25), 388, TX2 + ((iTempX + 1) * 25),
			449) == 1)
		{
			return (79 + iTempX);
		}
	}
	return (0);
}
/*****************************************************************************/
int Modulo (int iOne, int iTwo, int iType)
/*****************************************************************************/
{
	int iTemp;
	int iCount;

	iCount = 0;
	do {
		iTemp = iOne - iTwo;
		iOne = iTemp;
		iCount++;
	} while (iOne > 0);
	if (iType == 1)
	{
		if (iOne == 0)
		{
			return (iTwo); /*** The % modulo would return 0 here. ***/
		}
		else
		{
			return (iOne + iTwo);
		}
	} else {
		return (iCount);
	}
}
/*****************************************************************************/
void SetLocation (int iRoom, int iLocation, int iThing, int iModifier)
/*****************************************************************************/
{
	iThingA[iRoom][iLocation - 1] = iThing;
	iLastThing = iThing;
	iModifierA[iRoom][iLocation - 1] = iModifier;
	iLastModifier = iModifier;
}
/*****************************************************************************/
void CallSave (int iJustXML)
/*****************************************************************************/
{
	if (iJustXML != 1) { CreateBAK(); }
	switch ((int)luLevelNr)
	{
		case 0: if (iJustXML != 1) { SavePLV (OUT_DIR "/demo.plv"); }
			SaveXML (XML_DIR "/demo.xml"); break;
		case 1: if (iJustXML != 1) { SavePLV (OUT_DIR "/level1.plv"); }
			SaveXML (XML_DIR "/level1.xml"); break;
		case 2: if (iJustXML != 1) { SavePLV (OUT_DIR "/level2.plv"); }
			SaveXML (XML_DIR "/level2.xml"); break;
		case 3: if (iJustXML != 1) { SavePLV (OUT_DIR "/level3.plv"); }
			SaveXML (XML_DIR "/level3.xml"); break;
		case 4: if (iJustXML != 1) { SavePLV (OUT_DIR "/level4.plv"); }
			SaveXML (XML_DIR "/level4.xml"); break;
		case 5: if (iJustXML != 1) { SavePLV (OUT_DIR "/level5.plv"); }
			SaveXML (XML_DIR "/level5.xml"); break;
		case 6: if (iJustXML != 1) { SavePLV (OUT_DIR "/level6.plv"); }
			SaveXML (XML_DIR "/level6.xml"); break;
		case 7: if (iJustXML != 1) { SavePLV (OUT_DIR "/level7.plv"); }
			SaveXML (XML_DIR "/level7.xml"); break;
		case 8: if (iJustXML != 1) { SavePLV (OUT_DIR "/level8.plv"); }
			SaveXML (XML_DIR "/level8.xml"); break;
		case 9: if (iJustXML != 1) { SavePLV (OUT_DIR "/level9.plv"); }
			SaveXML (XML_DIR "/level9.xml"); break;
		case 10: if (iJustXML != 1) { SavePLV (OUT_DIR "/level10.plv"); }
			SaveXML (XML_DIR "/level10.xml"); break;
		case 11: if (iJustXML != 1) { SavePLV (OUT_DIR "/level11.plv"); }
			SaveXML (XML_DIR "/level11.xml"); break;
		case 12: if (iJustXML != 1) { SavePLV (OUT_DIR "/level12a.plv"); }
			SaveXML (XML_DIR "/level12a.xml"); break;
		case 13: if (iJustXML != 1) { SavePLV (OUT_DIR "/level12b.plv"); }
			SaveXML (XML_DIR "/level12b.xml"); break;
		case 14: if (iJustXML != 1) { SavePLV (OUT_DIR "/princess.plv"); }
			SaveXML (XML_DIR "/princess.xml"); break;
		case 15: if (iJustXML != 1) { SavePLV (OUT_DIR "/potions.plv"); }
			SaveXML (XML_DIR "/potions.xml"); break;
	}
}
/*****************************************************************************/
int Checksum (void)
/*****************************************************************************/
{
	int iSum;
	int iTemp;
	int iTemp2;

	iSum = 0;

	/*** Level Foreground ***/
	for (iTemp = 1; iTemp <= ROOMS; iTemp++)
	{
		for (iTemp2 = 0; iTemp2 <= 29; iTemp2++)
		{
			iSum += iThingA[iTemp][iTemp2];
		}
	}

	/*** Level Modifier ***/
	for (iTemp = 1; iTemp <= ROOMS; iTemp++)
	{
		for (iTemp2 = 0; iTemp2 <= 29; iTemp2++)
		{
			iSum += iModifierA[iTemp][iTemp2];
		}
	}

	for (iTemp = 0; iTemp < 256; iTemp++) { iSum += sFirstDoorEvents[iTemp]; }
	for (iTemp = 0; iTemp < 256; iTemp++) { iSum += sSecondDoorEvents[iTemp]; }
	for (iTemp = 1; iTemp <= 96; iTemp++)
	{
		iSum += iRoomConnections[Modulo (iTemp, 4, 0)][Modulo (iTemp, 4, 1)];
	}
	for (iTemp = 0; iTemp < 64; iTemp++) { iSum += sUnknownI[iTemp]; }
	iSum = iSum + (char)luKidRoom + (char)(luKidPos - 1);
	if (((int)luLevelNr == 1) || ((int)luLevelNr == 13)) /*** 2 of 4 ***/
	{
		if (luKidDir == 0) { iSum += (char)255; }
		else if (luKidDir == 255) { iSum += (char)0; }
	} else {
		iSum += (char)luKidDir;
	}
	switch ((int)luLevelNr)
	{
		case 15: /*** potions level ***/
			for (iTemp = 0; iTemp < 3; iTemp++) { iSum += sUnknownIIandIII[iTemp]; }
		default:
			for (iTemp = 0; iTemp < 4; iTemp++) { iSum += sUnknownIIandIII[iTemp]; }
	}
	for (iTemp = 0; iTemp < 24; iTemp++) { iSum += sGuardLocations[iTemp]; }
	for (iTemp = 0; iTemp < 24; iTemp++) { iSum += sGuardDirections[iTemp]; }
	for (iTemp = 0; iTemp < 48; iTemp++) { iSum += sUnknownIVaandIVb[iTemp]; }
	for (iTemp = 0; iTemp < 24; iTemp++) { iSum += sGuardSkills[iTemp]; }
	for (iTemp = 0; iTemp < 24; iTemp++) { iSum += sUnknownIVc[iTemp]; }
	for (iTemp = 0; iTemp < 24; iTemp++) { iSum += sGuardColors[iTemp]; }
	for (iTemp = 0; iTemp < 16; iTemp++) { iSum += sUnknownIVd[iTemp]; }
	for (iTemp = 0; iTemp < 2; iTemp++) { iSum += sEndCode[iTemp]; }

	return (255 - Modulo (iSum, 256, 1));
}
/*****************************************************************************/
void SavePLV (char *sFileName)
/*****************************************************************************/
{
	int iFd;
	int iTemp;
	int iTemp2;
	unsigned int iHex;
	char sToWrite[MAX_TOWRITE + 2];
	char sToTemp[MAX_TOWRITE + 2];

	/*** Open file. ***/
	iFd = open (sFileName, O_WRONLY|O_TRUNC|O_CREAT|O_BINARY, 0600);
	if (iFd == -1)
	{
		printf ("[FAILED] Could not create level file: %s!\n", strerror (errno));
		exit (EXIT_ERROR);
	}

	WriteCharByChar (iFd, sMagicID, 7);
	WriteCharByChar (iFd, sPOPVersion, 1);
	WriteCharByChar (iFd, sPLVVersion, 1);
	WriteCharByChar (iFd, sLevelNumber, 1);
	WriteCharByChar (iFd, sNumberOfFields, 4);
	WriteCharByChar (iFd, sLevelSize, 4);
	snprintf (sToWrite, MAX_TOWRITE, "%c", Checksum());
	write (iFd, sToWrite, 1);

	/*** Level Foreground ***/
	for (iTemp = 1; iTemp <= ROOMS; iTemp++)
	{
		for (iTemp2 = 0; iTemp2 <= 29; iTemp2++)
		{
			snprintf (sToWrite, MAX_TOWRITE, "%c", iThingA[iTemp][iTemp2]);
			write (iFd, sToWrite, 1);
		}
	}

	/*** Level Modifier ***/
	for (iTemp = 1; iTemp <= ROOMS; iTemp++)
	{
		for (iTemp2 = 0; iTemp2 <= 29; iTemp2++)
		{
			snprintf (sToWrite, MAX_TOWRITE, "%c", iModifierA[iTemp][iTemp2]);
			write (iFd, sToWrite, 1);
		}
	}

	WriteCharByChar (iFd, sFirstDoorEvents, 256);
	WriteCharByChar (iFd, sSecondDoorEvents, 256);
	for (iTemp = 1; iTemp <= 96; iTemp++)
	{
		snprintf (sToWrite, MAX_TOWRITE, "%c",
			iRoomConnections[Modulo (iTemp, 4, 0)][Modulo (iTemp, 4, 1)]);
		write (iFd, sToWrite, 1);
	}
	WriteCharByChar (iFd, sUnknownI, 64);
	snprintf (sToWrite, MAX_TOWRITE, "%c", (int)luKidRoom);
	write (iFd, sToWrite, 1);
	snprintf (sToWrite, MAX_TOWRITE, "%c", (int)luKidPos - 1);
	write (iFd, sToWrite, 1);
	snprintf (sToWrite, MAX_TOWRITE, "%c", (int)luKidDir);
	if (((int)luLevelNr == 1) || ((int)luLevelNr == 13)) /*** 3 of 4 ***/
	{
		if (luKidDir == 0) { snprintf (sToWrite, MAX_TOWRITE, "%c", 255); }
		else if (luKidDir == 255) { snprintf (sToWrite, MAX_TOWRITE, "%c", 0); }
	}
	write (iFd, sToWrite, 1);
	switch ((int)luLevelNr)
	{
		case 15: /*** potions level ***/
			WriteCharByChar (iFd, sUnknownIIandIII, 3); break;
		default:
			WriteCharByChar (iFd, sUnknownIIandIII, 4); break;
	}
	WriteCharByChar (iFd, sGuardLocations, 24);
	WriteCharByChar (iFd, sGuardDirections, 24);
	WriteCharByChar (iFd, sUnknownIVaandIVb, 48);
	WriteCharByChar (iFd, sGuardSkills, 24);
	WriteCharByChar (iFd, sUnknownIVc, 24);
	WriteCharByChar (iFd, sGuardColors, 24);
	WriteCharByChar (iFd, sUnknownIVd, 16);
	WriteCharByChar (iFd, sEndCode, 2);

	/*** This is pretty complicated. ***/
	iHex = 0;
	snprintf (sToTemp, MAX_TOWRITE, "%08x", WriteUserData (-1, 2));
	for (iTemp = 0; iTemp <= 3; iTemp++)
	{
		snprintf (sToWrite, MAX_TOWRITE, "%c%c", sToTemp[6 - (2 * iTemp)],
			sToTemp[7 - (2 * iTemp)]);
		sscanf (sToWrite, "%x", &iHex);
		snprintf (sToWrite, MAX_TOWRITE, "%c", iHex);
		write (iFd, sToWrite, 1);
	}

	WriteUserData (iFd, 1);

	close (iFd);

	if (system (PR_EXECUTABLE " -i -f --resource=" PR_RESOURCES
		" " LEVELS_DAT " > " DEVNULL) == -1)
	{
		printf ("[FAILED] Could not import the levels: %s!\n", strerror (errno));
		exit (EXIT_ERROR);
	}

	PlaySound ("wav/save.wav");

	iChanged = 0;
}
/*****************************************************************************/
int WriteUserData (int iFd, int iType)
/*****************************************************************************/
{
	int iTemp;
	char sToWrite[MAX_TOWRITE + 2];
	char sDateTime[MAX_DATETIME + 2];
	int iChars;

	iChars = 0;
	for (iTemp = 1; iTemp < iInformationNr; iTemp++)
	{
		snprintf (sToWrite, MAX_TOWRITE, "%s", sInformation[iTemp][0]);
		if (iType == 1) { write (iFd, sToWrite, strlen (sToWrite)); }
			else { iChars += strlen (sToWrite); }
		snprintf (sToWrite, MAX_TOWRITE, "%c", 0x00);
		if (iType == 1) { write (iFd, sToWrite, 1); }
			else { iChars += 1; }
		if (strcmp (sInformation[iTemp][0], "Editor Name") == 0)
		{
			snprintf (sToWrite, MAX_TOWRITE, "%s", EDITOR_NAME);
		}
		else if (strcmp (sInformation[iTemp][0], "Level Author") == 0)
		{
			snprintf (sToWrite, MAX_TOWRITE, "%s", sAuthor);
		}
		else if (strcmp (sInformation[iTemp][0], "Editor Version") == 0)
		{
			snprintf (sToWrite, MAX_TOWRITE, "%s", EDITOR_VERSION);
		}
		else if (strcmp (sInformation[iTemp][0], "Time Last Modified") == 0)
		{
			DateTime (sDateTime);
			snprintf (sToWrite, MAX_TOWRITE, "%s", sDateTime);
		}
		else
		{
			snprintf (sToWrite, MAX_TOWRITE, "%s", sInformation[iTemp][1]);
		}
		if (iType == 1) { write (iFd, sToWrite, strlen (sToWrite)); }
			else { iChars += strlen (sToWrite); }
		snprintf (sToWrite, MAX_TOWRITE, "%c", 0x00);
		if (iType == 1) { write (iFd, sToWrite, 1); }
			else { iChars += 1; }
	}

	return (iChars);
}
/*****************************************************************************/
void WriteCharByChar (int iFd, unsigned char *sString, int iLength)
/*****************************************************************************/
{
	int iTemp;
	char sToWrite[MAX_TOWRITE + 2];

	for (iTemp = 0; iTemp < iLength; iTemp++)
	{
		snprintf (sToWrite, MAX_TOWRITE, "%c", sString[iTemp]);
		write (iFd, sToWrite, 1);
	}
}
/*****************************************************************************/
void ShowUsage (void)
/*****************************************************************************/
{
	printf ("%s %s\n%s\n\n", EDITOR_NAME, EDITOR_VERSION, COPYRIGHT);
	printf ("Usage:\n");
	printf ("  %s [OPTIONS]\n\nOptions:\n", EDITOR_NAME);
	printf ("  -h, -?,    --help           display this help and exit\n");
	printf ("  -v,        --version        output version information and"
		" exit\n");
	printf ("  -x,        --xml            export all levels as XML and exit\n");
	printf ("  -d,        --debug          also show levels on the console\n");
	printf ("  -n,        --noaudio        do not play sound effects\n");
	printf ("  -i,        --improved       \"improved\" instead of"
		" \"megahit\"\n");
	printf ("  -c='code', --cheat='code'   \"code\" instead of \"megahit\"\n");
	printf ("  -a='name', --author='name'  use \"name\" as the author when"
		" saving\n");
	printf ("  -z,        --zoom           double the interface size\n");
	printf ("  -l=NR,     --level=NR       start in level NR\n\n");
	exit (EXIT_NORMAL);
}
/*****************************************************************************/
void PrIfDe (char *sString)
/*****************************************************************************/
{
	if (iDebug == 1) { printf ("%s", sString); }
}
/*****************************************************************************/
void MixAudio (void *unused, Uint8 *stream, int iLen)
/*****************************************************************************/
{
	int iTemp;
	Uint32 amount;

	for (iTemp = 0; iTemp < NUM_SOUNDS; iTemp++)
	{
		amount = (sounds[iTemp].dlen-sounds[iTemp].dpos);
		if (amount > iLen)
		{
			amount = iLen;
		}
		SDL_MixAudio (stream, &sounds[iTemp].data[sounds[iTemp].dpos], amount,
			SDL_MIX_MAXVOLUME);
		sounds[iTemp].dpos += amount;
	}
}
/*****************************************************************************/
void PlaySound (char *sFile)
/*****************************************************************************/
{
	int iIndex;
	SDL_AudioSpec wave;
	Uint8 *data;
	Uint32 dlen;
	SDL_AudioCVT cvt;

	if (iNoAudio == 1) { return; }
	for (iIndex = 0; iIndex < NUM_SOUNDS; iIndex++)
	{
		if (sounds[iIndex].dpos == sounds[iIndex].dlen)
		{
			break;
		}
	}
	if (iIndex == NUM_SOUNDS) { return; }

	if (SDL_LoadWAV (sFile, &wave, &data, &dlen) == NULL)
	{
		printf ("[FAILED] Could not load %s: %s!\n", sFile, SDL_GetError());
		exit (EXIT_ERROR);
	}
	SDL_BuildAudioCVT (&cvt, wave.format, wave.channels, wave.freq, AUDIO_S16, 2,
		22050);
	cvt.buf = (Uint8 *)malloc (dlen*cvt.len_mult);
	memcpy (cvt.buf, data, dlen);
	cvt.len = dlen;
	SDL_ConvertAudio (&cvt);
	SDL_FreeWAV (data);

	if (sounds[iIndex].data)
	{
		free(sounds[iIndex].data);
	}
	SDL_LockAudio();
	sounds[iIndex].data = cvt.buf;
	sounds[iIndex].dlen = cvt.len_cvt;
	sounds[iIndex].dpos = 0;
	SDL_UnlockAudio();
}
/*****************************************************************************/
int BrokenRoomLinks (int iRoomConnections[ROOMS + 2][4 + 2], int iPrint)
/*****************************************************************************/
{
	int iBroken;
	int iTemp;

	for (iTemp = 1; iTemp <= ROOMS; iTemp++)
	{
		iDone[iTemp] = 0;
		iRoomConnectionsBroken[iTemp][1] = 0;
		iRoomConnectionsBroken[iTemp][2] = 0;
		iRoomConnectionsBroken[iTemp][3] = 0;
		iRoomConnectionsBroken[iTemp][4] = 0;
	}
	CheckSides ((int)luKidRoom, 0, 0);
	iBroken = 0;

	for (iTemp = 1; iTemp <= ROOMS; iTemp++)
	{
		/*** If the room is in use... ***/
		if (iDone[iTemp] == 1)
		{
			/*** check left ***/
			if (iRoomConnections[iTemp][1] != 0)
			{
				if ((iRoomConnections[iTemp][1] == iTemp) ||
					(iRoomConnections[iRoomConnections[iTemp][1]][2] != iTemp))
				{
					iRoomConnectionsBroken[iTemp][1] = 1;
					iBroken = 1;
					if ((iDebug == 1) && (iPrint == 1))
					{
						printf ("[ INFO ] The left of room %i has a broken link.\n",
							iTemp);
					}
				}
			}
			/*** check right ***/
			if (iRoomConnections[iTemp][2] != 0)
			{
				if ((iRoomConnections[iTemp][2] == iTemp) ||
					(iRoomConnections[iRoomConnections[iTemp][2]][1] != iTemp))
				{
					iRoomConnectionsBroken[iTemp][2] = 1;
					iBroken = 1;
					if ((iDebug == 1) && (iPrint == 1))
					{
						printf ("[ INFO ] The right of room %i has a broken link.\n",
							iTemp);
					}
				}
			}
			/*** check up ***/
			if (iRoomConnections[iTemp][3] != 0)
			{
				if ((iRoomConnections[iTemp][3] == iTemp) ||
					(iRoomConnections[iRoomConnections[iTemp][3]][4] != iTemp))
				{
					iRoomConnectionsBroken[iTemp][3] = 1;
					iBroken = 1;
					if ((iDebug == 1) && (iPrint == 1))
					{
						printf ("[ INFO ] The top of room %i has a broken link.\n",
							iTemp);
					}
				}
			}
			/*** check down ***/
			if (iRoomConnections[iTemp][4] != 0)
			{
				if ((iRoomConnections[iTemp][4] == iTemp) ||
					(iRoomConnections[iRoomConnections[iTemp][4]][3] != iTemp))
				{
					iRoomConnectionsBroken[iTemp][4] = 1;
					iBroken = 1;
					if ((iDebug == 1) && (iPrint == 1))
					{
						printf ("[ INFO ] The bottom of room %i has a broken link.\n",
							iTemp);
					}
				}
			}
		}
	}

	return (iBroken);
}
/*****************************************************************************/
int BrokenRoomChange (int iRoom, int iSide)
/*****************************************************************************/
{
	char sOnAdj[MAX_ADJ + 2];

	if (iRoom < 10)
		{ snprintf (sOnAdj, MAX_ADJ, "20%i%i", iRoom, iSide); }
			else { snprintf (sOnAdj, MAX_ADJ, "2%i%i", iRoom, iSide); }

	return (atoi (sOnAdj));
}
/*****************************************************************************/
int MouseSelectAdj (SDL_Event event)
/*****************************************************************************/
{
	int iOnAdj;
	int iRoom;
	int iAdjBaseX;
	int iAdjBaseY;

	iOnAdj = 0;
	for (iRoom = 1; iRoom <= ROOMS; iRoom++)
	{
		switch (iRoom)
		{
			case 1: iAdjBaseX = ADJ_BASE_X + (63 * 0);
				iAdjBaseY = ADJ_BASE_Y + (63 * 0); break;
			case 2: iAdjBaseX = ADJ_BASE_X + (63 * 1);
				iAdjBaseY = ADJ_BASE_Y + (63 * 0); break;
			case 3: iAdjBaseX = ADJ_BASE_X + (63 * 2);
				iAdjBaseY = ADJ_BASE_Y + (63 * 0); break;
			case 4: iAdjBaseX = ADJ_BASE_X + (63 * 3);
				iAdjBaseY = ADJ_BASE_Y + (63 * 0); break;
			case 5: iAdjBaseX = ADJ_BASE_X + (63 * 0);
				iAdjBaseY = ADJ_BASE_Y + (63 * 1); break;
			case 6: iAdjBaseX = ADJ_BASE_X + (63 * 1);
				iAdjBaseY = ADJ_BASE_Y + (63 * 1); break;
			case 7: iAdjBaseX = ADJ_BASE_X + (63 * 2);
				iAdjBaseY = ADJ_BASE_Y + (63 * 1); break;
			case 8: iAdjBaseX = ADJ_BASE_X + (63 * 3);
				iAdjBaseY = ADJ_BASE_Y + (63 * 1); break;
			case 9: iAdjBaseX = ADJ_BASE_X + (63 * 0);
				iAdjBaseY = ADJ_BASE_Y + (63 * 2); break;
			case 10: iAdjBaseX = ADJ_BASE_X + (63 * 1);
				iAdjBaseY = ADJ_BASE_Y + (63 * 2); break;
			case 11: iAdjBaseX = ADJ_BASE_X + (63 * 2);
				iAdjBaseY = ADJ_BASE_Y + (63 * 2); break;
			case 12: iAdjBaseX = ADJ_BASE_X + (63 * 3);
				iAdjBaseY = ADJ_BASE_Y + (63 * 2); break;
			case 13: iAdjBaseX = ADJ_BASE_X + (63 * 0);
				iAdjBaseY = ADJ_BASE_Y + (63 * 3); break;
			case 14: iAdjBaseX = ADJ_BASE_X + (63 * 1);
				iAdjBaseY = ADJ_BASE_Y + (63 * 3); break;
			case 15: iAdjBaseX = ADJ_BASE_X + (63 * 2);
				iAdjBaseY = ADJ_BASE_Y + (63 * 3); break;
			case 16: iAdjBaseX = ADJ_BASE_X + (63 * 3);
				iAdjBaseY = ADJ_BASE_Y + (63 * 3); break;
			case 17: iAdjBaseX = ADJ_BASE_X + (63 * 0);
				iAdjBaseY = ADJ_BASE_Y + (63 * 4); break;
			case 18: iAdjBaseX = ADJ_BASE_X + (63 * 1);
				iAdjBaseY = ADJ_BASE_Y + (63 * 4); break;
			case 19: iAdjBaseX = ADJ_BASE_X + (63 * 2);
				iAdjBaseY = ADJ_BASE_Y + (63 * 4); break;
			case 20: iAdjBaseX = ADJ_BASE_X + (63 * 3);
				iAdjBaseY = ADJ_BASE_Y + (63 * 4); break;
			case 21: iAdjBaseX = ADJ_BASE_X + (63 * 0);
				iAdjBaseY = ADJ_BASE_Y + (63 * 5); break;
			case 22: iAdjBaseX = ADJ_BASE_X + (63 * 1);
				iAdjBaseY = ADJ_BASE_Y + (63 * 5); break;
			case 23: iAdjBaseX = ADJ_BASE_X + (63 * 2);
				iAdjBaseY = ADJ_BASE_Y + (63 * 5); break;
			case 24: iAdjBaseX = ADJ_BASE_X + (63 * 3);
				iAdjBaseY = ADJ_BASE_Y + (63 * 5); break;
		}
		if (InArea (event, iAdjBaseX + 1, iAdjBaseY + 16,
			iAdjBaseX + 15, iAdjBaseY + 30) == 1)
		{
			iChangingBrokenRoom = iRoom;
			iChangingBrokenSide = 1; /*** left ***/
			iOnAdj = 1;
			break;
		}
		if (InArea (event, iAdjBaseX + 31, iAdjBaseY + 16,
			iAdjBaseX + 45, iAdjBaseY + 30) == 1)
		{
			iChangingBrokenRoom = iRoom;
			iChangingBrokenSide = 2; /*** right ***/
			iOnAdj = 1;
			break;
		}
		if (InArea (event, iAdjBaseX + 16, iAdjBaseY + 1,
			iAdjBaseX + 30, iAdjBaseY + 14) == 1)
		{
			iChangingBrokenRoom = iRoom;
			iChangingBrokenSide = 3; /*** up ***/
			iOnAdj = 1;
			break;
		}
		if (InArea (event, iAdjBaseX + 16, iAdjBaseY + 31,
			iAdjBaseX + 30, iAdjBaseY + 45) == 1)
		{
			iChangingBrokenRoom = iRoom;
			iChangingBrokenSide = 4; /*** down ***/
			iOnAdj = 1;
			break;
		}
	}

	return (iOnAdj);
}
/*****************************************************************************/
void CustomBlitSurface (SDL_Surface* src, SDL_Rect* srcrect,
	SDL_Surface* dst, SDL_Rect *dstrect, int arOptions[1])
/*****************************************************************************/
{
	SDL_Rect stuff;
	SDL_Surface *src_rota;

	stuff.x = dstrect->x * iScale;
	stuff.y = dstrect->y * iScale;
	if (arOptions[0] == 1)
	{
		src_rota = rotozoomSurface (src, 0, iScale, 0);
	} else {
		src_rota = src;
	}
	SDL_BlitSurface (src_rota, srcrect, dst, &stuff);
	SDL_FreeSurface(src_rota);
}
/*****************************************************************************/
void LoadFonts (void)
/*****************************************************************************/
{
	font1 = TTF_OpenFont ("ttf/lazy.ttf", FONT_SIZE_1 * iScale);
	if (font1 == NULL) { printf ("[FAILED] Font gone!\n"); exit (EXIT_ERROR); }
	font2 = TTF_OpenFont ("ttf/lazy.ttf", FONT_SIZE_2 * iScale);
	if (font2 == NULL) { printf ("[FAILED] Font gone!\n"); exit (EXIT_ERROR); }
	font3 = TTF_OpenFont ("ttf/lazy.ttf", FONT_SIZE_3 * iScale);
	if (font3 == NULL) { printf ("[FAILED] Font gone!\n"); exit (EXIT_ERROR); }
}
/*****************************************************************************/
void CustomTile (int iThing, int iModifier, char *sRetString)
/*****************************************************************************/
{
	int iTemp;
	char cRandom;
	char cMod;

	iTemp = iThing;
	cRandom = '0';
	if (iTemp >= 192) { iTemp-=192; cRandom = '3'; }
	if (iTemp >= 128) { iTemp-=128; cRandom = '2'; }
	if (iTemp >= 64) { iTemp-=64; cRandom = '1'; }
	if (iTemp >= 32) { iTemp-=32; cMod = 'Y'; }
		else { cMod = 'N'; }
	snprintf (sRetString, MAX_CUSTOM, "%c%c%i/%i",
		cRandom, cMod, iTemp, iModifier);
}
/*****************************************************************************/
void ShowCurrentCustom (SDL_Surface *screen)
/*****************************************************************************/
{
	int iTemp;

	iTemp = 0;
	switch (iChangeForeRand)
	{
		case 0: break;
		case 1: iTemp+=64; break;
		case 2: iTemp+=128; break;
		case 3: iTemp+=192; break;
	}
	if (iChangeForeMod == 1) { iTemp+=32; }
	iTemp+=iChangeFore;

	ShowImage (iTemp, iChangeMod, screen, 91, 0, 0, 120, 155);
}
/*****************************************************************************/
void CreateBAK (void)
/*****************************************************************************/
{
	FILE *fDAT;
	FILE *fBAK;
	int iData;

	fDAT = fopen (LEVELS_DAT, "rb");
	if (fDAT == NULL)
		{ printf ("[FAILED] Could not open %s: %s!\n",
			LEVELS_DAT, strerror (errno)); }
	fBAK = fopen (LEVELS_BAK, "wb");
	if (fBAK == NULL)
		{ printf ("[FAILED] Could not open %s: %s!\n",
			LEVELS_BAK, strerror (errno)); }

	while (1)
	{
		iData = fgetc (fDAT);
		if (iData == EOF) { break; }
			else { putc (iData, fBAK); }
	}

	fclose (fDAT);
	fclose (fBAK);
}
/*****************************************************************************/
void Help (int iAtScreen)
/*****************************************************************************/
{
	SDL_Event event;
	int iHelp;

	iHelp = 1;

	PlaySound ("wav/popup.wav");
	ShowHelp();
	while (iHelp == 1)
	{
		while (SDL_PollEvent (&event))
		{
			switch (event.type)
			{
				case SDL_KEYDOWN:
					switch (event.key.keysym.sym)
					{
						case SDLK_ESCAPE:
						case SDLK_KP_ENTER:
						case SDLK_RETURN:
						case SDLK_SPACE:
						case SDLK_o:
							iHelp = 0;
						default: break;
					}
					break;
				case SDL_MOUSEMOTION:
					SDL_GetMouseState (&iXPos, &iYPos);
					break;
				case SDL_MOUSEBUTTONDOWN:
					if (event.button.button == 1)
					{
						if (InArea (event, 590, 405, 674, 436) == 1) /*** OK ***/
						{
							if (iHelpOK != 1) { iHelpOK = 1; }
							ShowHelp();
						}
					}
					break;
				case SDL_MOUSEBUTTONUP:
					iHelpOK = 0;
					if (event.button.button == 1) /*** left mouse button ***/
					{
						if (InArea (event, 590, 405, 674, 436) == 1)
							{ iHelp = 0; }
					}
					ShowHelp(); break;
				case SDL_QUIT:
					Quit(); break;
			}
		}

		/*** prevent CPU eating ***/
		gamespeed = REFRESH;
		while ((SDL_GetTicks() - looptime) < gamespeed)
		{
			SDL_Delay (10);
		}
		looptime = SDL_GetTicks();
	}
	PlaySound ("wav/popup_close.wav");
	ShowScreen (iAtScreen);
}
/*****************************************************************************/
void ShowHelp (void)
/*****************************************************************************/
{
	ShowImage (-9, 10, screen, 31, 0, 0, 692, 455); /*** help ***/
	switch (iHelpOK)
	{
		case 0:
			ShowImage (-9, 3, screen, 43, 0, 0, 85, 32); break; /*** OK off ***/
		case 1:
			ShowImage (-9, 4, screen, 43, 0, 0, 85, 32); break; /*** OK on ***/
	}

	/*** refresh screen ***/
	if (SDL_Flip (screen) == -1)
		{ printf ("[FAILED] SDL_Flip error!\n"); exit (EXIT_ERROR); }
}
/*****************************************************************************/
void Zoom (void)
/*****************************************************************************/
{
	if (iScale == 1) { iScale = 2; } else { iScale = 1; }
	screen = SDL_SetVideoMode ((SCREEN_WIDTH) * iScale,
		(SCREEN_HEIGHT) * iScale, SCREEN_BPP,
		SDL_DOUBLEBUF|SDL_SWSURFACE);
	TTF_CloseFont (font1);
	TTF_CloseFont (font2);
	TTF_CloseFont (font3);
	LoadFonts();
}
/*****************************************************************************/
void Sprinkle (void)
/*****************************************************************************/
{
	int iTemp;
	int iTemp2;
	int iRandom;

	for (iTemp2 = 1; iTemp2 <= ROOMS; iTemp2++)
	{
		for (iTemp = 0; iTemp <= 29; iTemp++)
		{
			if (iCurType == 1) /*** dungeon ***/
			{
				if (((iThingA[iTemp2][iTemp] == 0) &&
					(iModifierA[iTemp2][iTemp] == 0)) ||
					((iThingA[iTemp2][iTemp] == 32) &&
					(iModifierA[iTemp2][iTemp] == 0)) ||
					((iThingA[iTemp2][iTemp] == 0) &&
					(iModifierA[iTemp2][iTemp] == 255)) ||
					((iThingA[iTemp2][iTemp] == 32) &&
					(iModifierA[iTemp2][iTemp] == 255)))
				{
					/*** 1-4 ***/
					iRandom = 1 + (int) (4.0 * RANDOM() / (RAND_MAX + 1.0));
					switch (iRandom)
					{
						case 1:
							iThingA[iTemp2][iTemp] = 0;
							iModifierA[iTemp2][iTemp] = 1;
							break;
						case 2:
							iThingA[iTemp2][iTemp] = 0;
							iModifierA[iTemp2][iTemp] = 2;
							break;
						default: break;
					}
				}
				if (((iThingA[iTemp2][iTemp] == 1) &&
				(iModifierA[iTemp2][iTemp] == 0)) ||
				((iThingA[iTemp2][iTemp] == 33) &&
				(iModifierA[iTemp2][iTemp] == 0)) ||
				((iThingA[iTemp2][iTemp] == 1) &&
				(iModifierA[iTemp2][iTemp] == 255)) ||
				((iThingA[iTemp2][iTemp] == 33) &&
				(iModifierA[iTemp2][iTemp] == 255)))
				{
					/*** 1-12 ***/
					iRandom = 1 + (int) (12.0 * RANDOM() / (RAND_MAX + 1.0));
					switch (iRandom)
					{
						case 1:
							iThingA[iTemp2][iTemp] = 1;
							iModifierA[iTemp2][iTemp] = 1;
							break;
						case 2:
							iThingA[iTemp2][iTemp] = 1;
							iModifierA[iTemp2][iTemp] = 2;
							break;
						case 3:
							iThingA[iTemp2][iTemp] = 14;
							iModifierA[iTemp2][iTemp] = 0;
							break;
						case 4:
							iThingA[iTemp2][iTemp] = 19;
							iModifierA[iTemp2][iTemp] = 0;
							break;
						case 5:
							iThingA[iTemp2][iTemp] = 21;
							iModifierA[iTemp2][iTemp] = 0;
							break;
						case 6:
							iThingA[iTemp2][iTemp] = 30;
							iModifierA[iTemp2][iTemp] = 0;
							break;
						default: break;
					}
				}
			} else { /*** palace ***/
				if (((iThingA[iTemp2][iTemp] == 0) &&
					(iModifierA[iTemp2][iTemp] == 1)) ||
					((iThingA[iTemp2][iTemp] == 32) &&
					(iModifierA[iTemp2][iTemp] == 1)))
				{
					/*** 1-2 ***/
					iRandom = 1 + (int) (2.0 * RANDOM() / (RAND_MAX + 1.0));
					if (iRandom == 1)
					{
						iThingA[iTemp2][iTemp] = 0;
						iModifierA[iTemp2][iTemp] = 2;
					}
				}
				if (((iThingA[iTemp2][iTemp] == 1) &&
				(iModifierA[iTemp2][iTemp] == 0)) ||
				((iThingA[iTemp2][iTemp] == 33) &&
				(iModifierA[iTemp2][iTemp] == 0)) ||
				((iThingA[iTemp2][iTemp] == 1) &&
				(iModifierA[iTemp2][iTemp] == 255)) ||
				((iThingA[iTemp2][iTemp] == 33) &&
				(iModifierA[iTemp2][iTemp] == 255)))
				{
					/*** 1-8 ***/
					iRandom = 1 + (int) (8.0 * RANDOM() / (RAND_MAX + 1.0));
					switch (iRandom)
					{
						case 1:
							iThingA[iTemp2][iTemp] = 1;
							iModifierA[iTemp2][iTemp] = 2;
							break;
						case 2:
							iThingA[iTemp2][iTemp] = 14;
							iModifierA[iTemp2][iTemp] = 0;
							break;
						case 3:
							iThingA[iTemp2][iTemp] = 19;
							iModifierA[iTemp2][iTemp] = 0;
							break;
						case 4:
							iThingA[iTemp2][iTemp] = 30;
							iModifierA[iTemp2][iTemp] = 0;
							break;
						default: break;
					}
				}
			}
		}
	}
}
/*****************************************************************************/
void LoadLevel (int iLevel)
/*****************************************************************************/
{
	switch (iLevel)
	{
		case 1: LoadPLV (IN_DIR "/level1.plv"); iCurType = 1; break;
		case 2: LoadPLV (IN_DIR "/level2.plv"); iCurType = 1; break;
		case 3: LoadPLV (IN_DIR "/level3.plv"); iCurType = 1; break;
		case 4: LoadPLV (IN_DIR "/level4.plv"); iCurType = 2; break;
		case 5: LoadPLV (IN_DIR "/level5.plv"); iCurType = 2; break;
		case 6: LoadPLV (IN_DIR "/level6.plv"); iCurType = 2; break;
		case 7: LoadPLV (IN_DIR "/level7.plv"); iCurType = 1; break;
		case 8: LoadPLV (IN_DIR "/level8.plv"); iCurType = 1; break;
		case 9: LoadPLV (IN_DIR "/level9.plv"); iCurType = 1; break;
		case 10: LoadPLV (IN_DIR "/level10.plv"); iCurType = 2; break;
		case 11: LoadPLV (IN_DIR "/level11.plv"); iCurType = 2; break;
		case 12: LoadPLV (IN_DIR "/level12a.plv"); iCurType = 1; break;
		case 13: LoadPLV (IN_DIR "/level12b.plv"); iCurType = 1; break;
		case 14: LoadPLV (IN_DIR "/princess.plv"); iCurType = 2; break;
		case 15: LoadPLV (IN_DIR "/potions.plv"); iCurType = 1; break;
		case 0: LoadPLV (IN_DIR "/demo.plv"); iCurType = 1; break;
	}
}
/*****************************************************************************/
void GetOptionValue (char *sArgv, char *sValue)
/*****************************************************************************/
{
	int iTemp;
	char sTemp[MAX_OPTION + 2];

	iTemp = strlen (sArgv) - 1;
	snprintf (sValue, MAX_OPTION, "%s", "");
	while (sArgv[iTemp] != '=')
	{
		snprintf (sTemp, MAX_OPTION, "%c%s", sArgv[iTemp], sValue);
		snprintf (sValue, MAX_OPTION, "%s", sTemp);
		iTemp--;
	}
}
/*****************************************************************************/
void FlipRoom (int iRoom, int iAxis)
/*****************************************************************************/
{
	int iThingTempA[30 + 2];
	int iModifierTempA[30 + 2];
	int iTemp;
	int iTileUse;
	int iKidPC;
	int iGuardPC;

	for (iTemp = 0; iTemp <= 29; iTemp++)
	{
		iThingTempA[iTemp] = iThingA[iRoom][iTemp];
		iModifierTempA[iTemp] = iModifierA[iRoom][iTemp];
	}

	if (iAxis == 1)
	{
		/*** tiles ***/
		for (iTemp = 0; iTemp <= 29; iTemp++)
		{
			if ((iTemp >= 0) && (iTemp <= 9)) { iTileUse = 9 - iTemp; }
			if ((iTemp >= 10) && (iTemp <= 19)) { iTileUse = 29 - iTemp; }
			if ((iTemp >= 20) && (iTemp <= 29)) { iTileUse = 49 - iTemp; }
			iThingA[iRoom][iTemp] = iThingTempA[iTileUse];
			iModifierA[iRoom][iTemp] = iModifierTempA[iTileUse];
		}

		/*** prince ***/
		if ((int)luKidRoom == iRoom)
		{
			if (luKidDir == 0) { luKidDir = 255; } else { luKidDir = 0; }
			iKidPC = (int)luKidPos;
			if ((iKidPC >= 1) && (iKidPC <= 10)) { luKidPos = 11 - iKidPC; }
			if ((iKidPC >= 11) && (iKidPC <= 20)) { luKidPos = 31 - iKidPC; }
			if ((iKidPC >= 21) && (iKidPC <= 30)) { luKidPos = 51 - iKidPC; }
		}

		/*** guard ***/
		if (sGuardLocations[iRoom - 1] != 30)
		{
			if (sGuardDirections[iRoom - 1] == 0)
				{ sGuardDirections[iRoom - 1] = 255; }
					else { sGuardDirections[iRoom - 1] = 0; }
			iGuardPC = sGuardLocations[iRoom - 1];
			if ((iGuardPC >= 0) && (iGuardPC <= 9))
				{ sGuardLocations[iRoom - 1] = 9 - iGuardPC; }
			if ((iGuardPC >= 10) && (iGuardPC <= 19))
				{ sGuardLocations[iRoom - 1] = 29 - iGuardPC; }
			if ((iGuardPC >= 20) && (iGuardPC <= 29))
				{ sGuardLocations[iRoom - 1] = 49 - iGuardPC; }
		}
	} else {
		/*** tiles ***/
		for (iTemp = 0; iTemp <= 29; iTemp++)
		{
			if ((iTemp >= 0) && (iTemp <= 9)) { iTileUse = iTemp + 20; }
			if ((iTemp >= 10) && (iTemp <= 19)) { iTileUse = iTemp; }
			if ((iTemp >= 20) && (iTemp <= 29)) { iTileUse = iTemp - 20; }
			iThingA[iRoom][iTemp] = iThingTempA[iTileUse];
			iModifierA[iRoom][iTemp] = iModifierTempA[iTileUse];
		}

		/*** prince ***/
		if ((int)luKidRoom == iRoom)
		{
			iKidPC = (int)luKidPos;
			if ((iKidPC >= 1) && (iKidPC <= 10)) { luKidPos = iKidPC + 20; }
			if ((iKidPC >= 21) && (iKidPC <= 30)) { luKidPos = iKidPC - 20; }
		}

		/*** guard ***/
		if (sGuardLocations[iRoom - 1] != 30)
		{
			iGuardPC = sGuardLocations[iRoom - 1];
			if ((iGuardPC >= 0) && (iGuardPC <= 9))
				{ sGuardLocations[iRoom - 1] = iGuardPC + 20; }
			if ((iGuardPC >= 20) && (iGuardPC <= 29))
				{ sGuardLocations[iRoom - 1] = iGuardPC - 20; }
		}
	}
}
/*****************************************************************************/
void CopyPaste (int iRoom, int iAction)
/*****************************************************************************/
{
	int iTemp;

	if (iAction == 1) /*** copy ***/
	{
		for (iTemp = 0; iTemp <= 29; iTemp++)
		{
			iThingACopyPaste[iTemp] = iThingA[iRoom][iTemp];
			iModifierACopyPaste[iTemp] = iModifierA[iRoom][iTemp];
		}
		iGuardACopyPaste[0] = sGuardLocations[iRoom - 1];
		iGuardACopyPaste[1] = sGuardDirections[iRoom - 1];
		iGuardACopyPaste[2] = sGuardSkills[iRoom - 1];
		iGuardACopyPaste[3] = sGuardColors[iRoom - 1];
		iCopied = 1;
	} else { /*** paste ***/
		for (iTemp = 0; iTemp <= 29; iTemp++)
		{
			if (iCopied == 1)
			{
				iThingA[iRoom][iTemp] = iThingACopyPaste[iTemp];
				iModifierA[iRoom][iTemp] = iModifierACopyPaste[iTemp];
			} else {
				iThingA[iRoom][iTemp] = 0;
				iModifierA[iRoom][iTemp] = 0;
			}
		}
		if (iCopied == 1)
		{
			sGuardLocations[iRoom - 1] = iGuardACopyPaste[0];
			sGuardDirections[iRoom - 1] = iGuardACopyPaste[1];
			sGuardSkills[iRoom - 1] = iGuardACopyPaste[2];
			sGuardColors[iRoom - 1] = iGuardACopyPaste[3];
		} else {
			sGuardLocations[iRoom - 1] = 30;
		}
	}
}
/*****************************************************************************/
void DateTime (char *sDateTime)
/*****************************************************************************/
{
	time_t clock;
	struct tm *loctim;

	clock = time (0);
	loctim = localtime (&clock);
	snprintf (sDateTime, MAX_DATETIME, "%04i-%02i-%02i %02i:%02i:%02i",
		loctim->tm_year + 1900, loctim->tm_mon + 1, loctim->tm_mday,
		loctim->tm_hour, loctim->tm_min, loctim->tm_sec);
}
/*****************************************************************************/
void SaveXML (char *sFileName)
/*****************************************************************************/
{
	int iFd;
	int iKidDir;
	int iTemp;
	int iTile;
	int iGuardLocation;
	int iGuardDirection;
	int iGuardSkill;
	int iGuardColors;
	char sToWrite[MAX_TOWRITE + 2];
	char sDateTime[MAX_DATETIME + 2];
	char sKey[100 + 2];
	char sValue[100 + 2];
	struct stat stStatus;

	/*** Create directory. ***/
	if (stat ("xml", &stStatus) == -1)
	{
		mkdir ("xml", 0700);
	}

	/*** Open file. ***/
	iFd = open (sFileName, O_WRONLY|O_TRUNC|O_CREAT|O_BINARY, 0600);
	if (iFd == -1)
	{
		printf ("[FAILED] Could not create XML file: %s!\n", strerror (errno));
		exit (EXIT_ERROR);
	}

	snprintf (sToWrite, MAX_TOWRITE, "%s",
		"<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n");
	write (iFd, sToWrite, strlen (sToWrite));
	DateTime (sDateTime);
	snprintf (sToWrite, MAX_TOWRITE,
		"<!-- PoP1 level, exported %s with %s %s. -->\n",
		sDateTime, EDITOR_NAME, EDITOR_VERSION);
	write (iFd, sToWrite, strlen (sToWrite));
	snprintf (sToWrite, MAX_TOWRITE, "<level number=\"%lu\">\n",
		luLevelNr);
	write (iFd, sToWrite, strlen (sToWrite));

	/*** rooms ***/
	write (iFd, "\t<rooms>\n", 9);
	for (iTemp = 1; iTemp <= ROOMS; iTemp++)
	{
		snprintf (sToWrite, MAX_TOWRITE,
			"\t\t<room number=\"%i\">\n",
			iTemp);
		write (iFd, sToWrite, strlen (sToWrite));

		/*** tiles ***/
		for (iTile = 0; iTile <= 29; iTile++)
		{
			snprintf (sToWrite, MAX_TOWRITE,
				"\t\t\t<tile element=\"%i\" modifier=\"%i\">\n",
				iThingA[iTemp][iTile], iModifierA[iTemp][iTile]);
			write (iFd, sToWrite, strlen (sToWrite));
		}

		/*** guard ***/
		iGuardLocation = sGuardLocations[iTemp - 1] + 1;
		if (iGuardLocation == 31) { iGuardLocation = 0; }
		if (sGuardDirections[iTemp - 1] == 255) { iGuardDirection = 1; }
		else if (sGuardDirections[iTemp - 1] == 0) { iGuardDirection = 2; }
		iGuardSkill = sGuardSkills[iTemp - 1];
		iGuardColors = sGuardColors[iTemp - 1];
		snprintf (sToWrite, MAX_TOWRITE,
			"\t\t\t<guard location=\"%i\" direction=\"%i\""
			" skill=\"%i\" colors=\"%i\">\n",
			iGuardLocation, iGuardDirection, iGuardSkill, iGuardColors);
		write (iFd, sToWrite, strlen (sToWrite));

		/*** links ***/
		snprintf (sToWrite, MAX_TOWRITE,
			"\t\t\t<links left=\"%i\" right=\"%i\" up=\"%i\" down=\"%i\">\n",
			iRoomConnections[iTemp][1], iRoomConnections[iTemp][2],
			iRoomConnections[iTemp][3], iRoomConnections[iTemp][4]);
		write (iFd, sToWrite, strlen (sToWrite));

		write (iFd, "\t\t</room>\n", 10);
	}
	write (iFd, "\t</rooms>\n", 10);

	/*** events ***/
	write (iFd, "\t<events>\n", 10);
	for (iTemp = 0; iTemp < 256; iTemp++)
	{
		snprintf (sToWrite, MAX_TOWRITE,
			"\t\t<event number=\"%i\" room=\"%i\" location=\"%i\" next=\"%i\">\n",
			iTemp + 1, EventInfo (iTemp, 1),
			EventInfo (iTemp, 2), EventInfo (iTemp, 3));
		write (iFd, sToWrite, strlen (sToWrite));
	}
	write (iFd, "\t</events>\n", 11);

	/*** prince ***/
	switch ((int)luKidDir)
	{
		case 0: iKidDir = 1; break;
		case 255: iKidDir = 2; break;
	}
	if (((int)luLevelNr == 1) || ((int)luLevelNr == 13)) /*** 4 of 4 ***/
	{
		if (luKidDir == 0) { iKidDir = 2; }
		else if (luKidDir == 255) { iKidDir = 1; }
	}
	snprintf (sToWrite, MAX_TOWRITE,
		"\t<prince room=\"%lu\" location=\"%lu\" direction=\"%i\">\n",
		luKidRoom, luKidPos, iKidDir);
	write (iFd, sToWrite, strlen (sToWrite));

	/*** userdata ***/
	snprintf (sToWrite, MAX_TOWRITE,
		"\t<userdata fields=\"%lu\">\n", luNumberOfFields - 1);
	write (iFd, sToWrite, strlen (sToWrite));
	for (iTemp = 1; iTemp < iInformationNr; iTemp++)
	{
		snprintf (sKey, 200, "%s", sInformation[iTemp][0]);
		if (strcmp (sInformation[iTemp][0], "Editor Name") == 0)
			{ snprintf (sValue, MAX_TOWRITE, "%s", EDITOR_NAME); }
		else if (strcmp (sInformation[iTemp][0], "Level Author") == 0)
			{ snprintf (sValue, MAX_TOWRITE, "%s", sAuthor); }
		else if (strcmp (sInformation[iTemp][0], "Editor Version") == 0)
			{ snprintf (sValue, MAX_TOWRITE, "%s", EDITOR_VERSION); }
		else if (strcmp (sInformation[iTemp][0], "Time Last Modified") == 0)
			{ snprintf (sValue, MAX_TOWRITE, "%s", sDateTime); }
		else
			{ snprintf (sValue, MAX_TOWRITE, "%s", sInformation[iTemp][1]); }
		snprintf (sToWrite, MAX_TOWRITE,
			"\t\t<field key=\"%s\" value=\"%s\">\n",
			sKey, sValue);
		write (iFd, sToWrite, strlen (sToWrite));
	}
	write (iFd, "\t</userdata>\n", 13);

	write (iFd, "</level>\n", 9);

	close (iFd);
}
/*****************************************************************************/
